package jargon;
/**
 * The Asm operator produces a Jlcm object containing the multigroup collision
 * probability matrices or multigroup information required by the iterative
 * flux solution.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Asm extends Operator {
  static {
    System.out.println("Asm class loaded");
  }
  static String fr = "calcul d'un objet systeme";
  static String en = "calculation of a system object";
  private static final long serialVersionUID = 1L;
  private String type;
  /** set to <tt>true</tt> to specify that only the reduced collision
   * probability matrix is to be computed. By default, the scattering
   * modified collision probability matrix is also computed.
   */
  public boolean skip;
  /** type of <i>a posteriori</i> normalization for the scattering-reduced
   * collision probability matrix. This variable is chosen among the
   * following values:
   * <dl>
   * <dt> <tt>"NORM"</tt> <dd> the scattering-reduced collision probability
   * matrix is to be normalized in such a way as to eliminate all neutron
   * loss (even if the region under consideration has external albedo
   * boundary conditions which should result in neutron loss). When used
   * with a void boundary  condition (zero reentrant current), this option
   * is equivalent to imposing <i>a posteriori</i> a uniform reentrant current.
   * <dt> <tt>"ALBS"</tt> <dd> a consistent Selengut normalization of the
   * scattering-reduced collision probability matrix is to be used both for
   * the Flux operator and in the equivalence calculation. This option results
   * in storing the scattering-reduced escape probabilities in the record
   * named <tt>"DRAGON-WIS"</tt> on the embedded LCM structure. For all the
   * cases where this option is used, it is necessary to define a geometry
   * with <tt>VOID</tt> external boundary conditions.
   * </dl>
   */
  public String norm;
  /** type of numerical technique used to normalize the collision, leakage
   * and escape probability matrices. These probabilities are normalized in
   * such a way as to satisfy explicitly the neutron conservation laws.
   * This option compensates for the errors which will arise in the
   * numerical evaluation of these probabilities and may result in
   * non-conservative collision probability matrices. This variable is
   * chosen among the following values:
   * <dl>
   * <dt> <tt>"NONE"</tt> <dd> the probability matrices are not to be
   * renormalized. 
   * <dt> <tt>"DIAG"</tt> <dd> only the diagonal element of the
   * probability matrices will be modified in order to insure the validity
   * of the conservation laws.
   * <dt> <tt>"GELB"</tt> <dd> the Gelbard algorithm will be used to
   * normalize the collision probability matrices.
   * <dt> <tt>"HELI"</tt> <dd> the Helios algorithm will be used to
   * normalize the collision probability matrices. This is the default
   * option.
   * <dt> <tt>"NONL"</tt> <dd> a non-linear multiplicative algorithm
   * will be used to normalize the collision probability matrices.
   * </dl> 
   */
  public String pnor;
  /** set to <tt>true</tt> to compute the P1-scattering reduced collision
   * probability or system matrices required by the ECCO isotropic
   * streaming model. By default, this information is not calculated.
   */
  public boolean ecco;
  /** 
   * Use this constructor to create an Asm object of specific type.
   * @param name user-defined name of the Asm object
   * @param type type of the information written in the object. This
   * variable is chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"PIJ"</tt> <dd> the standard scattering-reduced collision
   * probabilities must be computed. This is the default option.
   * <dt> <tt>"PIJK"</tt> <dd> both the directional and standard
   * scattering-reduced collision probabilities must be computed. Moreover,
   * the additional directional collision probability matrix can only be
   * used if <tt>"HETE"</tt> option is activated. Finally, the <tt>"PIJK"</tt>
   * option is only available with methods capable of computing collision
   * probabilities over the complete domain.
   * <dt> <tt>"ARM"</tt> <dd> a flux calculation is carried out without
   * building the full collision probability matrices.
   * </dl></ul>
   */
  public Asm(String name, String type) {
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
    this.type = type;
  }
  /** perform the assembly of the multigroup system matrices, as
   * specified by the instance variables of this object and using
   * cross sections recovered from a Macrolib object.
   * @param a Macrolib object containing the cross sections.
   * @param b Tracking object containing energy-independent information.
   */
  public void exec(Macrolib a, Tracking b) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    if (b.fileObj != null) super.inputJlcm(b.fileObj);
    super.exec();
  }
  
  /** perform the assembly of the multigroup system matrices, as
   * specified by the instance variables of this object and using
   * cross sections recovered from a Microlib object.
   * @param a Microlib object containing the cross sections.
   * @param b Tracking object containing energy-independent information.
   */
  public void exec(Microlib a, Tracking b) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    if (b.fileObj != null) super.inputJlcm(b.fileObj);
    super.exec();
  }
  /** perform the assembly of the multigroup system matrices, as
   * specified by the instance variables of this object and using
   * cross sections recovered from an Autop object.
   * @param a Autop object containing the cross sections.
   * @param b Tracking object containing energy-independent information.
   */
  public void exec(Autop a, Tracking b) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    if (b.fileObj != null) super.inputJlcm(b.fileObj);
    super.exec();
  }
  protected void makeText() {
    this.operator="ASM: ";
    super.write("EDIT", super.edit);
    super.write(this.type);
    if (this.skip)  super.write("\nSKIP");
    if (this.norm != null) {
      super.write("\n"); super.write(this.norm);
    }
    if (this.pnor != null) {
      super.write("\nPNOR"); super.write(this.pnor);
    }
    if (this.ecco)  super.write("\nECCO");
    super.write(";");
  }
}
