package jargon;

import java.io.*;
/**
 * The Autop operator is used to compute the self-shielding effects on the
 * microscopic cross sections.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Autop extends Operator {
  static {
    System.out.println("Autop class loaded");
  }
  static String fr = "operateur d'auroprotection des resonnances";
  static String en = "resonance self-shielding operator";
  private static final long serialVersionUID = 31L;
  protected Composition[] mixs;
  private SimpleAutop[] simpleSS;
  private String type; //into=('Shiba','Ribox')
  /** first group index considered during the self-shielding process. By
   * default, <tt>grmin</tt> is set to the first group index containing
   * self-shielding data in the library.
   */
  public int grmin;
  /** last group index considered during the self-shielding process. By
   * default, <tt>grmax</tt> is set to the last group index containing
   * self-shielding data in the library.
   */
  public int grmax;
  /** the number of outer iterations during the self-shielding process.
   * The default is <tt>passes = 2</tt> if the embedded LCM object is created
   * and <tt>passes = 1</tt> if the embedded LCM object is updated. This
   * variable is not used with the generalized Stamm'ler method (<tt>"Shiba"</tt>
   * method).
   */
  public int passes;
  /** the maximum number of outer iterations during the self-shielding process.
   * The default is <tt>iterations = 20</tt>. This variable is used only with
   * the generalized Stamm'ler method (<tt>"Shiba"</tt> method).
   */
  public int iterations;
  /** convergence criterion for the self-shielding iteration. The default is
   * <tt>eps = 1.0e-4f</tt>. This variable is used only with the generalized
   * Stamm'ler method (<tt>"Shiba"</tt> method).
   */
  public float eps;
  /** set to <tt>true</tt> to activate the Goldstein-Cohen approximation in
   * cases where Goldstein-Cohen parameters are stored on the internal library.
   * These parameters may not be available with some libraries (e.g., <tt>APLIB1</tt>,
   * <tt>APLIB2</tt> or MATXS-type libraries). The Goldstein-Cohen parameters
   * can always be imposed using the <tt>IRSET</tt> variable of object Microlib.
   * The default option is <tt>GoldsteinCohen = true</tt>.
   */
  public boolean GoldsteinCohen = true;
  /** set to <tt>true</tt> to activate the SPH equivalence or Livolant-Jeanpierre
   * scheme which modifies the self-shielded averaged neutron fluxes in
   * heterogeneous geometries. The default option is <tt>sph = true</tt>.
   */
  public boolean sph = true;
  /** set to <tt>true</tt> to activate the transport correction option for
   * self-shielding calculations. The default option is <tt>traCor = true</tt>.
   */
  public boolean traCor = true;
  /** type of flux solution technique used within the self-shielding
   * model. This variable is chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"PIJ"</tt> <dd> use collision probabilities (default method).
   * <dt> <tt>"ARM"</tt> <dd> use a response matrix or an iterative approach.
   * </dl></ul>
   */
  public String solution = "PIJ";
  /** 
   * Use this constructor to create a new Autop object.
   * @param name user-defined name of the Autop object
   * @param type type of self-shielding model. This variable is
   * chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"Shiba"</tt> <dd> use the generalized Stamm'ler model.
   * <dt> <tt>"Ribox"</tt> <dd> use a subgroup method with physical
   * probability tables at high energies and slowing-down correlated
   * probability tables at low energies. A mutual resonancu shielding
   * model is available.
   * </dl></ul>
   */
  public Autop(String name, String type) {
    if ((!type.equals("Shiba")) & (!type.equals("Ribox"))) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
    this.type = type;
  }
  private class SimpleAutop implements Serializable {
    private static final long serialVersionUID = 32L;
    private String suffix;
    private Isotope isotope;
    private Composition[] media = null;
    private SimpleAutop(String suffix, Isotope isotope, Composition[] media) {
      this.suffix = suffix;
      this.isotope = isotope;
      this.media = media;
    }
  }
  private void addSimpleSS() {
    if (this.simpleSS == null) {
      this.simpleSS = new SimpleAutop[1];
    } else {
      SimpleAutop[] work = this.simpleSS;
      this.simpleSS = new SimpleAutop[work.length+1];
      for (int i=0;i<work.length;i++) this.simpleSS[i] = work[i];
    }
  }
  /** print the operator data in a form compatible with REDGET. Use this call
   * if method <tt>setSimpleSS</tt> is used; otherwise, use the inherited call.
   * @param a Microlib object where self-shielding related information
   * is recovered.
   */
  public void text(Microlib a) throws JargonException {
    this.mixs = a.mixs;
    super.text();
  }
  /** create the embedded LCM structure and perform a resonance
   * self-shielding calculation.
   * @param a Microlib object where self-shielding related information
   * is recovered (including probability tables).
   * @param b tracking object where flux-related numerotation is recovered.
   */
  public void exec(Microlib a, Tracking b) throws JargonException {
    this.mixs = a.mixs;
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    if (b.fileObj != null) super.inputJlcm(b.fileObj);
    super.exec();
  }
  /** perform a resonance self-shielding calculation and update the
   * embedded LCM structure.
   * @param a Autop object containing previously self-shielded information.
   * @param b Microlib object where self-shielding related information
   * is recovered (including probability tables).
   * @param c tracking object where flux-related numerotation is recovered.
   */
  public void exec(Autop a, Microlib b, Tracking c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    if (a.type != "Shiba") super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    if (c.fileObj != null) super.inputJlcm(c.fileObj);
    super.exec();
  }
  /** activate the simplified self-shielding approximation in which a
   * single self-shielded isotope is shared by many resonant mixtures.
   * @param suffix user-defined 4-character suffix for the isotope
   * names that will be self-shielded together in a group
   * @param isotope self-shielded isotope in this group
   * @param media array of medium that will share the same self-shielded
   * isotope
   */
  public void setSimpleSS(String suffix, Isotope isotope, Composition[] media) {
    this.addSimpleSS();
    this.simpleSS[this.simpleSS.length-1] = new SimpleAutop(suffix, isotope, media);
  }
  /** activate the simplified self-shielding approximation in which a
   * single self-shielded isotope is shared by many resonant mixtures.
   * @param suffix user-defined 4-character suffix for the isotope
   * names that will be self-shielded together in a group
   * @param isotope self-shielded isotope in this group. All media of
   * the domain will share the same self-shielded isotope.
   */
  public void setSimpleSS(String suffix, Isotope isotope) {
    this.addSimpleSS();
    this.simpleSS[this.simpleSS.length-1] = new SimpleAutop(suffix, isotope, null);
  }
  
  protected void makeText() {
    super.write("EDIT", super.edit);
    if (this.grmin != 0) super.write("GRMIN", this.grmin);
    if (this.grmax != 0) super.write("GRMAX", this.grmax);
    if (this.type.equals("Shiba")) {
      this.operator="SHI: ";
      if (this.iterations != 0) super.write("MXIT", this.iterations);
      if (this.eps != 0.0f) super.write("EPS", this.eps);
      if (this.sph) super.write("LJ");
      if (!this.sph) super.write("NOLJ");
      if (this.GoldsteinCohen) super.write("GC");
      if (!this.GoldsteinCohen) super.write("NOGC");
    } else if (this.type.equals("Ribox")) {
      this.operator="USS: ";
      if (this.passes != 0) super.write("PASS", this.passes);
      if (!this.sph) super.write("NOSP");
      super.write(this.solution);
    }
    if (this.traCor) super.write("TRAN");
    if (!this.traCor) super.write("NOTR");
    if (this.simpleSS != null) {
      // simplified self-shielding data
      super.write("CALC");
      for (int i=0; i<this.simpleSS.length; i++) {
        super.write("REGI", this.simpleSS[i].suffix);
        super.write(this.simpleSS[i].isotope.name);
        if (this.simpleSS[i].media == null) {
          super.write("ALL");
        } else {
          for (int j=0; j<this.simpleSS[i].media.length; j++) {
            Composition medium = this.simpleSS[i].media[j];
            int imix = 0;
            for (int k=0; k<this.mixs.length; k++) {
              if (medium.name.equals(this.mixs[k].name)) imix = k+1;
            }
            if (imix == 0) {
              throw new RuntimeException("unable to find medium " + medium.name);
            }
            super.write(imix);
          }
        }
      }
      super.write("ENDC");
    }
    super.write(";");
  }
}
