package jargon;

/**
 * The Bivac tracking operator produces a Jlcm structure containing the
 * energy-independent numerotation information required to solve the
 * 2D diffusion equation with Bivac.
 * <P>
 * A Bivac object may be used within the Edition operator to perform SPH
 * equivalence calculations.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Bivac extends Tracking {
  static {
    System.out.println("Bivac class loaded");
  }
  static String fr = "definition d'un tracking bivac";
  static String en = "definition of a bivac tracking";
  private static final long serialVersionUID = 40L;
  private String bivacType;
  private int[] order;
  private int isplh = -1;
  /** 
   * Use this constructor to create a new Bivac object in hexagonal geometry.
   * @param name user-defined name of the Bivac object
   * @param bivacType type of finite element approach. This variable is chosen
   * among the following values:
   * <ul><dl>
   * <dt> <tt>"PRIM"</tt> <dd> primal finite element (classical) discretization.
   * <dt> <tt>"DUAL"</tt> <dd> mixed-dual finite element discretization.
   * </dl></ul>
   * @param order two-component array containing the discretization type.
   * <tt>order[0]</tt> is the order of the finite element representation.
   * Its value is chosen among the following values:
   * <ul><dl>
   * <dt> 1 <dd> linear polynomials or finite differences (default value)
   * <dt> 2 <dd> parabolic polynomials
   * <dt> 3 <dd> cubic polynomials
   * <dt> 4 <dd> quartic polynomials
   * </dl></ul>
   * Discretization of an hexagonal geometry is only available with
   * <tt>ielem = 1</tt>.
   * <tt>order[1]</tt> is the type of quadrature used to integrate the mass
   * matrices. Its value is chosen among the following values:
   * <ul><dl>
   * <dt> 1 <dd> analytical integration
   * <dt> 2 <dd> Gauss-Lobatto quadrature (default value)
   * <dt> 3 <dd> Gauss-Legendre quadrature
   * </dl></ul>
   * The analytical integration corresponds to classical finite elements;
   * the Gauss-Lobatto quadrature corresponds to a variational or nodal
   * type collocation and the Gauss-Legendre quadrature corresponds to
   * superconvergent finite elements.
   * @param isplh type of hexagonal mesh-splitting. This data is given only
   * if the geometry is 2D hexagonal. The values permitted are:
   * <ul><dl>
   * <dt> 1 <dd> full hexagons
   * <dt> 2 <dd> splitting each hexagon into 6 triangles
   * <dt> 3 <dd> splitting each hexagon into 24 triangles
   * <dt> 5 <dd> splitting each hexagon into 96 triangles
   * <dt> 9 <dd> splitting each hexagon into 384 triangles
   * <dt> 17 <dd> splitting each hexagon into 1536 triangles
   * </dl></ul>
   */
  public Bivac(String name, String bivacType, int[] order, int isplh) {
    if ((!bivacType.equals("PRIM")) & (!bivacType.equals("DUAL"))) {
      throw new RuntimeException("invalid type (" + bivacType + ")");
    }
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
    this.bivacType = bivacType;
    this.order = order;
    this.isplh = isplh;
  }
  /**
   * Use this constructor to create a new Bivac object in Cartesian geometry.
   * @param name user-defined name of the Bivac object
   * @param bivacType type of finite element approach. This variable is chosen
   * among the following values:
   * <ul><dl>
   * <dt> <tt>"PRIM"</tt> <dd> primal finite element (classical) discretization.
   * <dt> <tt>"DUAL"</tt> <dd> mixed-dual finite element discretization.
   * </dl></ul>
   * @param order two-component array containing the discretization type.
   * <tt>order[0]</tt> is the order of the finite element representation.
   * Its value is chosen among the following values:
   * <ul><dl>
   * <dt> 1 <dd> linear polynomials or finite differences (default value)
   * <dt> 2 <dd> parabolic polynomials
   * <dt> 3 <dd> cubic polynomials
   * <dt> 4 <dd> quartic polynomials
   * </dl></ul>
   * Discretization of an hexagonal geometry is only available with
   * <tt>order[0] = 1</tt>.
   * <tt>order[1]</tt> is the type of quadrature used to integrate the mass
   * matrices. Its value is chosen among the following values:
   * <ul><dl>
   * <dt> 1 <dd> analytical integration
   * <dt> 2 <dd> Gauss-Lobatto quadrature (default value)
   * <dt> 3 <dd> Gauss-Legendre quadrature
   * </dl></ul>
   * The analytical integration corresponds to classical finite elements;
   * the Gauss-Lobatto quadrature corresponds to a variational or nodal
   * type collocation and the Gauss-Legendre quadrature corresponds to
   * superconvergent finite elements.
   */
  public Bivac(String name, String bivacType, int[] order) {
    this(name, bivacType, order, -1);
  }
  /** load the embedded lcmObj structure corresponding to this Bivac tracking.
   * @param a 1D or 2D Cartesian or hexagonal geometry to analyse.
   */
  public void exec(Gigogne a) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.exec();
  }
  protected void makeText() {
    this.operator="BIVACT: ";
    super.write("EDIT", super.edit);
    if (super.title != null) {
      super.write("\nTITLE '"); super.write(this.title); super.write("'");
    }
    if (super.maxr != 0) super.write("\nMAXR", this.maxr);
    super.write("\n", this.bivacType);
    super.write(this.order);
    if (this.isplh != -1) super.write(this.isplh);
    super.write(";");
  }
}
