package jargon;

import java.io.*;
import java.util.Vector;
/**
 * Call a parametrized Cle2000 procedure. A CLE-2000 procedure is implemented
 * as a 80-column ascii file. The name of that file must have a <tt>".c2m"</tt>
 * suffix. A Lifo object is used to manage input/output parameters for this
 * CLE-2000 procedure.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2010)
 *
 * @author Alain Hebert
 */
public class Cle2000 {
  static {
    System.out.println("Cle2000 class loaded");
    System.loadLibrary("JargonNat");
    System.out.println("\n-----------------------------------------");
    System.out.println("| DRAGON: code neutronique deterministe |");
    System.out.println("|             Version Java              |");
    System.out.println("-----------------------------------------");
    System.out.flush();
  }
  static private native synchronized void execNat(String procName, int iprint, Lifo jstack);
  private String procName;
  private int edit;
  private Lifo stack;

  /** attach a lifo stack to the Cle2000 object
   * @param myLifo Lifo stack containing input/output parameters
   */
  public void setLifo(Lifo myLifo) {
    this.stack = myLifo;
  }
  /** use this constructor to create a new Cle2000 object
   * @param sname string containing the name of the ascii file containing the CLE-2000
   * procedure (without the <tt>".c2m"</tt> suffix)
   */
  public Cle2000(String sname) {
    this.edit = 0;
    this.procName = sname;
    this.stack = null;
  }
  /** use this constructor to create a new Cle2000 object
   * @param sname string containing the name of the ascii file containing the CLE-2000
   * procedure (without the <tt>".c2m"</tt> suffix)
   * @param edit user-defined edition index. Increasing value of <tt>edit</tt> will
   * cause increasing amount of listing information. Set <tt>edit</tt> to zero to avoid
   * listing information.
   */
  public Cle2000(String sname, int edit) {
    this.edit = edit;
    this.procName = sname;
    this.stack = null;
  }
  /** use this constructor to create a new Cle2000 object with an embedded Lifo stack
   * @param sname string containing the name of the ascii file containing the CLE-2000
   * procedure (without the <tt>".c2m"</tt> suffix)
   * @param jstack Lifo stack to include in Cle2000 object
   */
  public Cle2000(String sname, Lifo jstack) {
    this.edit = 0;
    this.procName = sname;
    this.stack = jstack;
  }
  /** use this constructor to create a new Cle2000 object with an embedded Lifo stack
   * @param sname string containing the name of the ascii file containing the CLE-2000
   * procedure (without the <tt>".c2m"</tt> suffix)
   * @param edit user-defined edition index. Increasing value of <tt>edit</tt> will
   * cause increasing amount of listing information. Set <tt>edit</tt> to zero to avoid
   * listing information.
   * @param jstack Lifo stack to include in Cle2000 object
   */
  public Cle2000(String sname, int edit, Lifo jstack) {
    this.edit = edit;
    this.procName = sname;
    this.stack = jstack;
  }
  /** call the native CLE-2000 procedure using JNI
   */
  public void exec() throws JargonException {
    try {
      execNat(this.procName, this.edit, this.stack);
    } catch(Exception ex) {
      System.out.println("Jargon exception encountered in Cle2000:" + ex);
      throw new JargonException(ex.toString());
    }
    Runtime.getRuntime().gc();
  }
}
