package jargon;

import java.util.Enumeration;
import java.util.Hashtable;
import java.io.*;
/**
 * The Compo operator is used to create or feed a <i>multi-parameter
 * reactor database</i> containing the information required by the
 * few-group diffusion (or simplified Pn) calculation.
 * <P>
 * This component of the lattice code is dedicated to the constitution of the
 * reactor database intended to store <i>all</i> the nuclear data, produced in
 * the lattice code, that is useful in reactor calculations including fuel
 * management and space-time kinetics. Multigroup lattice calculations are
 * too expensive to be executed dynamically from the driver of the global
 * reactor calculation. A more feasible approach is to create a reactor database
 * where a finite number of lattice calculation results are tabulated against
 * selected <i>global</i> and/or <i>local parameters</i> chosen so as to
 * represent expected operating conditions of the reactor.
 * <P>
 * The Compo operator is used to create and construct a Compo object.
 * This object is generally <i>persistent</i> and used to collect information gathered
 * from many Dragon <i>elementary calculations</i> performed under various conditions.
 * <P>
 * For each elementary calculation, the results are recovered from the output of the
 * Edition operator and stored in a list of <i>homogenized mixture</i>
 * directories. The Edition operator is responsible for performing condensation
 * in energy, homogenization in space of the microscopic cross sections and constitution
 * of <i>macroscopic sets</i> for collecting together many isotopes. All the elementary
 * calculations gathered in a single Compo object are characterized by the same
 * number of <i>homogenized mixtures</i> and by a specific output energy-group structure.
 * <P>
 * Each elementary calculation is characterized by a tuple of <i>global</i> and/or
 * <i>local parameters</i>. Global parameters are characteristics of the complete
 * lattice, while local parameters are characteristics of each homogenized mixture.
 * These parameters are of different types, depending on the nature of the
 * study under consideration: type of assembly, power, temperature in a mixture,
 * concentration of an isotope, time, burnup or exposure rate in a depletion calculation,
 * etc. Each step of a depletion calculation represents an elementary calculation.
 * The Compo object is often presented as a <i>multi-parameter reactor database</i>.
 * <P>
 * The Compo object is organized as shown in Figure 1. The root of the
 * object contains table-of-content information for global and local parameters
 * and two lists of directories. Each component of the first list (<tt>'MIXTURES'</tt>)
 * contains the directory <tt>'TREE'</tt> (the parameter tree) and the list
 * (<tt>'CALCULATIONS'</tt>) made of Microlib objects. Each component of the
 * second list (<tt>'GEOMETRIES'</tt>) contains the homogenized geometry of an
 * elementary calculation.
 * <P><BR><img src="../../src/figures/compo.gif">
 * <BR>Figure 1. <b>Organization of a Compo object</b>
 * <P>
 * The localization of an elementary calculation is done using a tuple of global
 * and/or local parameters. The elementary calculation indices are stored in a tree
 * with the number of levels equal to the number of global and local parameters.
 * An example of a tree with three parameters is shown in Figure 2. Each node
 * of this tree is associated with the index of the corresponding parameter and with the
 * reference to the daughter nodes if they exist. The number of leaves is equal to the number
 * of nodes for the last (third) parameter and is equal to the number of elementary
 * calculations stored in the Compo object. The index of each elementary calculation is
 * therefore an attribute of each leaf.
 * <P><BR><img src="../../src/figures/tree.gif">
 * <BR><BR>Figure 2. <b>Parameter tree in a Compo object</b>
 * <P>
 * In each homogenized mixture component, the Compo operator recover cross sections
 * for a number of <i>particularized isotopes</i> and of a single <i>macroscopic
 * set</i>, a collection of the remaining isotopic cross sections weighted by isotopic
 * number densities. Other information is also recovered: multigroup neutron
 * fluxes, isotopic number densities, fission spectrum, delayed neutron data, etc.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Compo extends Operator {
  static {
    System.out.println("Compo class loaded");
  }
  static String fr = "operateur pour la constitution d'un objet compo";
  static String en = "operator for the constitution of a compo object";
  private static final long serialVersionUID = 50L;
  /** maximum number of elementary calculations to be stored in the Compo
   * object.
   * <BR><tt>maxcal</tt>=10 by default. This maximum size is automatically
   * increased when the number of elementary calculations exceeds the current
   * value of <tt>maxcal</tt>.
   */
  public int maxcal = 0;
  /** user-defined comment
   */
  public String comment = null;
  /** index of the elementary calculation associated to the father node in
   * the parameter tree
   */
  public int orig = 0;
  private Parameter[] para = null;
  private Parameter[] loca = null;
  private Hashtable<String, Object> parkey = null;
  private Set set = null;
  
  private class Set implements Serializable {
    private static final long serialVersionUID = 51L;
    private float xtr;
    private String unit;
  }
  /** set the time flag
   * @param xtr numerical value of the time flag
   * @param unit unit of <tt>xtr</tt>. This value is chosen among
   * <tt>"S"</tt>, <tt>"DAY"</tt> or <tt>"YEAR"</tt>.
   */
  public void setSet(float xtr, String unit) {
    this.set = new Set();
    this.set.xtr = xtr;
    this.set.unit = unit;
  }
  /** unset the time flag
   */
  public void unSet() {
    this.set = null;
  }
  private class Parameter implements Serializable {
    private static final long serialVersionUID = 52L;
    private String parkey;
    private String type;
    private Isotope isotope = null;
    private Operator microlib = null;
    private int mix = 0;
    private String value = null;
    private Parameter(String parkey, String type) {
      this.parkey = parkey;
      this.type = type;
    }
  }
  /** 
   * Use this constructor to create a new Compo object.
   * @param name user-defined name of the Compo object
   */
  public Compo(String name) {
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
  }
  private void addGlobal() {
    if (this.para == null) {
      this.para = new Parameter[1];
    } else {
      Parameter[] work = this.para;
      this.para = new Parameter[work.length+1];
      for (int i=0;i<work.length;i++) this.para[i] = work[i];
    }
  }
  private void addLocal() {
    if (this.loca == null) {
      this.loca = new Parameter[1];
    } else {
      Parameter[] work = this.loca;
      this.loca = new Parameter[work.length+1];
      for (int i=0;i<work.length;i++) this.loca[i] = work[i];
    }
  }
  /** set a new global parameter.
   * @param parkey user-defined 4-character key-word associated to the
   * global parameter
   * @param type type of the global parameter. This variable is
   * chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"IRRA"</tt> <dd> burnup (in MWday/Tonne)
   * <dt> <tt>"FLUB"</tt> <dd> exposure rate (in n/Kb) The exposure rate
   * is recovered from the Evo object referenced in the <tt>exec()</tt>
   * method.
   * <dt> <tt>"POWR"</tt> <dd> power
   * <dt> <tt>"MASL"</tt> <dd> mass density of heavy isotopes
   * <dt> <tt>"FLUX"</tt> <dd> volume-averaged, energy-integrated flux
   * <dt> <tt>"TIME"</tt> <dd> time (in second)
   * </dl></ul>
   */
  public void setGlobal(String parkey, String type) {
    if ((!type.equals("IRRA"))&(!type.equals("FLUB"))&
       (!type.equals("POWR"))&(!type.equals("MASL"))&
       (!type.equals("FLUX"))&(!type.equals("TIME"))) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.addGlobal();
    this.para[this.para.length-1] = new Parameter(parkey, type);
  }
  /** set a new user-defined global parameter.
   * @param parkey user-defined 4-character key-word associated to the
   * global parameter
   * @param type must be equal to <tt>"VALU"</tt>
   * @param value type of the user-defined global parameter. This
   * variable is chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"REAL"</tt> <dd> float-value (real number)
   * <dt> <tt>"CHAR"</tt> <dd> String-value (12-character string)
   * <dt> <tt>"INTE"</tt> <dd> integer-value (integer number)
   * </dl></ul>
   */
  public void setGlobal(String parkey, String type, String value)  {
    if (!type.equals("VALU")) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    if ((!value.equals("REAL"))&(!value.equals("CHAR"))&
       (!value.equals("INTE"))) {
      throw new RuntimeException("invalid value type (" + value+ ")");
    }
    this.addGlobal();
    this.para[this.para.length-1] = new Parameter(parkey, type);
    this.para[this.para.length-1].value = value;
  }
  /** set a new temperature-type global parameter.
   * @param parkey user-defined 4-character key-word associated to the
   * global parameter
   * @param type must be equal to <tt>"TEMP"</tt>
   * @param microlib Microlib object containing the mixture reference
   * @param medium medium in <tt>microlib</tt> containing the temperature used
   * as global parameter
   */
  public void setGlobal(String parkey, String type, Microlib microlib, Composition medium) {
    if (!type.equals("TEMP")) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.addGlobal();
    this.para[this.para.length-1] = new Parameter(parkey, type);
    this.para[this.para.length-1].microlib = microlib;
    int imix = 0;
    for (int j=0;j<microlib.mixs.length;j++) {
      if (medium.name.equals(microlib.mixs[j].name)) imix = j+1;
    }
    if (imix == 0) {
      throw new RuntimeException("unable to find medium " + medium.name);
    }
    this.para[this.para.length-1].mix = imix;
  }
  /** set a new isotopic concentration-type global parameter.
   * @param parkey user-defined 4-character key-word associated to the
   * global parameter
   * @param type must be equal to <tt>"CONC"</tt>
   * @param microlib Microlib object containing the mixture reference
   * @param isotope Isotope object corresponding to the concentration
   * used as global parameter
   * @param medium medium in <tt>microlib</tt> containing the isotope
   */
  public void setGlobal(String parkey, String type, Isotope isotope, Microlib microlib, Composition medium) {
    if (!type.equals("CONC")) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.addGlobal();
    this.para[this.para.length-1] = new Parameter(parkey, type);
    this.para[this.para.length-1].isotope = isotope;
    this.para[this.para.length-1].microlib = microlib;
    int imix = 0;
    for (int j=0;j<microlib.mixs.length;j++) {
      if (medium.name.equals(microlib.mixs[j].name)) imix = j+1;
    }
    if (imix == 0) {
      throw new RuntimeException("unable to find medium " + medium.name);
    }
    this.para[this.para.length-1].mix = imix;
  }
  /** set a new temperature-type global parameter.
   * @param parkey user-defined 4-character key-word associated to the
   * global parameter
   * @param type must be equal to <tt>"TEMP"</tt>
   * @param microlib Autop object containing the mixture reference
   * @param medium medium in <tt>microlib</tt> containing the temperature used
   * as global parameter
   */
  public void setGlobal(String parkey, String type, Autop microlib, Composition medium) {
    if (!type.equals("TEMP")) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.addGlobal();
    this.para[this.para.length-1] = new Parameter(parkey, type);
    this.para[this.para.length-1].microlib = microlib;
    int imix = 0;
    for (int j=0;j<microlib.mixs.length;j++) {
      if (medium.name.equals(microlib.mixs[j].name)) imix = j+1;
    }
    if (imix == 0) {
      throw new RuntimeException("unable to find medium " + medium.name);
    }
    this.para[this.para.length-1].mix = imix;
  }
  /** set a new isotopic concentration-type global parameter.
   * @param parkey user-defined 4-character key-word associated to the
   * global parameter
   * @param type must be equal to <tt>"CONC"</tt>
   * @param isotope Isotope object corresponding to the concentration
   * used as global parameter
   * @param microlib Autop object containing the mixture reference
   * @param medium medium in <tt>microlib</tt> containing the isotope
   */
  public void setGlobal(String parkey, String type, Isotope isotope, Autop microlib, Composition medium) {
    if (!type.equals("CONC")) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.addGlobal();
    this.para[this.para.length-1] = new Parameter(parkey, type);
    this.para[this.para.length-1].isotope = isotope;
    this.para[this.para.length-1].microlib = microlib;
    int imix = 0;
    for (int j=0;j<microlib.mixs.length;j++) {
      if (medium.name.equals(microlib.mixs[j].name)) imix = j+1;
    }
    if (imix == 0) {
      throw new RuntimeException("unable to find medium " + medium.name);
    }
    this.para[this.para.length-1].mix = imix;
  }
  /** set a new local parameter.
   * @param parkey user-defined 4-character key-word associated to the
   * global parameter
   * @param type type of the local parameter. This variable is
   * chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"TEMP"</tt> <dd> temperature
   * <dt> <tt>"IRRA"</tt> <dd> burnup (in MWday/Tonne)
   * <dt> <tt>"FLUB"</tt> <dd> exposure rate (in n/kb)
   * <dt> <tt>"POWR"</tt> <dd> power
   * <dt> <tt>"MASL"</tt> <dd> mass density of heavy isotopes
   * <dt> <tt>"FLUX"</tt> <dd> volume-averaged, energy-integrated flux
   * </dl></ul>
   */
  public void setLocal(String parkey, String type) {
    if ((!type.equals("TEMP"))&(!type.equals("IRRA"))&
       (!type.equals("FLUB"))&(!type.equals("POWR"))&
       (!type.equals("MASL"))&(!type.equals("FLUX"))) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.addLocal();
    this.loca[this.loca.length-1] = new Parameter(parkey, type);
  }
  /** set a new isotopic concentration-type local parameter.
   * @param parkey user-defined 4-character key-word associated to the
   * local parameter
   * @param type must be equal to <tt>"CONC"</tt>
   * @param isotope Isotope object corresponding to the concentration
   * used as local parameter
   */
  public void setLocal(String parkey, String type, Isotope isotope) {
    if (!type.equals("CONC")) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.addLocal();
    this.loca[this.loca.length-1] = new Parameter(parkey, type);
    this.loca[this.loca.length-1].isotope = isotope;
  }
  /** set a float-value type global parameter
   * @param parkey user-defined 4-character key-word associated to the
   * local parameter
   * @param value float-value (float) assigned to the global parameter
   */
  public void setParam(String parkey, float value) {
    if (this.parkey == null) this.parkey = new Hashtable<String, Object>();
    this.parkey.put(parkey, new Float(value));
  }
  /** set a integer-value type global parameter
   * @param parkey user-defined 4-character key-word associated to the
   * local parameter
   * @param value integer-value (int) assigned to the global parameter
   */
  public void setParam(String parkey, int value) {
    if (this.parkey == null) this.parkey = new Hashtable<String, Object>();
    this.parkey.put(parkey, new Integer(value));
  }
    /** set a String-value type global parameter
   * @param parkey user-defined 4-character key-word associated to the
   * local parameter
   * @param value String-value (12-character String) assigned to the
   * global parameter
   */
  public void setParam(String parkey, String value) {
    if (this.parkey == null) this.parkey = new Hashtable<String, Object>();
    this.parkey.put(parkey, value);
  }
  /** initialize the LCM structure (database) embedded in this Compo object
   */
  public void exec() throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.exec();
    this.comment = null;
    this.para = null;
    this.loca = null;
  }
  /** add a new elementary calculation to the Compo Database
   * @param a existing Compo object containing the database
   * @param b Edition object of the elementary calculation
   * @param c Evo object corresponding to the elementary calculation
   * @param d Microlib object corresponding to the elementary calculation
   */
  public void exec(Compo a, Edition b, Evo c, Microlib d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    super.exec();
  }
  /** add a new elementary calculation to the Compo Database
   * @param a existing Compo object containing the database
   * @param b Edition object of the elementary calculation
   * @param c Evo object corresponding to the elementary calculation
   * @param d Autop object corresponding to the elementary calculation
   */
  public void exec(Compo a, Edition b, Evo c, Autop d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    super.exec();
  }
  /** catenate a Compo object into another
   * @param a existing Compo object containing the database
   * @param b Compo object we want to catenate into <tt>a</tt>
   */
  public void exec(Compo a, Compo b) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.exec();
  }
  protected void makeText() {
    this.operator="COMPO: ";
    super.write("EDIT", super.edit);
    if (this.maxcal != 0) super.write("MAXCAL", this.maxcal);
    if (this.comment != null) {
      super.write("\nCOMM", this.comment);
      super.write("ENDC");
    }
    if (this.para != null) {
      for (int i=0;i<this.para.length;i++) {
        super.write("\nPARA", this.para[i].parkey);
        if (this.para[i].type.equals("TEMP")) {
          super.write(this.para[i].type, this.para[i].microlib.name);
          super.write(this.para[i].mix);
        } else if (this.para[i].type.equals("CONC")) {
          super.write(this.para[i].type, this.para[i].isotope.name);
          super.write(this.para[i].microlib.name, this.para[i].mix);
        } else if (this.para[i].type.equals("VALU")) {
          super.write(this.para[i].type, this.para[i].value);
        } else {
          super.write(this.para[i].type);
        }
      }
    }
    if (this.loca != null) {
      for (int i=0;i<this.loca.length;i++) {
        super.write("\nLOCA", this.loca[i].parkey);
        if (this.loca[i].type.equals("CONC")) {
          super.write(this.loca[i].type, this.loca[i].isotope.name);
        } else {
          super.write(this.loca[i].type);
        }
      }
    }
    if ((this.para != null) | (this.loca != null)) super.write("\nINIT");
    if (this.parkey != null) {
      Enumeration ee = this.parkey.keys();
      while(ee.hasMoreElements()) {
        String key = (String)ee.nextElement();
        super.write("\n");
        super.write(key, this.parkey.get(key));
      }
    }
    if (this.orig != 0) super.write("\nORIG", this.orig);
    if (this.set != null) {
      super.write("\nSET", this.set.xtr);
      super.write(this.set.unit);
    }
    super.write(";");
  }
}
