package jargon;

import java.io.*;
/**
 * This class defines the physical properties of a single medium,
 * as required in a lattice calculation.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Composition implements Serializable, Cloneable {
  static {
    System.out.println("Composition class loaded");
  }
  static String fr = "definition d'une composition";
  static String en = "definition of a composition";
  private static final long serialVersionUID = 7L;
  protected String name;
  protected String type = "medium"; //into=('mixture','material','medium')
  /** temperature in Celsius of the medium.
   */
  public float temperature;
  /** massic density of the material/medium.
   */
  public float massDensity;
  /** type of relative fractions. This variable is chosen among the
   * following values:
   * <ul>
   * <dt> <tt>"mass_fract"</tt> <dd> mass fractions
   * <dt> <tt>"number_fract"</tt> <dd> particle fractions
   */
  public String method;
  /** set to true to normalize the relative fractions to 1.0.
   */
  public boolean norm;
  /** single Composition used to make the mixture/material/medium.
   */
  public Composition alias;
  /** single Isotope used to make the mixture/material/medium.
   */
  public Isotope isotope;
  /** definition of an array of components making the mixture/material/medium.
   */
  public CompoInner[] isoCompo;
  /** This class defines a single mixture/material/medium component.
   * <tt>alias</tt>, <tt>isotope</tt> and <tt>isoCompo</tt> are exclusive
   */
  public class CompoInner implements Serializable, Cloneable {
    private static final long serialVersionUID = 9L;
    /** the component is an isotope.
     */
    public Isotope isotope;
    /** the component is a composition. <tt>isotope</tt> and <tt>composition</tt>
     * fields are exclusive.
     */
    public Composition composition; // isotope and composition fields are exclusive
    /** concentration of the component.
     */
    public float concentration;
    public Object clone() {
      try {
        CompoInner o = (CompoInner)super.clone();
        if (this.isotope != null) o.isotope = (Isotope)this.isotope.clone();
        if (this.composition != null) o.composition = (Composition)this.composition.clone();
        return o;
      } catch (CloneNotSupportedException e) {
        throw new InternalError();
      }
    }
  }
  /** use this constructor to define a composition with a specific type.
   * @param name user-defined name of the composition
   * @param type type of composition. This variable is chosen among the
   * following values:
   * <ul><dl>
   * <dt> <tt>"mixture"</tt> <dd> a composition defined only by its
   * isotopes and relative proportion of its constituents. The mass
   * density and temperature are not defined.
   * <dt> <tt>"material"</tt> <dd> a composition defined only by its
   * isotopes and absolute proportion of its constituents. The temperature
   * is not defined.
   * <dt> <tt>"medium"</tt> <dd> a completely defined composition,
   * including a temperature attribute.
   * </dl></ul>
   */
  public Composition(String name, String type) {
    this.name = name;
    if ((!type.equals("mixture"))&(!type.equals("material"))&
       (!type.equals("medium"))) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.type = type;
  }
  /** use this constructor to define a composition of type <tt>"medium"</tt>.
   * @param name user-defined name of the composition
   */
  public Composition(String name) {
    this(name, "medium");
  }
  /** definition of Isotope-type components.
   * @param iso set of Isotopes used as components.
   * @param conc concentration of the components.
   */
  public void setComposition(Isotope[] iso, float[] conc) {
    int nisot = iso.length;
    if (conc.length != nisot) {
      throw new RuntimeException("invalid length (" + nisot + ")");
    }
    this.isoCompo = new CompoInner[nisot];
    for (int i=0; i<nisot; i++) {
      this.isoCompo[i] = new CompoInner();
      this.isoCompo[i].isotope = iso[i];
      this.isoCompo[i].concentration = conc[i];
    }
  }
  /** definition of Isotope-type components.
   * @param factor factor used to multiply the concentrations.
   * @param iso set of Isotopes used as components.
   * @param conc concentration of the components.
   */
  public void setComposition(float factor, Isotope[] iso, float[] conc) {
    int nisot = iso.length;
    if (conc.length != nisot) {
      throw new RuntimeException("invalid length (" + nisot + ")");
    }
    this.isoCompo = new CompoInner[nisot];
    for (int i=0; i<nisot; i++) {
      this.isoCompo[i] = new CompoInner();
      this.isoCompo[i].isotope = iso[i];
      this.isoCompo[i].concentration = factor*conc[i];
    }
  }
  /** definition of Composition-type components.
   * @param compo set of Compositions used as components.
   * @param conc concentration of the components.
   */
  public void setComposition(Composition[] compo, float[] conc) {
    int nisot = compo.length;
    if (conc.length != nisot) {
      throw new RuntimeException("invalid length (" + nisot + ")");
    }
    this.isoCompo = new CompoInner[nisot];
    for (int i=0; i<nisot; i++) {
      this.isoCompo[i] = new CompoInner();
      this.isoCompo[i].composition = compo[i];
      this.isoCompo[i].concentration = conc[i];
    }
  }
  /** implementation of the Cloneable interface.
   */
  public Object clone() {
    try {
      Composition o = (Composition)super.clone();
      if(this.alias != null) o.alias = (Composition)this.alias.clone();
      o.isoCompo = this.isoCompo.clone();
      for (int i=0; i<isoCompo.length; i++) {
        o.isoCompo[i] = (CompoInner)this.isoCompo[i].clone();
      }
      return o;
    } catch (CloneNotSupportedException e) {
      throw new InternalError();
    }
  }
  /** create a clone of a Composition object and rename it.
   * @param name user-defined name of the composition
   */
  public Object clone(String name) {
    Composition o = (Composition)this.clone();
    o.name = name;
    return o;
  }
}
