package jargon;
  /**
   * The Edition operator is used to perform homogenization and/or
   * condensation of multi-region/multigroup information with the
   * optional use of a superhomogenization (SPH) model. The information
   * saved in the embedded LCM structure include integrated fluxes
   * and both macroscopic and microscopic cross section defined for
   * a set of selected isotopes.
   * <P>
   * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
   *
   * @author Alain Hebert
   */

public class Edition extends Operator {
  static {
    System.out.println("Edition class loaded");
  }
  static String fr = "definition d'un objet edition";
  static String en = "definition of an edition object";
  private static final long serialVersionUID = 55L;
  /** specify that the fluxes, the macroscopic and microscopic cross sections
   * and the volumes corresponding to homogenized regions are to be saved
   * on the embedded LCM structure. <tt>save</tt> is the name of the directory
   * on which the homogenized/condensed information is to be stored. If this
   * name is set to <tt>"yes"</tt>, a default name is used.
   */
  public String save;
  /** set to <tt>true</tt> to specify that the reaction rates and the
   * condensed and/or homogenized cross sections are corrected so as
   * to eliminate up-scattering. This option is useful for reactor analysis
   * codes which cannot take into account such cross sections. By default,
   * no correction is performed.
   */
  public boolean ups;
  /** array of increasing energy group limits that will be associated with
   * each of the <tt>ngcond</tt> condensed groups. The final value of
   * <tt>icond</tt> will automatically be set to <tt>ngroup</tt> while the
   * values of <tt>icond > ngroup</tt> will be droped from the condensation.
   * We must have <tt>ngcond <= ngroup</tt>. By default, no condensation is
   * performed.
   */
  public int[] cond;
  /** type of homogenization. This variable is chosen among the following
   * values:
   * <dl>
   * <dt> <tt>"COMP"</tt> <dd> a complete homogenization is to take place.
   * <dt> <tt>"CELL"</tt> <dd> a cell-by-cell homogenization (without SPH
   * equivalence) is to take place.
   * <dt> <tt>"NONE"</tt> <dd> the reference geometry is not going to be
   * homogenized (default).
   * <dt> <tt>"REGI"</tt> <dd> the homogenized regions are defined from
   * region indices of the reference geometry. The <tt>regi</tt> variable
   * is defined.
   * <dt> <tt>"MIX"</tt> <dd>  the homogenized regions are defined from
   * mixture indices of the reference geometry. The <tt>mix</tt> variable
   * is used to define how different mixtures are homogenized together.
   * If the <tt>mix</tt> variable is not set, a mixture-by-mixture
   * homogenization is performed.
   * </dl>
   */
  public String merge;
  /** array of homogenized region indices to which are associated the
   * reference regions. In the editing routines a value of <tt>regi=0</tt>
   * allows the corresponding region to be neglected.
   */
  public int[] regi;
  /** array of homogenized region indices to which are associated the
   * isotopic mixtures. In the editing routines a value of <tt>mix=0</tt>
   * allows the corresponding isotopic mixtures to be neglected. For a
   * mixture in this library which is not used in the geometry one should
   * insert a value of 0 for the new region index associated with this
   * mixture.
   */
  public int[] mix;
  /** automatic condensation of microscopic cross section information.
   * This variable is chosen among the following values:
   * <dl>
   * <dt> <tt>"ALL"</tt> <dd> all the isotopes present in the homogenized
   * regions are to be processed individually. No macroscopic residual
   * isotope is produced.
   * <dt> <tt>"RES"</tt> <dd> all the isotopes present in the homogenized
   * region will be combined in a unique macroscopic residual isotope (with
   * a number density equal to 1.0)
   * </dl>
   */
  public String micr1;
  /** condensation of microscopic cross section information for the
   * selected isotopes and for a macroscopic residual isotope. <tt>micr2</tt>
   * is the array of selected Isotope objects.
   */
  public Isotope[] micr2;
  /** user-selected options. This variable is chosen among the following
   * values:
   * <dl>
   * <dt> <tt>"P0W"</tt> <dd> the P1 information is to be homogenized and
   * condensed using the scalar flux. This is the default option in
   * cases where no streaming model is used (i.e., if options <tt>ECCO</tt>
   * and <tt>HETE</tt> are <i>not</i> used in operator Flux).
   * <dt> <tt>"P1W"</tt> <dd> the P1 information is to be homogenized and
   * condensed using the fundamental current recovered from a streaming
   * model. Fundamental current-weighted total cross sections are computed.
   * This is the default option in cases where a streaming model is used
   * (i.e., if options <tt>ECCO</tt> or <tt>HETE</tt> is used in operator Flux).
   * <dt> <tt>"NOHF"</tt> <dd> suppress the calculation and edition of the
   * H-factors (sum of all the cross sections producing energy times the
   * energy produced by each reaction). Note that this calculation may be
   * time-consuming. By default, the H-factors are computed and edited.
   * <dt> <tt>"NBAL"</tt> <dd> enable the editing of the four factors computed
   * from a group balance. In this case, the user must specify explicitly a
   * three group condensation.
   * </dl>
   */
  public String[] options;
  /** a superhomogenization (SPH) model is used to correct the integrated
   * fluxes and the homogenized/condensed cross sections. <tt>sph</tt>
   * is the Tracking object used in this operation.
   */
  public Tracking sph;
  /** 
   * Use this constructor to create a new Edition object.
   * @param name user-defined name of the Edition object
   */
  public Edition(String name) {
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
  }
  /** initialize the Edition operator and perform the actions specified
   * by its instance variables.
   * @param a Flux object containing the reference fluxes.
   * @param b Macrolib object where reference macroscopic cross sections
   * are recovered.
   * @param c Tracking object corresponding to the reference flux.
   */
  public void exec(Flux a, Macrolib b, Tracking c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    if (c.fileObj != null) super.inputJlcm(c.fileObj);
    super.exec();
  }
  /** perform the actions specified by the instance variables of this Edition
   * object and update the embedded LCM structure.
   * @param a Existing Edition object.
   * @param b Flux object containing the reference fluxes.
   * @param c Macrolib object where reference macroscopic cross sections
   * are recovered.
   * @param d Tracking object corresponding to the reference flux.
   */
  public void exec(Edition a, Flux b, Macrolib c, Tracking d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    if (d.fileObj != null) super.inputJlcm(d.fileObj);
    super.exec();
  }
  /** initialize the Edition operator and perform the actions specified
   * by its instance variables in case where <tt>cond = "CELL"</tt> or
   * where <tt>sph</tt> is set.
   * @param a Flux object containing the reference fluxes.
   * @param b Macrolib object where reference macroscopic cross sections
   * are recovered.
   * @param c Tracking object corresponding to the reference flux.
   * @param d Gigogne object of the reference geometry.
   */
  public void exec(Flux a, Macrolib b, Tracking c, Gigogne d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    if (c.fileObj != null) super.inputJlcm(c.fileObj);
    super.inputJlcm(d.lcmObj);
    super.exec();
  }
  /** perform the actions specified by the instance variables of this Edition
   * object and update the embedded LCM structure in case where <tt>cond = "CELL"</tt>
   * or where <tt>sph</tt> is set.
   * @param a Existing Edition object.
   * @param b Flux object containing the reference fluxes.
   * @param c Macrolib object where reference macroscopic cross sections
   * are recovered.
   * @param d Tracking object corresponding to the reference flux.
   * @param e Gigogne object of the reference geometry.
   */
  public void exec(Edition a, Flux b, Macrolib c, Tracking d, Gigogne e) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    if (d.fileObj != null) super.inputJlcm(d.fileObj);
    super.inputJlcm(e.lcmObj);
    super.exec();
  }
  /** initialize the Edition operator and perform the actions specified
   * by its instance variables.
   * @param a Flux object containing the reference fluxes.
   * @param b Microlib object where reference cross sections are recovered.
   * @param c Tracking object corresponding to the reference flux.
   */
  public void exec(Flux a, Microlib b, Tracking c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    if (c.fileObj != null) super.inputJlcm(c.fileObj);
    super.exec();
  }
  /** perform the actions specified by the instance variables of this Edition
   * object and update the embedded LCM structure.
   * @param a Existing Edition object.
   * @param b Flux object containing the reference fluxes.
   * @param c Microlib object where reference cross sections are recovered.
   * @param d Tracking object corresponding to the reference flux.
   */
  public void exec(Edition a, Flux b, Microlib c, Tracking d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    if (d.fileObj != null) super.inputJlcm(d.fileObj);
    super.exec();
  }
  /** initialize the Edition operator and perform the actions specified
   * by its instance variables in case where <tt>cond = "CELL"</tt> or
   * where <tt>sph</tt> is set.
   * @param a Flux object containing the reference fluxes.
   * @param b Microlib object where reference cross sections are recovered.
   * @param c Tracking object corresponding to the reference flux.
   * @param d Gigogne object of the reference geometry.
   */
  public void exec(Flux a, Microlib b, Tracking c, Gigogne d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    if (c.fileObj != null) super.inputJlcm(c.fileObj);
    super.inputJlcm(d.lcmObj);
    super.exec();
  }
  /** perform the actions specified by the instance variables of this Edition
   * object and update the embedded LCM structure in case where <tt>cond = "CELL"</tt>
   * or where <tt>sph</tt> is set.
   * @param a Existing Edition object.
   * @param b Flux object containing the reference fluxes.
   * @param c Microlib object where reference cross sections are recovered.
   * @param d Tracking object corresponding to the reference flux.
   * @param e Gigogne object of the reference geometry.
   */
  public void exec(Edition a, Flux b, Microlib c, Tracking d, Gigogne e) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    if (d.fileObj != null) super.inputJlcm(d.fileObj);
    super.inputJlcm(e.lcmObj);
    super.exec();
  }
  /** initialize the Edition operator and perform the actions specified
   * by its instance variables.
   * @param a Flux object containing the reference fluxes.
   * @param b Autop object where reference cross sections are recovered.
   * @param c Tracking object corresponding to the reference flux.
   */
  public void exec(Flux a, Autop b, Tracking c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    if (c.fileObj != null) super.inputJlcm(c.fileObj);
    super.exec();
  }
  /** perform the actions specified by the instance variables of this Edition
   * object and update the embedded LCM structure.
   * @param a Existing Edition object.
   * @param b Flux object containing the reference fluxes.
   * @param c Autop object where reference cross sections are recovered.
   * @param d Tracking object corresponding to the reference flux.
   */
  public void exec(Edition a, Flux b, Autop c, Tracking d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    if (d.fileObj != null) super.inputJlcm(d.fileObj);
    super.exec();
  }
  /** initialize the Edition operator and perform the actions specified
   * by its instance variables in case where <tt>cond = "CELL"</tt> or
   * where <tt>sph</tt> is set.
   * @param a Flux object containing the reference fluxes.
   * @param b Autop object where reference cross sections are recovered.
   * @param c Tracking object corresponding to the reference flux.
   * @param d Gigogne object of the reference geometry.
   */
  public void exec(Flux a, Autop b, Tracking c, Gigogne d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    if (c.fileObj != null) super.inputJlcm(c.fileObj);
    super.inputJlcm(d.lcmObj);
    super.exec();
  }
  /** perform the actions specified by the instance variables of this Edition
   * object and update the embedded LCM structure in case where <tt>cond = "CELL"</tt>
   * or where <tt>sph</tt> is set.
   * @param a Existing Edition object.
   * @param b Flux object containing the reference fluxes.
   * @param c Autop object where reference cross sections are recovered.
   * @param d Tracking object corresponding to the reference flux.
   * @param e Gigogne object of the reference geometry.
   */
  public void exec(Edition a, Flux b, Autop c, Tracking d, Gigogne e) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    if (d.fileObj != null) super.inputJlcm(d.fileObj);
    super.inputJlcm(e.lcmObj);
    super.exec();
  }
  protected void makeText() {
    this.operator="EDI: ";
    super.write("EDIT", super.edit);
    if (this.save != null) {
      if (this.save == "yes") {
        super.write("\nSAVE");
      } else {
        super.write("\nSAVE ON"); super.write("'"+this.save+"'");
      }
    }
    if (this.ups) super.write("\nUPS");
    if (this.options != null) super.write("\n", this.options);
    if (this.cond != null) super.write("\nCOND", this.cond);
    if (this.merge != null) {
      super.write("\nMERG"); super.write(this.merge);
      if (this.merge == "REGI") {
        super.write(this.regi);
      } else if ((this.merge == "MIX")&(this.mix != null)) {
        super.write(this.mix);
      }
    }
    if (this.micr1 != null) {
      super.write("\nMICR"); super.write(this.micr1);
    } else if (this.micr2 != null) {
      super.write("\nMICR"); super.write(this.micr2.length);
      for (int is=0;is<this.micr2.length;is++) super.write(this.micr2[is].name);
    }
    if (this.sph != null) {
      super.write("\nSPH :::");
      super.write(this.sph.getClass().getName().toUpperCase().substring(7)+"T:");
      this.sph.makeText();
    }
    super.write(";");
  }
}
