package jargon;

import java.io.*;
/**
 * The Evo operator is used to compute and record the isotopic depletion
 * history of the materials present in the domain.
 * <P>
 * The depletion equations for the various isotopes of the Microlib object
 * are solved using the burnup chains also present in the Microlib object.
 * Both in-core and out-of-core calculations  can be considered. For in-core
 * depletion calculations, one assumes linear flux variation over each
 * irradiation period (time stage). The initial (and possibly final) flux
 * distributions are recovered from previous flux calculations. In-core
 * depletion can be performed at constant flux or constant power (expressed
 * in W/cc of material or MW/Tonne of initial heavy elements) but these values
 * can undergo step variations from one time stage to another.  All the
 * information required for successive burnup calculation is stored in the
 * LCM structure embedded in the Evo object. Thus it is possible at any point
 * in time to return to a previous time step and restart the calculations.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Evo extends Operator {
  static {
    System.out.println("Evo class loaded");
  }
  static String fr = "operateur d'evolution isotopique";
  static String en = "isotopic depletion operator";
  private static final long serialVersionUID = 57L;
  private Save save;
  private Depl depl;
  private Set set;
  /** tolerance used in the algorithm for the solution of the depletion
   * equations.
   */
  public float eps1;
  /** tolerance used in the search algorithm for a final fixed power. This
   * variable is used used if the <tt>"POWR"</tt> or <tt>"W/CC"</tt> option
   * is activated.
   */
  public float eps2;
  /** selection criterion for non-fissile isotopes that are at saturation.
   * The isotopes for which <tt>lambda * (xtf - xti) >= this.expm</tt> will
   * be treated by a saturation approximation. Here, <tt>lambda</tt> is the
   * sum of the radioactive decay constant and microscopic neutron
   * absorption rate. The default value is <tt> this.expm = 80.0f</tt>.
   * In order to remove the saturation approximation for all isotopes set
   * <tt>expm</tt> to a very large number such as <tt>1.0e5f</tt>.
   */
  public float expm;
  /** estimate of the relative width of the time step used in the solution
   * of burnup equations. This value is optimized dynamically by the program.
   * The default value is <tt>h1 = 1.0e-4f</tt>.
   */
  public float h1;
  /** numerical method used to solve the depletion (or Bateman) equations.
   * This variable is chosen among the following values:
   * <dl>
   * <dt> <tt>"RUNG"</tt> <dd> use the 5-th order Cash-Karp algorithm
   * <dt> <tt>"KAPS"</tt> <dd> use the 4-th order Kaps-Rentrop algorithm
   * </dl>
   */
  public String solution;
  /** set to <tt>true</tt> to perform a linear extrapolation of the microscopic
   * reaction rates, using the available information preceding the initial
   * time <tt>xti</tt>. Default: <tt>extr = true</tt>.
   */
  public boolean extr = true;
  /** set to <tt>true</tt> to compute the burnup using the energy released in
   * the complete geometry. This option have an effect only in cases where
   * some energy is released outside the fuel (e.g., due to (n,gamma) reactions).
   * This option affects both the meaning the argument <tt>value</tt> of the
   * method <tt>setDepl()</tt> (with <tt>type = "POWR"</tt>) and the value
   * of the burnup, as computed by the Evo operator. Default: <tt>glob = false</tt>.
   */
  public boolean glob = false;
  /** set to <tt>true</tt> to save the saturated initial number densities in
   * the LCM structure embedded in the Evo object. Default: <tt>satureInit
   * = false</tt> (i.e. the non-saturated initial number densities are saved).
   */
  public boolean satureInit = false;
  /** set to <tt>true</tt> to compute the saturated number densities using
   * a Dirac density component.  Default: <tt>dirac = false</tt>.
   */
  public boolean dirac = false;
  /** 
   * Use this constructor to create a new Evo object.
   * @param name user-defined name of the Evo object
   */
  public Evo(String name) {
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
  }
  private class Save implements Serializable {
    private static final long serialVersionUID = 58L;
    private float xts;
    private String unit; //into=('S','DAY','YEAR')
    private String type = null; //into=('FLUX','POWR','W/CC','NOSA')
    private float value = 0.0f;
  }
  /** the current isotopic concentration and the microscopic reaction rates
   * resulting from the last transport calculation will be normalized and
   * stored on the embedded LCM structure in a sub-directory corresponding
   * to a specific time.
   * @param xts time associated with the transport calculation.
   * @param unit unit of <tt>xts</tt>. This value is chosen among
   * <tt>"S"</tt>, <tt>"DAY"</tt> or <tt>"YEAR"</tt>.
   * @param type type of power normalization. This variable is
   * chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"FLUX"</tt> <dd> integrated flux normalization. <tt>value</tt>
   * is expressed in 1/(cm**2 s).
   * <dt> <tt>"POWR"</tt> <dd> power normalization. The energy released
   * outside the fuel (e.g., by (n,gamma) reactions) is <i>not</i> taken
   * into account in the flux normalization, unless <tt>glob == true</tt>.
   * <tt>value</tt> is expressed in MW/Tonne of heavy initial elements.
   * <dt> <tt>"W/CC"</tt> <dd> power normalization. The energy released
   * outside the fuel (e.g., by (n,gamma) reactions) is always taken into
   * account in the flux normalization. <tt>value</tt> is expressed in W/cc
   * (Power per unit assembly volume).
   * </dl></ul>
   * @param value flux normalization factor.
   */
  public void setSave(float xts, String unit, String type, float value) {
    if ((!type.equals("FLUX"))&(!type.equals("POWR"))&
       (!type.equals("W/CC"))) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.save = new Save();
    this.save.xts = xts;
    this.save.unit = unit;
    this.save.type = type;
    this.save.value = value;
  }
  /** the current isotopic concentration and the microscopic reaction rates
   * resulting from the last transport calculation will not be stored in the
   * embedded LCM structure.
   * @param type must be equal to <tt>"NOSA"</tt>.
   */
  public void setSave(String type) {
    if (!type.equals("NOSA")) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.save.type = type;
  }
  
  private class Depl implements Serializable {
    private static final long serialVersionUID = 59L;
    private float xti, xtf;
    private String unit;
    private String type;
    private float value = 0.0f;
  }
  /** a burnup calculation between an initial and a final time must be
   * performed. In the case where the <tt>setSave()</tt> method is
   * not used, the initial isotopic concentration will be stored in the
   * embedded LCM structure on a sub-directory corresponding to the initial
   * time. If the <tt>setSet()</tt> method is not used, the isotopic
   * concentration corresponding to the final burnup time will be used to
   * update the embedded LCM structure.
   * @param xti initial time of the burnup calculation.
   * @param xtf final time of the burnup calculation.
   * @param unit unit of <tt>xts</tt>. This value is chosen among
   * <tt>"S"</tt>, <tt>"DAY"</tt> or <tt>"YEAR"</tt>.
   * @param type type of power normalization. This variable is
   * chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"COOL"</tt> <dd> a zero flux burnup calculation is to be
   * performed.
   * <dt> <tt>"FLUX"</tt> <dd> a constant flux burnup calculation is to be
   * performed. <tt>value</tt> is expressed in 1/(cm**2 s).
   * <dt> <tt>"POWR"</tt> <dd> a constant fuel power depletion calculation
   * is to be performed. The energy released outside the fuel (e.g., by
   * (n,gamma) reactions) is <i>not</i> taken into account in the flux
   * normalization, unless <tt>glob == true</tt>. <tt>value</tt> is
   * expressed in MW/Tonne of heavy initial elements.
   * <dt> <tt>"W/CC"</tt> <dd> a constant assembly power depletion
   * calculation is to be performed. The energy released outside the fuel
   * (e.g., by (n,gamma) reactions) is always taken into account in the
   * flux normalization. <tt>value</tt> is expressed in W/cc (Power per
   * unit assembly volume).
   * </dl></ul>
   * @param value flux normalization factor.
   */
  public void setDepl(float xti, float xtf, String unit, String type, float value) {
    if ((!type.equals("COOL"))&(!type.equals("FLUX"))&
       (!type.equals("POWR"))&(!type.equals("W/CC"))) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.depl = new Depl();
    this.depl.xti = xti;
    this.depl.xtf = xtf;
    this.depl.unit = unit;
    this.depl.type = type;
    if (type != "COOL") this.depl.value = value;
  }
  /** unset the instance variables related to the depletion calculation.
   * No depletion calculation will be berformed by the <tt>exec()</tt>
   * method.
   */
  public void unsetDepl() {
    this.depl = null;
  }
  private class Set implements Serializable {
    private static final long serialVersionUID = 60L;
    private float xtr;
    private String unit;
  }
  /** recover the isotopic concentration already stored in the embedded
   * LCM structure from a sub-directory corresponding to a specific time.
   * @param xtr time flag where the isotopic concentration are recovered.
   * @param unit unit of <tt>xtr</tt>. This value is chosen among
   * <tt>"S"</tt>, <tt>"DAY"</tt> or <tt>"YEAR"</tt>.
   */
  public void setSet(float xtr, String unit) {
    this.set = new Set();
    this.set.xtr = xtr;
    this.set.unit = unit;
  }
  /** initialize and execute the Evo operator and perform the actions
   * specified by the <tt>setSave()</tt>, <tt>setDEPL()</tt> and/or
   * <tt>setSet()</tt> methods.
   * @param a Microlib object where microscipic cross sections are recovered.
   * @param b Flux object where flux values are recovered.
   * @param c tracking object where flux-related numerotation is recovered.
   */
  public void exec(Microlib a, Flux b, Tracking c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.outputJlcm(a.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.exec();
  }
  /** execute the Evo operator and perform the actions specified by the
   * <tt>setSave()</tt>, <tt>setDEPL()</tt> and/or <tt>setSet()</tt>
   * methods. The information in the embedded LCM structure is updated.
   * @param a existing EVO object.
   * @param b Microlib object where microscipic cross sections are recovered.
   * @param c Flux object where flux values are recovered.
   * @param d tracking object where flux-related numerotation is recovered.
   */
  public void exec(Evo a, Microlib b, Flux c, Tracking d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.outputJlcm(b.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    super.exec();
  }
  /** initialize and execute the Evo operator and perform the actions
   * specified by the <tt>setSave()</tt>, <tt>setDEPL()</tt> and/or
   * <tt>setSet()</tt> methods.
   * @param a Autop object where microscipic cross sections are recovered.
   * @param b Flux object where flux values are recovered.
   * @param c tracking object where flux-related numerotation is recovered.
   */
  public void exec(Autop a, Flux b, Tracking c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.outputJlcm(a.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.exec();
  }
  /** execute the Evo operator and perform the actions specified by the
   * <tt>setSave()</tt>, <tt>setDEPL()</tt> and/or <tt>setSet()</tt>
   * methods. The information in the embedded LCM structure is updated.
   * @param a existing EVO object.
   * @param b Autop object where microscipic cross sections are recovered.
   * @param c Flux object where flux values are recovered.
   * @param d tracking object where flux-related numerotation is recovered.
   */
  public void exec(Evo a, Autop b, Flux c, Tracking d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.outputJlcm(b.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    super.exec();
  }
  protected void makeText() {
    this.operator="EVO: ";
    super.write("EDIT", super.edit);
    if ((this.save != null) && (this.save.type == "NOSA")) {
      super.write("\nNOSA");
    } else if (this.save != null) {
      super.write("\nSAVE", this.save.xts);
      super.write(this.save.unit);
      super.write(this.save.type);
      super.write(this.save.value);
    }
    if (this.depl != null) {
      super.write("\nDEPL", this.depl.xti);
      super.write(this.depl.xtf);
      super.write(this.depl.unit);
      super.write(this.depl.type);
      if (this.depl.type != "COOL") super.write(this.depl.value);
    }
    if (this.set != null) {
      super.write("\nSET", this.set.xtr);
      super.write(this.set.unit);
    }
    super.write("\n");
    if (this.eps1 != 0.0f) super.write("EPS1", this.eps1);
    if (this.eps2 != 0.0f) super.write("EPS2", this.eps2);
    if (this.expm != 0.0f) super.write("EXPM", this.expm);
    if (this.h1 != 0.0f) super.write("H1", this.h1);
    if (this.solution != null) super.write(this.solution);
    if (this.extr) {
      super.write("EXTR");
    } else {
      super.write("NOEX");
    }
    if (this.glob) {
      super.write("GLOB");
    } else {
      super.write("NOGL");
    }
    if (this.satureInit) {
      super.write("SAT");
    } else {
      super.write("NSAT");
    }
    if (this.dirac) {
      super.write("DIRA");
    } else {
      super.write("NODI");
    }
    super.write(";");
  }
}
