package jargon;

/**
 * The Excell tracking operator produces a Jlcm structure containing the
 * energy-independent numerotation information required to solve the
 * transport equation over a 2D or 3D domain using the method of
 * characteristics or the method of collision probability.
 * <P>
 * The Excell operator can be used to analyse:
 * <ol>
 * <li> The two-dimensional geometries <tt>"CAR2D"</tt> and <tt>"HEX"</tt>
 * including respectively <tt>"CARCEL"</tt> and <tt>"HEXCEL"</tt> sub-geometries. 
 * <li> The two-dimensional cluster geometries corresponding to a <tt>"TUBE"</tt>
 * sub-geometry superimposed on a global <tt>"TUBE"</tt>, <tt>"CARCEL"</tt> or
 * <tt>"HEXCEL"</tt> geometry. Here, the only restriction is that the pins are
 * fully located inside the annular part of the cell and they do not overlap even
 * if they can overlap with internal annular regions.
 * <li> The three-dimensional geometry <tt>"CAR3D"</tt> including
 * <tt>"CARCELX"</tt>, <tt>"CARCELY"</tt> and <tt>"CARCELZ"</tt> sub-geometries
 * and the three dimensional  geometry <tt>"HEXZ"</tt> including <tt>"HEXCELZ"</tt>
 * sub-geometries. 
 * </ol>
 * <P>
 * An Excell object may be used within the Edition operator to perform SPH
 * equivalence calculations.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002).
 * The embedded Fortran native application was written by Robert Roy and Guy
 * Marleau.
 *
 * @author Alain Hebert
 */
public class Excell extends Tracking {
  static {
    System.out.println("Excell class loaded");
  }
  static String fr = "definition d'un tracking Excell";
  static String en = "definition of an Excell tracking";
  private static final long serialVersionUID = 110L;
  /** order of anisotropy in collision probability calculation. A default value
   * of 1 represents isotropic calculations while a value of 2 correspond to
   * linearly anisotropic collision probability. For the <tt>"PIJK"</tt> option,
   * a value of 2 is required.
   */
  public int nanis;
  /** type of boundary conditions for the collision probability calculation. This
   * variable is chosen among the following values:
   * <dl>
   * <dt> <tt>"PISO"</tt> <dd> a collision probability calculation with isotropic
   * reflection boundary conditions is required. It is the default option if a
   * <tt>"TISO"</tt> type integration is chosen. To obtain accurate transmission
   * probabilities for the isotropic case it is recommended that the normalization
   * options in the Asm operator be used.
   * <dt> <tt>"PSPC"</tt> <dd>  a collision probability calculation with specular
   * reflection boundary conditions required; this is the default option if a
   * <tt>"TSPC"</tt> type integration is chosen. This calculation is only
   * possible if the file was initially constructed using the <tt>"TSPC"</tt>
   * option.
   * </dl>
   */
  public String bc;
  /** maximum error allowed on the exponential function used for specular
   * collision probability calculations. Tracks will be cut at a length such
   * that the error in the probabilities resulting from this reduced track
   * will be of the order of <tt>cut</tt>. By default, there is no cutting
   * of the tracks and <tt>cut = 0.0f</tt>. If this option is used in an
   * entirely reflected case, it is preferable to use the <tt>"NORM"</tt>
   * option in the Asm operator.
   */
  public float cut;
  /** type of tracking parameters. This variable is chosen among the following
   * values:
   * <dl>
   * <dt> <tt>"TISO</tt> <dd> isotropic tracking parameters will be supplied.
   * This is the default tracking option for cluster geometries.
   * <dt> <tt>"TSPC</tt> <dd> specular tracking parameters will be supplied.
   * </dl>
   */
  public String trak;
  /** angular quadrature parameter. For applications involving 3D cells, the
   * choices are <tt>nangl=2, 4, 8, 10, 12, 14 or 16</tt>. The EQN angular
   * quadratures present a rotational symmetry about the three Cartesian axes.
   * For 2D isotropic  applications, any value of <tt>nangl</tt> may be used,
   * equidistant angles will be selected. For 2D specular applications the
   * input value must be of the form <tt>p+1</tt> where <tt>p</tt> is a prime
   * number (for example <tt>p = 7, 11,</tt> etc.); the choice of
   * <tt>nangl = 8, 12, 14, 18, 20, 24, or 30</tt> are allowed. For cluster
   * type geometries the default value is <tt>nangl = 10</tt> for isotropic
   * cases and <tt>nangl = 12</tt> for specular cases.
   */
  public int nangl;
  /** density of the integration lines. This information is given in cm^{-1}
   * for 2D cases and cm^{-2} for 3D cases. This choice of density along the
   * plan perpendicular to each angle depends on the geometry of the cell to
   * be analyzed. If there are zones of very small volume, a high line
   * density is essential. This value will be readjusted by this operator.
   * In the case of the analysis of a cluster type geometry the default value
   * of this parameter is 5/rm where rm is the minimum radius of the pins or
   * the minimum thickness of an annular ring in the geometry.
   */
  public float dens;
  /** maximum distance (cm) between a line and the intersection of <tt>n >= 2</tt>
   * external surfaces where track redistribution will take place. Track
   * redistribution will take place if a line comes close to the intersection of
   * <tt>n >= 2</tt> external surfaces. In this case the line will be replicated
   * <tt>n</tt> times, each of these lines being associated with a different
   * external surface, while its weight is reduced by a factor of <tt>1/n</tt>.
   * This allows for a better distribution of tracks which are relatively close
   * to <tt>n</tt> external surfaces. By default, there is no treatment of the
   * corners and <tt>corn = 0.0f</tt>.
   */
  public float corn;
  /** index describing the rotation symmetry of the geometry. The fixed default
   * of this parameter is 1. Excell automatically try to take into account
   * geometric symmetries in order to reduce the number of tracks and the CPU
   * time. Setting <tt>symm = -1</tt> desactivates this automatic capability.
   */
  public int symm;
  /** option related to the choice of tracking angles. Instead of selecting
   * the angles located at the end of each angular interval, the angles located
   * in the middle of these intervals are selected. This is particularly useful
   * if one wants to avoid tracking angles that are parallel to the X or Y axis
   * as its is the case when the external region of a <tt>CARCEL</tt> geometry
   * is voided.
   */
  public boolean medi;
  /**
   * Use this constructor to create a new Excell object.
   * @param name user-defined name of the Excell object
   */
  public Excell(String name) {
    super.lcmObj = new Jlcm("LCM", name);
    super.fileObj = new Jlcm("BINARY", name + ".bin");
    super.name = name;
  }
  /** load the embedded lcmObj structure corresponding to this Excell tracking.
   * @param a 2D or 3D geometry to analyse.
   */
  public void exec(Gigogne a) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.outputJlcm(this.fileObj);
    super.inputJlcm(a.lcmObj);
    super.exec();
  }
  protected void makeText() {
    this.operator="EXCELT: ";
    super.write("EDIT", super.edit);
    super.write("\nTITLE '"); super.write(this.title); super.write("'");
    if (this.maxr != 0) super.write("\nMAXR", this.maxr);
    if (this.nanis != 0) super.write("\nANIS", this.nanis);
    if (this.bc != null) {
      if (this.bc.equals("PISO")) {
        super.write("\nPISO");
      } else if (this.bc.equals("PSPC")) {
        super.write("\nPSPC");
        if (this.cut != 0.0f) super.write("CUT", this.cut);
      } else {
        throw new RuntimeException("invalid type (" + this.bc + ")");
      }
    }
    if (this.trak != null) {
      super.write("\nTRAK");
      if (this.trak.equals("TISO")) {
        super.write("TISO");
        super.write(this.nangl);
        super.write(this.dens);
        if (this.corn != 0.0f) super.write("CORN", this.corn);
        if (this.symm == -1) super.write("NOSY");
        if (this.symm > 0) super.write("SYMM", this.symm);
      } else if (this.trak.equals("TSPC")) {
        super.write("TSPC");
        if (this.medi) super.write("MEDI");
        super.write(this.nangl);
        super.write(this.dens);
      } else {
        throw new RuntimeException("invalid type (" + this.trak + ")");
      }
    }
    super.write(";");
  }
}
