package jargon;

/**
 * The Flux operator is used to compute the multigroup neutron flux.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Flux extends Operator {
  static {
    System.out.println("Flux class loaded");
  }
  static String fr = "definition d'un objet flux";
  static String en = "definition of a flux object";
  private static final long serialVersionUID = 65L;
  private String type;
  /** type of leakage approximation for the flux calculation.
   * The variable <tt>leak[0]</tt> is chosen among the following
   * values:
   * <dl>
   * <dt> <tt>"LKRD"</tt> <dd> the leakage coefficients are recovered
   * from the LCM data structure embedded in the Microlib object. The
   * <tt>"LKRD"</tt> option is not available with the <tt>"ECCO"</tt>
   * and <tt>"HETE"</tt> leakage models.
   * <dt> <tt>"P0"</tt> <dd> the leakage coefficients are calculated
   * using a P0 model
   * <dt> <tt>"P1"</tt> <dd> the leakage coefficients are calculated
   * using a P1 model
   * <dt> <tt>"B0"</tt> <dd> the leakage coefficients are calculated
   * using a B0 model
   * <dt> <tt>"B1"</tt> <dd> the leakage coefficients are calculated
   * using a B1 model
   * <dt> <tt>"B0TR"</tt> <dd> the leakage coefficients are calculated
   * using a B0 model with transport correction.
   * </dl>
   * The variable <tt>leak[1]</tt> is chosen among the following
   * values:
   * <dl>
   * <dt> <tt>"PNLR"</tt> <dd> that the elements of the scattering
   * modified collision probability matrix are multiplied by the adequate
   * non-leakage homogeneous buckling dependent factor.
   * <dt> <tt>"PNL"</tt> <dd> the elements of the collision probability
   * matrix are multiplied by the adequate non-leakage homogeneous buckling
   * dependent factor.
   * <dt> <tt>"SIGS"</tt> <dd> an homogeneous buckling correction is to be
   * applied on the scattering cross section.
   * <dt> <tt>"ALBS"</tt> <dd> an homogeneous buckling contribution is
   * introduced by a group dependent correction of the albedo. This leakage
   * model is restricted to the collision probability method. It is then
   * necessary to define the geometry with an external boundary condition
   * of type <tt>"VOID"</tt> and to close the region in operator Asm using
   * the <tt>"ALBS"</tt> option
   * <dt> <tt>"ECCO"</tt> <dd>  perform an ECCO-type leakage calculation
   * taking into account isotropic streaming effects. This method
   * introduces an heterogeneous buckling contribution as a group dependent
   * correction to the source term. It is then necessary to set the option
   * <tt>"ECCO"</tt> in operator Asm.
   * <dt> <tt>"HETE"</tt> <dd> perform a Tibere-type leakage calculation
   * taking into account anisotropic streaming effects. This method
   * introduces an heterogeneous buckling contribution as a group dependent
   * correction to the source term. The heterogeneous buckling contribution
   * is introduced in the Bn model using directional collision probabilities
   * (PIJK method). It is then necessary to set the option <tt>"PIJK"</tt>
   * in operator Asm.
   * </dl>
   */
  public String[] leak;
  /** choice of the component(s) of the buckling chosen as eigenvalue when
   * <tt>leak[1] = "HETE"</tt>. This variable is chosen among the following
   * values:
   * <dl>
   * <dt> <tt>"G"</tt> <dd> use the 3D modulus of vector <b>B</b> as eigenvalue.
   * <dt> <tt>"R"</tt> <dd> use the 2D modulus in XY plane of vector <b>B</b> as
   * eigenvalue.
   * <dt> <tt>"X"</tt> <dd> use the X-directed component of vector <b>B</b> as
   * eigenvalue.
   * <dt> <tt>"Y"</tt> <dd> use the Y-directed component of vector <b>B</b> as
   * eigenvalue.
   * <dt> <tt>"Z"</tt> <dd> use the Z-directed component of vector <b>B</b> as
   * eigenvalue.
   * </dl>
   */
  public String hete;
  /** value of the fixed or initial total buckling in cm^{-2}.
   */
  public float buckling = 0.0f;
  /** value of the fixed effective multiplication constant. The default value
   * is <tt>valk = 1.0f</tt>.
   */
  public float keff = 0.0f;
  /** maximum number of outer (power) iterations.
   */
  public int outerMax = 0;
  /** convergence criterion for flux components in the outer iteration.
   */
  public float outerEps = 0.0f;
  /** maximum number of thermal (up-scattering) iterations.
   * In Trivac, thermal iterations are generally not required, except in
   * cases with a high number of thermal groups. The Trivac default value
   * is therefore <tt>thermalMax = 0</tt>.
   */
  public int thermalMax = 0;
  /** convergence criterion for flux components in the thermal iteration.
   * In Trivac, in cases where <tt>thermalMax > 0</tt>, the default value
   * of <tt>thermalEps</tt> is set to <tt>1.0e-2f</tt>.
   */
  public float thermalEps = 0.0f;
  /** two-component array containing the variational acceleration
   * parameters. This option is active by default with <tt>livolant[0] = 3</tt>
   * free iterations followed by <tt>livolant[1] = 3</tt> accelerated
   * iterations. 
   */
  public int[] livolant;
  /** number of alternating direction implicit (ADI) inner iterations per
   * outer iteration. The default value is <tt>nadi = 1</tt>. If this value
   * causes a failure of the acceleration process, it is recommended that a
   * larger value be tried. The optimal choice is generally the minimum value
   * of <tt>nadi</tt> which allows a convergence in less than 75 outer
   * iterations. <tt>nadi = 1</tt> or <tt>nadi = 2</tt> is generally the best
   * choice for production-type calculations. The greater <tt>nadi</tt> is,
   * the smaller the asymptotic convergence constant (ACC) becomes. Taking
   * an arbitrary large value (e.g., <tt>nadi = 20</tt>) leads to numerical
   * results identical to those of the inverse power method where the system
   * matrices are accurately inverted at each outer iteration (at a prohibitive
   * CPU cost). In this case, the ACC is almost equal to the dominance ratio
   * of the iterative matrix.
   * <P><b>Trivac-specific variable.</b>
   */
  public int adi;
  /** set to <tt>true</tt> to compute both the direct and adjoint fluxes.
   * Default value is <tt>false</tt>. The adjoint solution is required if
   * we subsequently want to perform a perturbation calculation or a
   * quasistatic transient calculation.
   * <P><b>Trivac-specific variable.</b>
   */
  public boolean adjoint = false;
  /** 
   * Use this constructor to create a new Flux object of specific type.
   * @param name user-defined name of the Flux object
   * @param type type of the iterative process used to compute the flux.
   * This variable is chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"N"</tt> <dd> no flux calculation is to be performed.
   * This option is usually activated when one simply wishes to initialize
   * the neutron flux distribution and to store this information in the
   * embedded LCM structure of the Flux object.
   * <dt> <tt>"S"</tt> <dd> a fixed source problem is to be treated. Such
   * problem can also include fission source contributions.
   * <dt> <tt>"K"</tt> <dd> a fission source eigenvalue problem is to be
   * treated. The eigenvalue is then the effective multiplication factor
   * with a fixed buckling. In this case, the fixed sources, if any is
   * present on the Macrolib or Microlib object, are not used.
   * <dt> <tt>"B"</tt> <dd> a fission source eigenvalue problem is to be
   * treated. The eigenvalue in this case is the critical buckling with a
   * fixed effective multiplication factor. The buckling eigenvalue has
   * meaning only in the case of a cell without leakages. It is also
   * possible to use an open geometry with <tt>"VOID"</tt> boundary
   * conditions  provided it is closed by the Asm operator using the
   * options <tt>"NORM"</tt> or <tt>"ALSB"</tt>.
   * <dt> <tt>"L"</tt> <dd> a non-multiplicative medium eigenvalue problem
   * is to be treated. The eigenvalue in this case is the critical buckling
   * with vanishing fission cross sections. The buckling eigenvalue has
   * meaning only in the case of a cell without leakages. It is also
   * possible to use an open geometry with <tt>"VOID"</tt> boundary
   * conditions  provided it is closed by the Asm operator using the
   * options <tt>"NORM"</tt> or <tt>"ALSB"</tt>.
   * <dt> <tt>"P"</tt> <dd> a fixed-source eigenvalue problem as used in
   * generalized perturbation theory (GPT).
   * </dl></ul>
   */
  public Flux(String name, String type) {
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
    this.type = type;
  }
  /** execute the Flux operator for a transport tracking using a default
   * initial condition.
   */
  public void exec(Asm a, Macrolib b, Tracking c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    if (c.fileObj != null) super.inputJlcm(c.fileObj);
    super.exec();
  }
  /** execute the Flux operator for a transport tracking using the flux embedded
   * in <tt>a</tt> as initial condition.
   */
  public void exec(Flux a, Asm b, Macrolib c, Tracking d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    if (d.fileObj != null) super.inputJlcm(d.fileObj);
    super.exec();
  }
  /** execute the Flux operator for a transport tracking using a default
   * initial condition.
   */
  public void exec(Asm a, Microlib b, Tracking c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    if (c.fileObj != null) super.inputJlcm(c.fileObj);
    super.exec();
  }
  /** execute the Flux operator for a transport tracking using the flux embedded
   * in <tt>a</tt> as initial condition.
   */
  public void exec(Flux a, Asm b, Microlib c, Tracking d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    if (d.fileObj != null) super.inputJlcm(d.fileObj);
    super.exec();
  }
  /** execute the Flux operator for a transport tracking using a default
   * initial condition.
   */
  public void exec(Asm a, Autop b, Tracking c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    if (c.fileObj != null) super.inputJlcm(c.fileObj);
    super.exec();
  }
  /** execute the Flux operator for a transport tracking using the flux embedded
   * in <tt>a</tt> as initial condition.
   */
  public void exec(Flux a, Asm b, Autop c, Tracking d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    if (d.fileObj != null) super.inputJlcm(d.fileObj);
    super.exec();
  }
  /** execute the Flux operator for a transport tracking using a default
   * initial condition.
   */
  public void exec(Macrolib a, Tracking b) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.exec();
  }
  /** execute the Flux operator for a transport tracking using the flux embedded
   * in <tt>a</tt> as initial condition.
   */
  public void exec(Flux a, Macrolib b, Tracking c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.exec();
  }
  /** execute the Flux operator for a Trivac tracking using a default
   * initial condition.
   */
  public void exec(Trivaa a, Tracking b, Macrolib c) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.exec();
  }
  /** execute the Flux operator for a Trivac tracking using the flux embedded
   * in <tt>a</tt> as initial condition.
   */
  public void exec(Flux a, Trivaa b, Tracking c, Macrolib d) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.inputJlcm(b.lcmObj);
    super.inputJlcm(c.lcmObj);
    super.inputJlcm(d.lcmObj);
    super.exec();
  }
  protected void makeText() {
    this.operator="FLU: ";
    boolean tdtflag = false;
    boolean pijflag = false;
    Jlcm[] Jlcmarray = this.getJlcm(false);
    for (int i=1;i<Jlcmarray.length;i++) {
      if ((Jlcmarray[i].getType() == 1) | (Jlcmarray[i].getType() == 2)) {
        String Ssign = (String)Jlcmarray[i].get("SIGNATURE");
        System.out.println("Signature found =" + Ssign);
        if (Ssign.equals("L_TRACK     ")) {
          String Stest = (String)Jlcmarray[i].get("TRACK-TYPE");
          System.out.println("Track type found =" + Stest);
          if (Stest.equals("TRIVAC      ")) this.operator="FLUD: ";
          if (Stest.equals("BIVAC       ")) this.operator="FLUD: ";
          if (Stest.equals("TDT         ")) tdtflag = true;
        }
        if (Ssign.equals("L_PIJ       ")) pijflag = true;
      }
    }
    if (tdtflag & !pijflag) {
      this.operator="FLUTDT: ";
      throw new RuntimeException("TDT not implemented");
    }
    super.write("EDIT", super.edit);
    if (this.operator.equals("FLU: ")) super.write("\nTYPE", this.type);
    if (this.leak != null) {
      super.write(this.leak);
      if ((this.hete != null) & (this.leak[1].equals("HETE"))) super.write(this.hete);
    }
    if ((this.buckling != 0.0f) | ((this.type.equals("K"))&(this.leak != null))) {
      super.write("\nBUCK",this.buckling);
    }
    if (this.keff != 0.0f) super.write("\nKEFF",this.keff);
    if ((this.outerMax != 0) & (this.outerEps != 0)) {
      super.write("\nEXTE",this.outerMax); super.write(this.outerEps);
    } else if ((this.outerMax != 0) & (this.outerEps == 0.0f)) {
      super.write("\nEXTE",this.outerMax);
    } else if ((this.outerMax == 0) & (this.outerEps != 0)) {
      super.write("\nEXTE",this.outerEps);
    }
    if ((this.thermalMax != 0) & (this.thermalEps != 0)) {
      super.write("\nTHER",this.thermalMax); super.write(this.thermalEps);
    } else if ((this.thermalMax != 0) & (this.thermalEps == 0.0f)) {
      super.write("\nTHER",this.thermalMax);
    } else if ((this.thermalMax == 0) & (this.thermalEps != 0)) {
      super.write("\nTHER",this.thermalEps);
    }
    if (this.livolant != null) super.write("\nACCE",this.livolant);
    if (this.adi != 0) super.write("\nADI",this.adi);
    if (this.adjoint) super.write("\nADJ");
    super.write(";");
  }
  /** recover the effective multiplication factor from the embedded LCM
   * structure. The <tt>exec()</tt> method must be called before
   * <tt>getKeff()</tt>.
   */
  public float getKeff() {
    float keff = ((float[])super.lcmObj.get("K-EFFECTIVE"))[0]; // LCM access
    return keff;
  }
  /** recover the buckling from the embedded LCM structure.
   * The <tt>exec()</tt> method must be called before <tt>getB2()</tt>.
   */
  public float getB2() {
    float b2 = ((float[])super.lcmObj.get("B2  B1HOM"))[0]; // LCM access
    return b2;
  }
}
