package jargon;

import java.util.Vector;
import java.util.Enumeration;
import java.io.*;
/**
 * The Gigogne operator is used to create or modify a gigogne geometry.
 * This operator permits all the characteristics (coordinates, region
 * mixture and boundary conditions) of a simple or complex geometry
 * to be specified. The method used to specify the geometry is independent
 * of the discretization operator to be used subsequently. An instance
 * of the Gigogne class can contains other Gigogne instances, in order to
 * create sub-geometries.
 * <P><BR><img src="../../src/figures/gigogne.gif">
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Gigogne extends Operator {
  static {
    System.out.println("Gigogne class loaded");
  }
  static String fr = "definition d'une geometrie gigogne";
  static String en = "definition of a gigogne geometry";
  private static final long serialVersionUID = 20L;
  private int[] ndim;
  private String type;
  private Composition[] compoArray;
  /** type of boundary conditions and/or symmetries in Cartesian geometry.
   * <P><BR><img src="../../src/figures/ebc.gif">
   * <P><BR><img src="../../src/figures/ebcr.gif">
   * <P><BR><img src="../../src/figures/ebct.gif">
   * <P><tt>bc[i][0]</tt> is a keyword identification of side <tt>i</tt>.
   * This variable is chosen among the following values:
   * <dl>
   * <dt> <tt>"X-"</tt> <dd> negative X surface of a Cartesian geometry
   * <dt> <tt>"X+"</tt> <dd> positive X surface of a Cartesian geometry
   * <dt> <tt>"Y-"</tt> <dd> negative Y surface of a Cartesian geometry
   * <dt> <tt>"Y+"</tt> <dd> positive Y surface of a Cartesian geometry
   * <dt> <tt>"Z-"</tt> <dd> negative Z surface of a Cartesian geometry
   * <dt> <tt>"Z+"</tt> <dd> positive Z surface of a Cartesian geometry
   * <dt> <tt>"R+"</tt> <dd> outer surface of a cylindrical or spherical
   * geometry
   * </dl>
   * <P><tt>bc[i][1]</tt> is a keyword identification of the boundary
   * condition or symmetry condition on side <tt>i</tt>. This variable
   * is chosen among the following values:
   * <dl>
   * <dt> <tt>"VOID"</tt> <dd> the surface under consideration has zero
   * re-entrant angular flux. This side is an external surface of the domain.
   * <dt> <tt>"REFL"</tt> <dd> the surface under consideration has a
   * reflective specular boundary condition.
   * <dt> <tt>"DIAG"</tt> <dd> the Cartesian surface under consideration has
   * the same properties as that associated with a diagonal through the
   * geometry. Note that two and only two <tt>DIAG</tt> surfaces must be
   * specified. The diagonal symmetry is only permitted for square geometry
   * and in the following combinations: <BR><tt>{"X+", "DIAG"},{"Y-", "DIAG"}</tt>
   * and <tt>{"X-", "DIAG"},{"Y+", "DIAG"}</tt>.
   * <dt> <tt>"TRAN"</tt> <dd> the surface under consideration is connected
   * to the opposite surface of a Cartesian domain. This option provides
   * the capability to treat an infinite geometry with translation symmetry.
   * The only combinations of translational symmetry permitted are:
   * <BR><tt>{"X-", "TRAN"},{"X+", "TRAN"}</tt>, <tt>{"Y-", "TRAN"},{"Y+", "TRAN"}</tt>
   * and <tt>{"Z-", "TRAN"},{"Z+", "TRAN"}</tt>.
   * <dt> <tt>"SYME"</tt> <dd> the Cartesian surface under consideration is
   * virtual and that a reflection symmetry is associated with the
   * adequately directed axis running through the center of the cells
   * closest to this surface.
   * <dt> <tt>"ALBE"</tt> <dd> the surface under consideration has an
   * isotropic reflection with arbitrary albedo. This side is an external
   * surface of the domain.
   * <dt> <tt>"ZERO"</tt> <dd> the surface under consideration has a
   * zero-flux boundary condition. This side is an external surface of
   * the domain.
   * <dt> <tt>"PI"</tt> <dd> the surface under consideration has a 180
   * degree rotational symmetry.
   * <dt> <tt>"PI/2"</tt> <dd> the surface under consideration has a 90
   * degree rotational symmetry. The only <tt>"PI/2"</tt> symmetry permitted
   * is related to sides <tt>X-</tt> and <tt>Y-</tt>. This condition can be
   * combined with a translation boundary condition:
   * <BR><tt>{"X-", "PI/2"},{"X+", "TRAN"}</tt> and/or
   * <tt>{"Y-", "PI/2"},{"Y+", "TRAN"}</tt>.
   * </dl>
   */
  public String[][] bc; //into=('X-','X+','Y-','Y+','Z-','Z+','R+')
                        //into=('VOID','REFL','DIAG','TRAN','SYME',
                        //'ALBE','ZERO','PI','PI/2')
  /** type of boundary conditions and symmetries in hexagonal geometry.
   * <P>
   * <BR><img src="../../src/figures/Gs30.gif">
   * <BR><img src="../../src/figures/Gsb60.gif">
   * <BR><img src="../../src/figures/Gr120.gif">
   * <BR><img src="../../src/figures/Gsa180.gif">
   * <BR><img src="../../src/figures/Gsb180.gif">
   * <P><BR><img src="../../src/figures/Gcomplete.gif">
   * <P><tt>hbc[0]</tt> is a keyword identification of the symmetry
   * condition of the hexagonal domain. This variable is chosen among
   * the following values:
   * <dl>
   * <dt> <tt>"S30"</tt> <dd> hexagonal symmetry of one twelfth of an
   * assembly.
   * <dt> <tt>"SA60"</tt> <dd> hexagonal symmetry of one sixth of an
   * assembly of type A.
   * <dt> <tt>"SB60"</tt> <dd>  hexagonal symmetry of one sixth of an
   * assembly of type B.
   * <dt> <tt>"S90"</tt> <dd> hexagonal symmetry of one quarter of an
   * assembly
   * <dt> <tt>"R120"</tt> <dd> rotation symmetry of one third of an
   * assembly .
   * <dt> <tt>"R180"</tt> <dd> rotation symmetry of a half assembly.
   * <dt> <tt>"SA180"</tt> <dd> hexagonal symmetry of half a type A
   * assembly
   * <dt> <tt>"SB180"</tt> <dd> hexagonal symmetry of half a type B
   * assembly
   * <dt> <tt>"COMPLETE"</tt> <dd> complete assembly
   * </dl>
   * <P><tt>hbc[1]</tt> is a keyword identification of the boundary
   * condition surrounding the hexagonal domain. This variable is chosen
   * among the following values:
   * <dl>
   * <dt> <tt>"VOID"</tt> <dd> the surface under consideration has zero
   * re-entrant angular flux. This side is an external surface of the domain.
   * <dt> <tt>"REFL"</tt> <dd> the surface under consideration has a
   * reflective specular boundary condition.
   * <dt> <tt>"SYME"</tt> <dd> the surface under consideration is
   * virtual and that a reflection symmetry is associated with the
   * adequately directed axis running through the center of the cells
   * closest to this surface. Only the hexagonal geometries <tt>S30</tt>
   * and <tt>SA60</tt> can be surrounded by a <tt>SYME</tt> boundary condition
   * <dt> <tt>"ALBE"</tt> <dd> the surface under consideration has an
   * arbitrary albedo. This side is an external surface of the domain.
   * <dt> <tt>"ZERO"</tt> <dd> the surface under consideration has a
   * zero-flux boundary condition. This side is an external surface of
   * the domain.
   * </dl>
   */
  public String[] hbc; //into=('S30','SA60','SB60','S90','R120',
                       //'R180','SA180','SB180','COMPLETE')
                       //into=('VOID','REFL','SYME','ALBE','ZERO')
  /** geometric albedo corresponding to each side
   */
  public float[] albedo;
  /** spatial mesh defining the regions along the X-axis
   */
  public float[] meshx;
  /** spatial mesh defining the regions along the Y-axis
   */
  public float[] meshy;
  /** spatial mesh defining the regions along the Z-axis
   */
  public float[] meshz;
  /** spatial mesh defining the regions along the radial direction
   */
  public float[] radius;
  /** number of zones that will be considered for each region along the
   * X-axis
   */
  public int[] splitx;
  /** number of zones that will be considered for each region along the
   * Y-axis
   */
  public int[] splity;
  /** number of zones that will be considered for each region along the
   * Z-axis
   */
  public int[] splitz;
  /** number of zones that will be considered for each region along the
   * radial direction. A negative value results in a splitting of the
   * regions into zones of equal volumes; a positive value results in a
   * uniform splitting along the radial direction.
   */
  public int[] splitr;
  /** length of a side of a hexagon.
   */
  public float hside;
  /** three-value array containing the displacement of the cylindric regions
   * from the center of a Cartesian cell.
   * <P><img src="../../src/figures/decentre.gif">
   */
  public float[] offcenter;
  /** Composition object containing an isotopic mixture associated with
   * each region inside the geometry. A Microlib or Macrolib object <b>must</b> be provided
   * as an argument to the <tt>exec()</tt> method. This instance variable is provided
   * as a replacement for <tt>mix</tt> in order to use real mediums and to
   * avoid using error-prone integer indices to represent media. A <tt>null</tt>
   * object can be used in empty (or virtual) locations.
   * When diagonal symmetries are considered, only the mixtures associated
   * with regions inside the symmetrized geometry need to be specified.
   */
  public Composition[] media;
  /** isotopic mixture index associated with each region inside the geometry (obsolete).
   * When diagonal symmetries are considered, only the mixtures associated
   * with regions inside the symmetrized geometry need to be specified.
   * This instance variable is obsolete.
   */
  public int[] mix;
  /** sub-geometry associated with each region inside the geometry.
   * When diagonal symmetries are considered, only the sub-geometries associated
   * with regions inside the symmetrized geometry need to be specified. When
   * a sub-geometry is located inside symmetrized geometry but outside the
   * calculation region it must be declared <i>virtual</i> (for example,
   * the corners of a nuclear reactor).
   */
  public Gigogne[] subgeo;
  /** array of numbers that associate a global sub-geometry or region
   * number with each sub-geometry or region. All the sub-geometries or
   * regions with the same global number will be attributed the same flux.
   */
  public int[] merge;
  /** orientation of the cells in a Cartesian or hexagonal assembly.
   * The letters <tt>A</tt> to <tt>L</tt> are used as keywords to specify
   * these rotation. For Cartesian geometries, the eight possible
   * orientations are shown in the following figure:
   * <P><img src="../../src/figures/Goricart.gif">
   * <P>For hexagonal geometries, the permitted orientations are shown in
   * the following figure:
   * <P><img src="../../src/figures/Gorihex.gif">
   * <P>For 3-D cells, the same letters can be used to describe the rotation
   * in the X-Y plane. However, an additional <tt>-</tt> sign can be glued
   * to the 2-D rotation identifier to indicate reflection of the cell
   * along the Z-axis (<tt>A</tt> to <tt>L</tt>).
   */
  public String[] turn;
  /** sectorization index. If <b>|sect|</b> > 100, the sectorization does not
   * split the embedded tubes.
   * <dl>
   * <dt> -999 <dd> non-sectorized cell processed as a sectorized cell
   * <dt> -1 or -101 <dd> <tt>X</tt>-type sectorization
   * <dt> 0 <dd> non-sectorized cell
   * <dt> 1 or 101 <dd> +-type sectorization
   * <dt> 2 or 102 <dd> simultaneous <tt>X</tt>- and +-type sectorization
   * </dl>
   * <BR><img src="../../src/figures/rect3b.gif">
   * <BR><img src="../../src/figures/hexa3.gif">
   */
  public int sect;
  private Cluster cluster;
  /** object containing double-heterogeneity media and information
   * relative to the double-heterogeneity model. A medium made of
   * spherical or cylindrical micro-structures can be inserted into
   * the current (macro) geometry. Each type of micro-structure is
   * characterized by its dimension and may have distinct volumetric
   * concentrations in each of the macro-geometry volumes. All the
   * micro-structures of a given type have the same nuclear properties
   * in a given macro-volume. The micro-structures of a given type
   * may have different nuclear properties within different macro-volumes.
   */
  public Bihet bihet = new Bihet();
  
  private class Cluster implements Serializable {
    private static final long serialVersionUID = 21L;
    private Gigogne[] subgeo;
    private int[] npin;
    private float[] rpin;
    private float[] apin;
    private float[] dpin;
  }
  /** class used in the definition of double heterogeneity mixtures
   */
  public class Bihet implements Serializable {
    private static final long serialVersionUID = 22L;
    private String type;
    private float[][] radius;
    private Mixture[] mixture;
    private class Mixture implements Serializable {
    private static final long serialVersionUID = 23L;
      private Composition cmix;
      private Composition cmixdil;
      private float[] fract;
      private Composition[][] cmixgr;
    }
    /** set the geometric type of micro structures.
     * @param type type of geometry. This variable is chosen among the
     * following values:
     * <ul><dl>
     * <dt> <tt>"TUBE"</tt> <dd> tubular micro structures
     * <dt> <tt>"SPHE"</tt> <dd> spherical micro structures
     * </dl></ul>
     */
    public void setGeometry(String type) {
      this.type = type;
    }
    /** set a single double-heterogeneity media. This method is called
     * for each double-heterogeneity media.
     * @param cmix composite medium associated with a
     * double-heterogeneity medium.
     * @param cmixdil diluant medium associated with a
     * double-heterogeneity medium. This is the medium surrounding
     * the micro-structures.
     * @param fract volumetric concentration (VG/VR) of each
     * micro-structure type (VG) in the double-heterogeneity
     * medium (VR).
     * @param cmixgr media associated with each region
     * of the micro-structures (inner loop) for each type of
     * micro-structure (outer loop) with a non-zero volumetric
     * concentration.
     */
    public void setMixture(Composition cmix, Composition  cmixdil, float[] fract, Composition[][] cmixgr) {
      if (this.mixture == null) {
        this.mixture = new Mixture[1];
      } else {
        Mixture[] work = this.mixture;
        this.mixture = new Mixture[work.length+1];
        for (int i=0;i<work.length;i++) this.mixture[i] = work[i];
      }
      this.mixture[this.mixture.length-1] = new Mixture();
      this.mixture[this.mixture.length-1].cmix = cmix;
      this.mixture[this.mixture.length-1].cmixdil = cmixdil;
      this.mixture[this.mixture.length-1].fract = fract;
      this.mixture[this.mixture.length-1].cmixgr = cmixgr;
    }
    /** define the radius for a single type of micro-structure. This
     * method is called for each type of micro-structure.
     * @param radius radius of the tubes or spherical shells making
     * up the micro-structures. We must have <tt>radius[0]=0.0f</tt>.
     */
    public void setRadius(float[] radius) {
      if (this.radius == null) {
        this.radius = new float[1][];
      } else {
        float[][] work = this.radius;
        this.radius = new float[work.length+1][];
        for (int i=0;i<work.length;i++) this.radius[i] = work[i];
      }
      this.radius[this.radius.length-1] = radius;
    }
  }
  /** 
   * Use this constructor to create a Gigogne object of a specified type.
   * @param name user-defined name of the Gigogne object
   * @param type type of geometry. This variable is chosen among the
   * following values:
   * <ul><dl>
   * <dt> <tt>"VIRTUAL"</tt> <dd> virtual geometry. This type of geometry is
   * used to complete an assembly that has irregular boundaries.
   * <dt> <tt>"HOMOGE"</tt> <dd> infinite homogeneous geometry
   * <dt> <tt>"CAR1D"</tt> <dd> one dimensional plane geometry (infinite slab)
   * <dt> <tt>"TUBE"</tt> <dd> cylindrical geometry (infinite tubes or cylinders)
   * <dt> <tt>"SPHERE"</tt> <dd>  spherical geometry (concentric spheres)
   * <dt> <tt>"CAR2D"</tt> <dd> two-dimensional Cartesian geometry
   * <dt> <tt>"TUBEZ"</tt> <dd> polar R-Z geometry
   * <dt> <tt>"CAR3D"</tt> <dd> three-dimensional Cartesian geometry
   * <dt> <tt>"HEX"</tt> <dd> two-dimensional hexagonal geometry
   * <dt> <tt>"HEXZ"</tt> <dd> three-dimensional hexagonal geometry
   * <dt> <tt>"CARCEL"</tt> <dd> two-dimensional mixed Cartesian cell (concentric
   * tubes surrounded by a rectangle)
   * <dt> <tt>"CARCELX"</tt> <dd> three-dimensional mixed Cartesian cell with
   * tubes oriented along the X axis
   * <dt> <tt>"CARCELY"</tt> <dd>  three-dimensional mixed Cartesian cell with
   * tubes oriented along the Y axis
   * <dt> <tt>"CARCELZ"</tt> <dd>  three-dimensional mixed Cartesian cell with
   * tubes oriented along the Z axis
   * <dt> <tt>"HEXCEL"</tt> <dd> two-dimensional mixed hexagonal cell (concentric
   * tubes surrounded by a hexagon)
   * <dt> <tt>"HEXCELZ"</tt> <dd> three-dimensional mixed hexagonal cell with
   * tubes oriented along the Z axis
   * <dt> <tt>"GROUP"</tt> <dd> <i>do-it-yourself</i> type geometry
   * <P><img src="../../src/figures/doit.gif">
   * </dl></ul>
   * @param ndim one-, two- or three-value array containing the size of
   * the geometry along each dimension.
   */
  public Gigogne(String name, String type, int[] ndim) {
    this.lcmObj = new Jlcm("LCM", name);
    super.name = name;
    if ((!type.equals("VIRTUAL"))&(!type.equals("HOMOGE"))&
       (!type.equals("CAR1D"))&(!type.equals("TUBE"))&
       (!type.equals("SPHERE"))&(!type.equals("CAR2D"))&
       (!type.equals("TUBEZ"))&(!type.equals("CAR3D"))&
       (!type.equals("HEX"))&(!type.equals("HEXZ"))&
       (!type.equals("HEXCEL"))&(!type.equals("HEXCELZ"))&
       (!type.equals("CARCEL"))&(!type.equals("CARCELX"))&
       (!type.equals("CARCELY"))&(!type.equals("CARCELZ"))&
       (!type.equals("GROUP"))) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.type = type;
    this.ndim = ndim;
  }
  
  /** print the operator data in a form compatible with REDGET. Use this call
   * if instance variable <tt>media</tt> is used; otherwise, use the inherited call.
   * @param a existing Microlib object containing the media used by the
   * Gigogne object
   */
  public void text(Microlib a) throws JargonException {
    this.compoArray = a.mixs;
    super.text();
  }
  /** print the operator data in a form compatible with REDGET. Use this call
   * if instance variable <tt>media</tt> is used; otherwise, use the inherited call.
   * @param a existing Macrolib object containing the media used by the
   * Gigogne object
   */
  public void text(Macrolib a) throws JargonException {
    this.compoArray = a.mixs;
    super.text();
  }
  /** load the embedded lcmObj object with the content of this Gigogne.
   */
  public void exec() throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.exec();
  }  
  /** load the embedded lcmObj object with the content of an existing
   * Gigogne.
   * @param a existing Gigogne object
   */
  public void exec(Gigogne a) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.exec();
  }
  /** load the embedded lcmObj object with the content of this Gigogne. Use
   * this call if instance variable <tt>media</tt> is used.
   * @param a existing Microlib object containing the media used by the
   * Gigogne object
   */
  public void exec(Microlib a) throws JargonException {
    this.compoArray = a.mixs;
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.exec();
  }  
  /** load the embedded lcmObj object with the content of this Gigogne. Use
   * this call if instance variable <tt>media</tt> is used.
   * @param a existing Macrolib object containing the media used by the
   * Gigogne object
   */
  public void exec(Macrolib a) throws JargonException {
    this.compoArray = a.mixs;
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.exec();
  }  
  /** load the embedded lcmObj object with the content of an existing
   * Gigogne. Use this call if instance variable <tt>media</tt> is used.
   * @param a existing Gigogne object
   * @param b existing Microlib object containing the media used by the
   * Gigogne object
   */
  public void exec(Gigogne a, Microlib b) throws JargonException {
    this.compoArray = b.mixs;
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.exec();
  }
  /** load the embedded lcmObj object with the content of an existing
   * Gigogne. Use this call if instance variable <tt>media</tt> is used.
   * @param a existing Gigogne object
   * @param b existing Macrolib object containing the media used by the
   * Gigogne object
   */
  public void exec(Gigogne a, Macrolib b) throws JargonException {
    this.compoArray = b.mixs;
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.exec();
  }
  /** define a cluster.
   * <P>
   * <BR><img src="../../src/figures/cluster.gif">
   * @param subgeo sub-geometry used to make the cluster
   * @param npin number of pins associated with this sub-geometry in the
   *             cluster
   * @param rpin radius of an imaginary cylinder where the centers of the
   *             pins are to be placed
   * @param apin angle (radian) of the first pin centered on an imaginary
   *        cylinder in a cluster. It can only be used for <tt>"TUBE"</tt>
   *        sub-geometry.
   */
  public void setCluster(Gigogne[] subgeo, int[] npin, float[] rpin,
  float[] apin) {
    this.cluster = new Cluster();
    this.cluster.subgeo = subgeo;
    this.cluster.npin = npin;
    this.cluster.rpin = rpin;
    this.cluster.apin = apin;
    this.cluster.dpin = null;
  }
  /** define a cluster with non unigorm angular separation between the pins
   * @param dpin angular separation (radian) between the pins in the ring
   *             centered on an imaginary cylinder in a cluster. It can only
   *             be used for <tt>"TUBE"</tt> sub-geometry.
   */
  public void setCluster(Gigogne[] subgeo, int[] npin, float[] rpin,
  float[] apin, float[] dpin) {
    this.cluster = new Cluster();
    this.cluster.subgeo = subgeo;
    this.cluster.npin = npin;
    this.cluster.rpin = rpin;
    this.cluster.apin = apin;
    this.cluster.dpin = dpin;
  }

  protected void makeText() {
    this.operator="GEO: ";
    this.makeTextOpen();
    super.write(";");
  }
  
  private void makeTextOpen() {
    super.write(this.type, this.ndim);
    super.write("EDIT", super.edit);
    if (this.bihet.type != null) {
      super.write("\nBIHET", this.bihet.type);
      super.write("(*NG=*)", this.bihet.radius.length);
      super.write("(*NMILG=*)", this.bihet.mixture.length);
      super.write("\n  (*NS=*)");
      for (int i=0; i<this.bihet.radius.length; i++) {
        super.write(this.bihet.radius[i].length-1);
      }
      for (int i=0; i<this.bihet.radius.length; i++) {
        super.write("\n  (*RADIUS" + i + "=*)", this.bihet.radius[i]);
      }
      super.write("\n  (*COMPOSITE MIXTURES*)");
      for (int i=0; i<this.bihet.mixture.length; i++) {

        if (this.compoArray == null) {
          throw new RuntimeException("(1)missing Macrolib or Microlib in exec() statement");
        }
        Composition[] work = this.compoArray;
        this.compoArray = new Composition[work.length+1];
        for (int j=0;j<work.length;j++) this.compoArray[j] = work[j];
        this.compoArray[work.length] = this.bihet.mixture[i].cmix;

        int imix = 0;
        for (int j=0;j<this.compoArray.length;j++) {
          if (this.bihet.mixture[i].cmix.name.equals(this.compoArray[j].name)) imix = j+1;
        }
        if (imix == 0) {
          throw new RuntimeException("(1)unable to find medium " + this.bihet.mixture[i].cmix.name);
        }
        super.write(imix);
      }
      super.write("\n  (*MIXTURES SURROUNDING M-S*)");
      for (int i=0; i<this.bihet.mixture.length; i++) {
        int imix = 0;
        if (this.compoArray == null) {
          throw new RuntimeException("(2)missing Macrolib or Microlib in exec() statement");
        }
        for (int j=0;j<this.compoArray.length;j++) {
          if (this.bihet.mixture[i].cmixdil.name.equals(this.compoArray[j].name)) imix = j+1;
        }
        if (imix == 0) {
          throw new RuntimeException("(2)unable to find medium " + this.bihet.mixture[i].cmixdil.name);
        }
        super.write(imix);
      }
      for (int i=0; i<this.bihet.mixture.length; i++) {
        Bihet.Mixture item = this.bihet.mixture[i];
        int imix = 0;
        if (this.compoArray == null) {
          throw new RuntimeException("(3)missing Macrolib or Microlib in exec() statement");
        }
        for (int j=0;j<this.compoArray.length;j++) {
          if (this.bihet.mixture[i].cmix.name.equals(this.compoArray[j].name)) imix = j+1;
        }
        super.write("\n  (*COMPOSITE MIXTURE NB.",imix); super.write("*)");
        super.write("\n    (*VOLUMIC FRACTIONS*)",item.fract);
        for (int j=0; j<item.cmixgr.length; j++) {
          super.write("\n    (*REAL MIXTURE" + j + " CONTENT*)");
          for (int k=0; k<item.cmixgr[j].length; k++) {
            int jmix = 0;
            for (int n=0;n<this.compoArray.length;n++) {
              if (item.cmixgr[j][k].name.equals(this.compoArray[n].name)) jmix = n+1;
            }
            if (jmix == 0) {
              throw new RuntimeException("(3)unable to find medium " + item.cmixgr[j][k].name);
            }
            super.write(jmix);
          }
        }
      }
    }
    if (this.bc != null) {
      super.write("\n", this.bc);
    }
    if (this.meshx != null) {
      super.write("\nMESHX", this.meshx);
    }
    if (this.meshy != null) {
      super.write("\nMESHY", this.meshy);
    }
    if (this.meshz != null) {
      super.write("\nMESHZ", this.meshz);
    }
    if (this.radius != null) {
      super.write("\nRADIUS", this.radius);
    }
    if (this.splitx != null) {
      super.write("\nSPLITX", this.splitx);
    }
    if (this.splity != null) {
      super.write("\nSPLITY", this.splity);
    }
    if (this.splitz != null) {
      super.write("\nSPLITZ", this.splitz);
    }
    if (this.splitr != null) {
      super.write("\nSPLITR", this.splitr);
    }
    if (this.hside != 0.0f) {
      super.write("\nSIDE",this.hside);
    }
    if (this.offcenter != null) {
      super.write("\nOFFCENTER",this.offcenter);
    }
    if (this.mix != null) {
      super.write("\nMIX", this.mix);
    } else if (this.media != null) {
      super.write("\nMIX");
      for (int i=0;i<this.media.length;i++) {
        int imix = 0;
        if (this.media[i] != null) {
          if (this.compoArray == null) {
            throw new RuntimeException("(4)missing Macrolib or Microlib in exec() statement");
          }
          for (int j=0;j<this.compoArray.length;j++) {
            if (this.media[i].name.equals(this.compoArray[j].name)) imix = j+1;
          }
          if (imix == 0) {
            throw new RuntimeException("unable to find medium " + this.media[i].name);
          }
        }
        super.write(imix);
      }
    } else if (this.subgeo != null) {
      super.write("\nMIX");
      Vector<Gigogne> geo_list = new Vector<Gigogne>();
      for (int i=0;i<this.subgeo.length;i++) outer: {
        super.write(this.subgeo[i].name);
        for (int j=1;j<geo_list.size();j++) {
          if (geo_list.elementAt(j) == this.subgeo[i]) break outer;
        }
        geo_list.addElement(this.subgeo[i]);
      }
      Enumeration enum0 = geo_list.elements();
      while(enum0.hasMoreElements()) {
        Gigogne geo = (Gigogne)enum0.nextElement();
        System.out.println("sub-geometry name=" + geo.name);
        geo.compoArray = this.compoArray;
        super.write("\n:::", geo.name);
        super.write(":= GEO:");
        geo.makeText();
      }
    } else {
      throw new RuntimeException("mix and subgeo are both null");
    }
    if (this.merge != null) super.write("\nMERGE", this.merge);
    if (this.turn != null) super.write("\nTURN", this.turn);
    if (this.sect != 0) super.write("\nSECT", this.sect);
    if (this.cluster != null) {
      super.write("\nCLUSTER");
      for (int i=0;i<this.cluster.subgeo.length;i++) super.write(this.cluster.subgeo[i].name);
      for (int i=0;i<this.cluster.subgeo.length;i++) {
        Gigogne geo = this.cluster.subgeo[i];
        super.write("\n:::", geo.name);
        super.write(":= GEO:");
        geo.makeTextOpen();
        super.write("\nNPIN", this.cluster.npin[i]);
        super.write("RPIN", this.cluster.rpin[i]);
        super.write("APIN", this.cluster.apin[i]);
        if (this.cluster.dpin != null) super.write("DPIN", this.cluster.dpin[i]);
        super.write(";");
      }
    }
  }
}
