package jargon;

import java.io.*;
/**
 * This class defines the physical properties of a single isotope,
 * as required in a lattice calculation.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Isotope implements Serializable, Cloneable {
  static {
    System.out.println("Isotope class loaded");
  }
  static String fr = "definition d'un isotope";
  static String en = "definition of an isotope";
  private static final long serialVersionUID = 10L;
  protected String name;
  protected String symbol;
  protected String libName;
  protected String lib;
  protected String fil;
  protected String shib = null;
  /** group independent microscopic dilution cross section (in barns) of
   * the isotope in this mixture. It is possible to recalculate a group
   * dependent dilution for an isotope by the use of the Autop operator.
   * In this case, <tt>dilution</tt> is only used as a starting point for
   * the self-shielding iterations and has no effect on the final result.
   * If the dilution is not given or is larger than <tt>1.0e10f</tt> barns,
   * an infinite dilution is assumed.
   */
  public float dilution = 1.0e10f;
  /** index of the resonant region associated with this isotope. By
   * default <tt>inrs = 0</tt> and the isotope is not a candidate for
   * self-shielding. When <tt>inrs != 0</tt>, the isotope can be self-shielded
   * where it is assumed that a given isotope distributed with different
   * concentrations in a number of mixtures and having the same value of
   * <tt>inrs</tt> will share the same fine flux. Should we wish to
   * self-shield both the clad and the fuel it is important to assign a
   * different <tt>inrs</tt> number to each. If a single type of fuel is
   * located in different mixture in <i>onion-peel fashion</i>, it is
   * necessary to attribute a single <tt>inrs</tt> value to this fuel.
   */
  public int inrs;
  /** set to <tt>true</tt> to compute the mutual resonance shielding
   * effects with respects to other isotopes sharing the same <tt>inrs</tt>
   * instance variable. This capability is only available with the
   * <tt>"Ribox"</tt> self-shielding model.
   */
  public boolean corr = false;
  /** set to <tt>true</tt> to force a nuclide to be non-depleting (even
   * in cases where it is potentially depleting). By default, the depleting
   * isotopes are automatically regognized as depleting.
   */
  public boolean noev = false;
  protected Irset irset = null;
  
  protected class Irset implements Serializable, Cloneable {
    private static final long serialVersionUID = 11L;
    protected float gC;
    protected int group;
    
    public Object clone() {
      try {
        return super.clone();
      } catch (CloneNotSupportedException e) {
        throw new InternalError();
      }
    }
  }
  /** use this constructor to define a new isotope without self-shielding
   * information.
   * @param name user-defined name for the isotope instance. This value
   * is generally identical to the variable name used to represent the
   * isotope instance.
   * @param symbol user-defined name common to all instances of the same
   * isotope. For example: <tt>"U235"</tt> or <tt>"AM242M"</tt>. This name
   * is always the same for an isotope of the nuclide chart (even in cases
   * where there are many instances of this isotope in the calculation).
   * @param libName name for the isotope used in the cross section library.
   * For a given isotope, this name may be different in different libraries.
   * @param lib type of cross section library. This variable is chosen
   * among the following values:
   * <ul><dl>
   * <dt> <tt>"DRAGON"</tt> <dd> the isotopic depletion chain and/or the
   * microscopic cross sections are in the DRAGLIB format.
   * <dt> <tt>"MATXS"</tt> <dd> the microscopic cross sections are in the
   * MATXS format of NJOY-II and NJOY-89 (no depletion data available for
   * libraries using this format).
   * <dt> <tt>"MATXS2"</tt> <dd> the microscopic cross sections are in the
   * MATXS format of NJOY-91 (no depletion data available for libraries using
   * this format).
   * <dt> <tt>"WIMSD4"</tt> <dd> the isotopic depletion chain and/or the
   * microscopic cross sections are in the WIMSD4 format.
   * <dt> <tt>"WIMSAECL"</tt> <dd> the isotopic depletion chain and/or the
   * microscopic cross sections are in the WIMS-AECL format.
   * <dt> <tt>"APLIB1"</tt> <dd> the microscopic cross sections are in the
   * APOLLO-1 format. There are no depletion chains available for libraries
   * using this format.
   * <dt> <tt>"APLIB2"</tt> <dd> that the microscopic cross sections are
   * in the APOLLO-2 direct access format. There are no depletion chains
   * available for libraries using this format. However, fission yields,
   * radioactive decay constants and energy released per fission or radiative
   * capture are recovered from the file. Only versions of the APOLIB-2
   * libraries subsequent or equal to CEA93-V4 can be processed.
   * </dl></ul>
   * @param fil file name of the cross section library.
   */
  public Isotope(String name, String symbol, String libName, String lib,
                  String fil) {
    this(name, symbol, libName, lib, fil, null);
  }
  /** use this constructor to define a new isotope with self-shielding
   * information.
   * @param name user-defined name for the isotope instance. This value
   * is generally identical to the variable name used to represent the
   * isotope instance.
   * @param symbol user-defined name common to all instances of the same
   * isotope. For example: <tt>"U235"</tt> or <tt>"AM242M"</tt>. This name
   * is always the same for an isotope of the nuclide chart (even in cases
   * where there are many instances of this isotope in the calculation).
   * @param libName name for the isotope used in the cross section library.
   * For a given isotope, this name may be different in different libraries.
   * @param lib type of cross section library. This variable is chosen
   * among the following values:
   * <ul><dl>
   * <dt> <tt>"DRAGON"</tt> <dd> the isotopic depletion chain and/or the
   * microscopic cross sections are in the DRAGLIB format.
   * <dt> <tt>"MATXS"</tt> <dd> the microscopic cross sections are in the
   * MATXS format of NJOY-II and NJOY-89 (no depletion data available for
   * libraries using this format).
   * <dt> <tt>"MATXS2"</tt> <dd> the microscopic cross sections are in the
   * MATXS format of NJOY-91 (no depletion data available for libraries using
   * this format).
   * <dt> <tt>"WIMSD4"</tt> <dd> the isotopic depletion chain and/or the
   * microscopic cross sections are in the WIMSD4 format.
   * <dt> <tt>"WIMSAECL"</tt> <dd> the isotopic depletion chain and/or the
   * microscopic cross sections are in the WIMS-AECL format.
   * <dt> <tt>"APLIB1"</tt> <dd> the microscopic cross sections are in the
   * APOLLO-1 format. There are no depletion chains available for libraries
   * using this format.
   * <dt> <tt>"APLIB2"</tt> <dd> that the microscopic cross sections are
   * in the APOLLO-2 direct access format. There are no depletion chains
   * available for libraries using this format. However, fission yields,
   * radioactive decay constants and energy released per fission or radiative
   * capture are recovered from the file. Only versions of the APOLIB-2
   * libraries subsequent or equal to CEA93-V4 can be processed.
   * </dl></ul>
   * @param fil file name of the cross section library.
   * @param shib name of the self-shielded isotope corresponding to the
   * main isotope <tt>libName</tt>. This variable is useful with cross
   * section libraries that store self-shielding information as distinct
   * pseudo-isotopes.
   */
  public Isotope(String name, String symbol, String libName, String lib,
                  String fil, String shib) {
    if ((!lib.equals("DRAGON"))&(!lib.equals("MATXS"))&
       (!lib.equals("MATXS2"))&(!lib.equals("WIMSD4"))&
       (!lib.equals("WIMSAECL"))&(!lib.equals("APLIB1"))&
       (!lib.equals("APLIB2"))) {
      throw new RuntimeException("invalid type (" + lib + ")");
    }
    this.name = name;
    this.symbol = symbol;
    this.libName = libName;
    this.lib = lib;
    this.fil = fil;
    this.shib = shib;
  }
  /** implementation of the Cloneable interface.
   */
  public Object clone() {
    try {
      Isotope o = (Isotope)super.clone();
      if(this.irset != null) o.irset = (Irset)this.irset.clone();
      return o;
    } catch (CloneNotSupportedException e) {
      throw new InternalError();
    }
  }
  /** set the Goldstein-Cohen parameters.
   * @param gC imposed Goldstein-Cohen IR parameter.
   * @param group the Goldstein-Cohen IR parameter <tt>gC</tt> is imposed
   * for energy groups with an index equal or greater than <tt>group</tt>.
   * A value of 1.0 is set in other groups. A value of 1.0 stands for
   * a statistical (ST) approximation. A value of 0.0 stands for an
   * infinite mass (IM or WR) approximation. By default, the value found
   * on the library is used. If no value is found on the library, a value
   * of 1.0 is used.
   */
  public void setIrset(float gC, int group) {
    this.irset = new Irset();
    this.irset.gC = gC;
    this.irset.group = group;
  }
  /** define the energy groups where the <tt>"Ribox"</tt>
   * self-shielding model is used.
   * @param group the <tt>"Ribox"</tt> self-shielding model is imposed
   * for energy groups with an index equal or greater than <tt>group</tt>.
   * Physical probability tables are used in other groups.
   */
  public void setRibox(int group) {
    this.irset = new Irset();
    this.irset.gC = -999.0f;
    this.irset.group = group;
  }
  /** Physical probability tables are used in all groups.
   */
  public void setRibox() {
    this.irset = new Irset();
    this.irset.gC = -999.0f;
    this.irset.group = -1;
  }
}
