package jargon;

import javax.swing.*;
import javax.swing.filechooser.*;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.awt.*;
import java.net.*;
import java.awt.event.*;
import bsh.*;
import bsh.util.*;
import java.util.zip.*;
import java.util.*;
import java.lang.reflect.*;
import java.util.Vector;

/** This class is a JFC/Swing based window for executing computational
 * schemes in the Jargon framework.
 * <p>
 * This class can be instanciated with the following Java program:
 * <PRE>
 * import java.awt.*;
 * import jargon.*;
 * public class Jargon {
 *   static JargonFrame window = new JargonFrame("Java Remote Dragon");
 *   public static void main(String[] args) {
 *     Toolkit theKit = window.getToolkit();        // Get the window toolkit
 *     Dimension wndSize = theKit.getScreenSize();  // Get screen size
 *     // Set the position to screen center
 *     int sizex = 400;
 *     int sizey = 260;
 *     window.setBounds((wndSize.width-sizex)/2, (wndSize.height-sizey)/2,
 *                      sizex, sizey);
 *     window.setVisible(true);
 *   }
 * }
 * </PRE>
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class JargonFrame extends JFrame implements ActionListener, Observer {

  private static final long serialVersionUID = 400L;
  private JMenuBar menuBar = new JMenuBar();      // Window menu bar
  private final JFileChooser fc1 = new JFileChooser("../.." + System.getProperty("user.dir"));
  private final JFileChooser fc2 = new JFileChooser(System.getProperty("user.dir"));
  private JMenuItem setItem, viewItem, startItem;
  private JLabel status = new JLabel(" Alain Hebert, Ecole Polytechnique");
  private File scriptFile = null;
  private File schemeFile = null;
  private JMenu serversMenu = new JMenu("Servers");     // Create Servers menu
  private JMenu setMenu = new JMenu("Set variables");   // Create Set menu
  private JMenu viewMenu = new JMenu("View");           // Create View menu
  private JComboBox calcSchemeBox = new JComboBox();
  private ComputationalScheme calcScheme = null;
  private ServerFrame servers = null;
  String msg = "";
  /** 
   * Use this constructor to create a JargonFrame object.
   * @param title user-defined title shown in the JargonFrame object
   */
  public JargonFrame(String title) {
    setTitle(title);                              // Set the window title
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    setJMenuBar(menuBar);                         // Add the menu bar to the window

    // Construct the pull down menu
    setItem = serversMenu.add("Set / Start");
    setItem.addActionListener(this);
      
    menuBar.add(serversMenu);
    menuBar.add(setMenu);
    menuBar.add(viewMenu);
      
    Container content = this.getContentPane();
    //content.setLayout(new BorderLayout(20,20));
    JButton button;
    
    // set the JComboBox for selection of computational scheme
    JPanel schemePanel = new JPanel();
    schemePanel.setBackground(Color.yellow);
    schemePanel.setLayout(new BoxLayout(schemePanel, BoxLayout.X_AXIS));
    schemePanel.setBorder(BorderFactory.createCompoundBorder(
      BorderFactory.createTitledBorder("Select Computational Scheme"),
      BorderFactory.createEmptyBorder(0,5,2,5)));
    this.schemeFile = new File("jargon.jar");
    String[] names = ListScheme(this.schemeFile);
    for (int j =0; j<names.length; j++) {
      calcSchemeBox.addItem(names[j]);
      System.out.println("   Additem: " + names[j]);
    }
    calcSchemeBox.addActionListener(this);
    schemePanel.add(calcSchemeBox);
    schemePanel.add(button = new JButton("Change file"));
    button.addActionListener(this);
    content.add(schemePanel, BorderLayout.NORTH);
    
    Box box1 = Box.createVerticalBox();
    box1.add(Box.createVerticalStrut(5));
    box1.add(button = new JButton("Script file"));
    button.addActionListener(this);
    box1.add(Box.createVerticalStrut(5));
    box1.add(button = new JButton("Exec"));
    button.addActionListener(this);
    box1.add(Box.createVerticalStrut(5));
    box1.add(button = new JButton("Exec in console"));
    button.addActionListener(this);
    box1.add(Box.createVerticalStrut(5));
    box1.add(button = new JButton("Reset"));
    button.addActionListener(this);
    box1.add(Box.createHorizontalStrut(10));

    Image myDrag = Toolkit.getDefaultToolkit().getImage("dragon.gif");
    ImagePanel imp = new ImagePanel(myDrag);
    Box box2 = Box.createVerticalBox();
    box2.add(Box.createVerticalStrut(10));
    box2.add(imp);
      
    Box box3 = Box.createHorizontalBox();
    box3.add(box1);
    box3.add(box2);
    content.add(box3, BorderLayout.CENTER);
    
    content.add(status, BorderLayout.SOUTH);
    
    fc1.addChoosableFileFilter(new JargonFilter("jar", "Just jar archives"));
    fc2.addChoosableFileFilter(new JargonFilter("bsh", "Just Bean Shell files"));
  }
  /** overrides <tt>getInsets()</tt> in class Container.
   */
  public Insets getInsets() {
    Insets size = super.getInsets();
    return new Insets(size.top, size.left+5, size.bottom, size.right+5);
  }
  
  // Define a custom class loader.
  private class MyClassLoader extends URLClassLoader {
    MyClassLoader(File file) throws MalformedURLException {
      super(new URL[]{ new URL("jar", "", file.getPath() + "!/") });
    }
    
    public Class loadClass(String name) throws ClassNotFoundException {
      return super.loadClass(name, true);
    }
  }
  
  // List the available computational schemes present in file.
  private String[] ListScheme(File file) {
    if (!file.exists()) return new String[0];
    Vector<String> myList = new Vector<String>();
    MyClassLoader jar = null;
    ZipFile zf = null;
    try {
      jar = new MyClassLoader(file);
      zf = new ZipFile(file);
    } catch(IOException ex0) {
      status.setText(" IOException: " + ex0);
    }
    for (Enumeration e = zf.entries(); e.hasMoreElements();) {
      final ZipEntry ze = (ZipEntry) e.nextElement();
      String completeName = ze.getName();
      String className = completeName.substring(0,completeName.length()-6);
      if (ze.getName().endsWith(".class")) {
        Class cls = null;
        boolean isScheme = false;
        try {
          cls = jar.loadClass(className);
          Class superclass = cls.getSuperclass();
          if (superclass != null) {
            isScheme = superclass.getName().equals("jargon.ComputationalScheme");
          }
        } catch (ClassNotFoundException ex1) {
          status.setText(" ClassNotFoundException: " + ex1);
        }
        if (isScheme) myList.addElement(className.substring(7));
      }
    }
    String[] array = new String[myList.size()];
    myList.copyInto(array);
    return array;
  }

  //Bsh file filter
  private class JargonFilter extends FileFilter {
    private String extension, comment;
    
    JargonFilter (String extension, String comment) {
      this.extension = extension;
      this.comment = comment;
    }
    
    public boolean accept(File f) {
        if (f.isDirectory()) {
            return true;
        }

        String ext = null;
        String s = f.getName();
        int i = s.lastIndexOf('.');

        if (i > 0 &&  i < s.length() - 1) {
            ext = s.substring(i+1).toLowerCase();
        }
	if (ext != null) {
            if (ext.equals(this.extension)) {
                return true;
            } else {
                return false;
            }
    	}
        return false;
    }
    
    // The description of this filter
    public String getDescription() {
        return this.comment;
    }
  }
  /** implement the Observer interface. Handle observable events
   */
  public void update(Observable obj, Object arg) {
    System.out.println("JargonFrame observer: " + (String)arg + " detected for " + ((ComputationalScheme)obj).getName());
    status.setText(" Computational scheme " + this.calcScheme.getName() + " completed");
    this.calcScheme.operatorOut.lcmObj.lib();
    this.calcScheme.operatorOut.lcmObj.expor();
    repaint();
  }
  /** implement ActionListener interface. Handle button events.
   */
  public void actionPerformed(ActionEvent e) {
    String str = e.getActionCommand();
    if (str.equals("comboBoxChanged")) {
      System.out.println("combo: " + str);
      String name = (String)calcSchemeBox.getSelectedItem();
      status.setText(" Computational scheme selected: " + name);
      try {
        Class<?> scheme = Class.forName("jargon."+name);
        // Construction of a computationalal scheme instance
        try {
          Class[] types = new Class[]{String.class};
          Constructor myConstruct = scheme.getConstructor(types);
          this.calcScheme = (ComputationalScheme)myConstruct.newInstance(new Object[]{name});
        } catch (Exception ex4) {
          status.setText(" newInstance error: " + ex4);
        }
        //
        // Set "variables" menu
        Method methods[] = scheme.getDeclaredMethods();
        setMenu.removeAll();
        for (int i =0; i<methods.length; i++) {
          if (methods[i].getName().substring(0,3).equals("set")) {
            JMenuItem variableItem = new JMenuItem(methods[i].getName().substring(3));
            setMenu.add(variableItem);
            Class params[] = methods[i].getParameterTypes();
            for (int j =0; j<params.length; j++) {
              System.out.println("   Parameters: " + params[j].getName());
            }
            if (methods[i].getName().equals("setThreadPool")) {
              if (this.servers != null) {
                if (this.servers.getPool() == null) {
                  System.out.println("  ------------no pool");
                }
                Object[] theData = new Object[]{this.servers.getPool()};
                try {
                  methods[i].invoke(this.calcScheme, theData);
                } catch(Exception ex) {
                  status.setText(" set pool exception: " + ex);
                  return;
                }
                variableItem.setIcon(new ImageIcon("middle.gif"));
              }
            }
          }
        }
      } catch (ClassNotFoundException ex3) {
        status.setText(" forName error: " + ex3);
        return;
      }
    } else if (str.equals("Change file")) {
      int returnVal = fc1.showOpenDialog(this);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        this.schemeFile = fc1.getSelectedFile();
        calcSchemeBox.removeActionListener(this);
        calcSchemeBox.removeAllItems();
        String[] names = ListScheme(this.schemeFile);
                System.out.println("  ------------260");
        for (int j =0; j<names.length; j++) {
          calcSchemeBox.addItem(names[j]);
        }
        calcSchemeBox.addActionListener(this);
        status.setText(" New computational scheme file selected: " + this.schemeFile.getName());
      }
    } else if (str.equals("Script file")) {
      if (this.calcScheme == null) {
        status.setText(" Computational scheme not defined");
        return;
      }
      Method methods[] = this.calcScheme.getClass().getDeclaredMethods();
      int returnVal = fc2.showOpenDialog(this);
      if (returnVal == JFileChooser.APPROVE_OPTION) {
        File file = fc2.getSelectedFile();
        this.scriptFile = file;
        status.setText(" Script file selected: " + file.getName());
        try {
          FileReader reader = new FileReader(this.scriptFile);
          Interpreter interpreter = new Interpreter();
          try {
            Object outputObject = null;
            interpreter.eval(reader);
            String[] classes = (String[])interpreter.eval("this.methods");
            for (int i =0; i<classes.length; i++) {
              String beanName = classes[i].substring(3);
              System.out.println("   BeanShell method: " + classes[i]);
              outputObject = interpreter.eval(classes[i]+"()");
              if (outputObject == null) {
                status.setText(" Null object: " + beanName);
                return;
              }
              for (int j =0; j<methods.length; j++) {
                if (methods[j].getName().substring(0,3).equals("set")) {
                  if (methods[j].getName().substring(3).equals(beanName)) {
                    Object[] theData = {outputObject};
                    methods[j].invoke(this.calcScheme, theData);
                    JMenuItem variableItem = (JMenuItem)setMenu.getMenuComponent(j);
                    variableItem.setIcon(new ImageIcon("middle.gif"));
                  }
                }
              }
            }
          } catch(TargetError e1) {
            status.setText(" BeanShell target error: " + e1);
          } catch(EvalError e2) {
            status.setText(" BeanShell eval error: " + e2);
          }
        } catch(Exception e3) {
          status.setText(" Script file exception: " + e3);
        }
      }
    } else if (str.equals("Exec")) {
      if (this.calcScheme == null) {
        status.setText(" Computational scheme not defined");
      }
      try {
        Method myObserver = this.calcScheme.getClass().getMethod("addObserver", new Class[]{Observer.class});
        myObserver.invoke(this.calcScheme, (Object[]) new JargonFrame[]{this});
        Method run = this.calcScheme.getClass().getMethod("startThread", (Class[]) null);
        run.invoke(this.calcScheme, (Object[]) null);
      } catch (InvocationTargetException ex) {
        status.setText(" Exec error: " + ex);
        Throwable ex2 = ex.getTargetException();
        JOptionPane.showMessageDialog(this, ex2.toString().trim());
      } catch (Exception ex) {
        status.setText(" Exec error: " + ex);
      }
      status.setText(" Computational scheme " + this.calcScheme.getName() + " running...");
    } else if (str.equals("Exec in console")) {
      if (this.calcScheme == null) {
        status.setText(" Computational scheme not defined");
      }
      if (this.scriptFile == null) {
        status.setText(" No input file selected");
      } else {
        try {
          JFrame aWindow = new JFrame("BeanShell Console Window");
          aWindow.setBounds(5,5,800,300);
          Container myPanel = aWindow.getContentPane();
          JConsole console = new JConsole();
          myPanel.add(console);
          Interpreter interpreter = new Interpreter(console);
          System.setOut(console.getOut());
          //try {
            try {
              interpreter.set("run", this.calcScheme.getClass().getMethod("run", (Class[]) null));
              interpreter.set("calcScheme", this.calcScheme);
              interpreter.eval("run.invoke(calcScheme, null)");
            } catch (Exception ex) {
              status.setText(" Exec error: " + ex);
            }
            new Thread(interpreter).start();
          //} catch(TargetError e1) {
            //status.setText(" BeanShell target error: " + e1);
          //} catch(EvalError e2) {
            //status.setText(" BeanShell eval error: " + e2);
          //}
          aWindow.setVisible(true);
        } catch(Exception e3) {
          status.setText(" Script file exception: " + e3);
        }
      }
      status.setText(" Computational scheme " + this.calcScheme.getName() + " completed");
    } else if (str.equals("Reset")) {
      this.scriptFile = null;
      status.setText(" Reset calculation");
    } else if (str.equals("Set / Start")) {
      if (this.servers == null) this.servers = new ServerFrame();
      this.servers.setVisible(true);
    } else {
      status.setText(" Unknown action: " + str);
    }
    System.out.println("Button pressed: " + str);   
    repaint();
  }    

  private class ImagePanel extends JPanel {
    private static final long serialVersionUID = 401L;
    Image image;

    public ImagePanel(Image image) {
        this.image = image;
    }

    public void paintComponent(Graphics g) {
        super.paintComponent(g); //paint background
        g.drawImage(image, 0, 10, 100, 100, this);
    }
  }
}
