package jargon;

/*--------------------------------*/
/* Java-LCM bindings              */
/*--------------------------------*/

/*
Copyright (C) 2002 Ecole Polytechnique de Montreal

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.
*/

import java.util.Hashtable;
import java.util.Vector;
import java.io.*;
import java.util.Enumeration;
import java.util.Date;
import java.util.zip.*;
/**
 * This class is an implementation of the Java bindings for LCM.
 * LCM capabilities are available for a program written in Java
 * by using methods belonging to the Jlcm class. These methods
 * encapsulate the LCM C API calls using Java Native Interface
 * (JNI).
 * <P> A Jlcm object in Java can encapsulate a native LCM object
 * which can be memory-resident or persistent (using the XSM C API)
 * or a file. A LCM or XSM native object can contains dictionaries
 * (aka associative tables or hash tables) and lists.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Jlcm implements Externalizable {
  /** Print "Jlcm class loaded" and load LCM native libraries.
   */
  static {
    System.out.println("Jlcm class loaded");
    System.loadLibrary("JlcmNat");
  }
  
  private static final long serialVersionUID = 6L;
  private native void lcmop(String s, int imp, int medium, int impx);
  private native void lcmcl(int iact);
  private native void lcmlib();
  private native int lcmAccess();
  private native void lcminf();
  private native String lcmnxt(String s);
  private native void lcmppd(String s, int ilong, int itylcm, Object object);
  private native void lcmput(String s, int ilong, int itylcm, Object object);
  private native boolean lcmgpd(String s, Object object);
  private native Object lcmget(String s);
  private native int[] lcmlen(String s);
  private native int[] lcmlel(int iset);
  private native long lcmdid(String s);
  private native long lcmlid(String s, int ilong);
  private native long lcmdil(int iset);
  private native long lcmlil(int iset, int ilong);
  private native long lcmgid(String s);
  private native long lcmgil(int iset);
  private native boolean lcmgpl(int iset, Object object);
  private native Object lcmgdl(int iset);
  private native void lcmppl(int iset, int ilong, int itylcm, Object object);
  private native void lcmpdl(int iset, int ilong, int itylcm, Object object);
  private native long lcmequ(String name, int imode);
  private native void lcmexp(int impx, String name, int imode, int idir);
  private native void lcmval(String s);
  
  private Hashtable<String, Object> global_dict; /* container for the global references */
  private Vector<Object> global_list; /* container for the global references */
  private transient long addr;  /* address of the LCM object or -1 for a file */
  private transient File aFile; /* embedded file if objtype >= 3 */
  private int lrda; /* used with direct access files only */
  private String name;   /* object name */
  private String nammy;  /* directory name */
  private boolean empty; /* empty flag */
  private int ilong;     /* -1 (dictionary) or list length */
  private int objtype;  /* type of the object =1: LCM; =2: XSM; =3: binary Fortran file
                =4: ASCII Fortran file; =5: direct access Fortran file */
  private int access; /* =0: closed object; =1: object in read/write mode
                 =2: object in read-only mode */
  private boolean untoutched;  /* brand new object flag */

  /**
   * Use this constructor to create a Jlcm object without embedded LCM
   * object or file. This no-parameter constructor is used by readExternal().
   */
  public Jlcm() {
    this.access = 0;
    this.addr = 0;
  }
  
  /**
   * Use this constructor to create a Jlcm object of a specified type.
   * The new LCM object is open in modification mode and is in a state
   * such that the <TT>isNew()</TT> method will return <TT>true</TT>.
   * If a XSM file <TT>name</TT> exists, the Jlcm object is pointing to
   * this file open in <TT>"READ-ONLY"</TT> mode and is in a state
   * such that the <TT>isNew()</TT> method will return <TT>false</TT>.
   * If the type is <TT>"LCM_IMP"</TT> or <TT>"XSM_IMP"</TT>, the new
   * LCM object is open in modification mode and the <TT>isNew()</TT>
   * method will return <TT>false</TT>.
   * @param stype type of the new LCM object. This variable is chosen among
   * the following values:
   * <ul><dl>
   * <dt> <tt>"LCM"</tt> <dd> memory-resident LCM object
   * <dt> <tt>"XSM"</tt> <dd> persistent LCM object
   * <dt> <tt>"BINARY"</tt> <dd> binary sequential file
   * <dt> <tt>"ASCII"</tt> <dd> ACSII sequential file
   * <dt> <tt>"DA"</tt> <dd> binary direct-access file (with 128-word records)
   * <dt> <tt>"LCM_IMP"</tt> <dd> memory-resident LCM object constructed by
   * importing from an ASCII sequential file named <TT>"_"</TT>+name
   * <dt> <tt>"XSM_IMP"</tt> <dd> persistent LCM object  constructed by importing
   * from an ASCII sequential file named <TT>"_"</TT>+name
   * </dl></ul>
   * @param name creation name of the new LCM object
   */
  public Jlcm(String stype, String name) {
    System.out.println("New JLCM object called '" + name + "'");
    boolean l_import = false;
    boolean l_exists = false;
    if (stype.equals("LCM")) {
      this.objtype = 1;
    } else if (stype.equals("XSM")) {
      this.objtype = 2;
      String[] names = new File(".").list();
      for (int i=0; i<names.length; i++) {
        if (names[i].equals(name)) l_exists = true;
      }
    } else if (stype.equals("BINARY")) {
      this.objtype = 3;
    } else if (stype.equals("ASCII")) {
      this.objtype = 4;
    } else if (stype.equals("DA")) {
      this.objtype = 5;
    } else if (stype.equals("LCM_IMP")) {
      this.objtype = 1;
      l_import = true;
    } else if (stype.equals("XSM_IMP")) {
      this.objtype = 2;
      l_import = true;
    } else {
      throw new RuntimeException("invalid type: '" + stype + "'");
    }
    this.access = 0;
    this.name = name;
    this.addr = 0;
    this.lrda = 128;
    try {
      if (l_exists) {
        this.addr = -1;
        System.out.println("XSM file " + name + " already exists");
        this.open("READ-ONLY");
        this.access = 2;
      } else {
        this.open("NEW");
        this.access = 1;
      }
    } catch(Exception e) {
      System.out.println("Exception catched by new (1): " + e);
      throw new RuntimeException(e.toString());
    }
    if (this.objtype <= 2) {
      this.global_dict = new Hashtable<String, Object>();
      this.nammy = "/";
      this.empty = true;
      this.ilong = -1;
    }
    if (l_import) {
      this.empty = false;
      this.untoutched = false;
      try {
        String[] names = new File(".").list();
        for (int i=0; i<names.length; i++) {
          if (names[i].indexOf('_') >= 0) {
            String fileName = names[i].substring(names[i].indexOf('_'));
            if (fileName.equals("_"+this.name)) {
              this.lcmexp(0, names[i], 2, 2);
              return;
            }
          }
        }
        throw new RuntimeException("File not found:" + "_"+this.name);
      } catch(Exception e) {
        System.out.println("Exception catched by new (2): " + e);
        throw new RuntimeException(e.toString());
      }
    }
  }
  /**
   * Use this constructor to create a Jlcm object of binary direct-access type.
   * @param stype type of the new LCM object. Can only be <TT>"DA"</TT>.
   * @param name name of the new LCM object
   * @param lrda size in words of a direct-access record
   */
  public Jlcm(String stype, String name, int lrda) {
    if (!stype.equals("DA")) throw new RuntimeException("lrda implies 'DA'");
    this.access = 0;
    this.name = name;
    this.addr = 0;
    this.lrda = lrda;
    this.open("NEW");
    this.access = 1;
  }
  private Jlcm(long daughter, int type, String name, int ilong, String key) {
    this.addr = daughter;
    if (ilong == -1) {
      this.global_dict = new Hashtable<String, Object>();
    } else {
      this.global_list = new Vector<Object>(ilong);
      for (int i=0; i<ilong; i++) this.global_list.addElement(null);
    }
    this.access = 1;
    this.name = name;
    this.nammy = key;
    this.empty = true;
    this.ilong = ilong;
    this.objtype = type;
    this.untoutched = false;
  }
  /** Return the name of the Jlcm object.
   */
    public String getName() {
    return this.name;
  }
  /** Return the type of the Jlcm object.
   * @return =1: memory-resident LCM object; =2: persistent LCM object;
   * =3: binary sequential file; =4: ASCII sequential file; =5 binary
   * direct-access file.
   */
  public int getType() {
    return this.objtype;
  }
  /** Return the length of a list-type Jlcm object. This method is not
   * available for file-type Jlcm objects.
   * @return =-1: dictionary; >=1: length of the list-type Jlcm object.
   */
  public int getLength() {
    if (this.objtype > 2) throw new RuntimeException("getLength not available for file objects");
    this.lcminf();
    return this.ilong;
  }
  /** Return the name of the accessible directory of a dictionary-type
   * Jlcm object. This method is not available for file-type Jlcm objects.
   * @return =<TT>"/"</TT> for a dictionary on root or name of the accessible
   * directory.
   */
  public String getDirectory() {
    if (this.objtype > 2) throw new RuntimeException("getDirectory not available for file objects");
    this.lcminf();
    return this.nammy;
  }
  /** Return the access type of a Jlcm object.
   * @return =0: the object is closed; =1: the object is open in modification
   * mode; =2: the object is open in read-only mode.
   */
  public int getAccess() {
    if (this.objtype <= 2) this.access = this.lcmAccess();
    return this.access;
  }
  /** Return the number of words in a record of a direct access-type Jlcm object.
   */
  public int getLrda() {
    return this.lrda;
  }
  /** Return true if the a dictionary-type Jlcm object is empty. This method
   * is not available for file-type Jlcm objects.
   */
  public boolean isEmpty() {
    if (this.objtype > 2) throw new RuntimeException("isEmpty not available for file objects");
    this.lcminf();
    return this.empty;
  }
  /** Return true if the dictionary-type Jlcm object is new. This method
   * is not available for file-type Jlcm objects.
   */
  public boolean isNew() {
    return this.untoutched;
  }
  /** Open a new Jlcm object or reopen an existing Jlcm object already closed by
   * the close(<TT>"KEEP"</TT>) method. In this case, the Jlcm object is reopen
   * in a state such as the <TT>isNew()</TT> method will return false.
   * @param saccess type of open. This variable is chosen among the following
   * values:
   * <ul><dl>
   * <dt> <tt>"READ/WRITE"</tt> <dd> open in modification mode.
   * <dt> <tt>"READ-ONLY"</tt> <dd> open in read-only mode.
   * </dl></ul>
   */
  public void open(String saccess) {
    if (this.access != 0) {
      throw new RuntimeException("the object " + this.name + " is already open");
    }
    if (saccess.equals("NEW")) {
      this.untoutched = true;
    } else if (saccess.equals("READ/WRITE")) {
      if ((this.addr == 0)&(this.objtype <= 2)) {
        throw new RuntimeException("the object " + this.name + " is destroyed");
      }
      this.access = 1;
      this.untoutched = false;
    } else if (saccess.equals("READ-ONLY")) {
      if ((this.addr == 0)&(this.objtype <= 2)) {
        throw new RuntimeException("the object " + this.name + " is destroyed");
      }
      this.access = 2;
      this.untoutched = false;
    } else {
      throw new RuntimeException("invalid access: '" + saccess + "'");
    }
    if (this.objtype <= 2) {
      try {
        String trunc;
        if (name.length() > 12) {
          trunc = new String(name.substring(0,12));
        } else {
          trunc = new String(name);
        }
        this.lcmop(trunc, this.access, this.objtype, 0);
      } catch(Exception e) {
        System.out.println("Exception catched by open (1): " + e);
        throw new RuntimeException(e.toString());
      }
    } else if (this.objtype <= 5) {
      this.addr = -1;
      this.aFile = new File(this.name);
      boolean exs = this.aFile.exists();
      if (this.access == 0) {
        if (exs) {
          String hsmg = "New file " + this.name + " exists";
          System.out.println(hsmg);
          throw new RuntimeException(hsmg);
        }
      } else if (this.access == 1) {
        if (!exs) {
          String hsmg = "Old file " + this.name + " does not exist";
          System.out.println(hsmg);
          throw new RuntimeException(hsmg);
        } else if (!this.aFile.canWrite()) {
          String hsmg = "Old file " + this.name + " is read-only";
          System.out.println(hsmg);
          throw new RuntimeException(hsmg);
        }
      } else if (this.access == 2) {
        if (!exs) {
          String hsmg = "Old file " + this.name + " does not exist";
          System.out.println(hsmg);
          throw new RuntimeException(hsmg);
        } else if (!this.aFile.canRead()) {
          String hsmg = "Old file " + this.name + " is not permitted to read";
          System.out.println(hsmg);
          throw new RuntimeException(hsmg);
        }
      }
    } else {
      throw new RuntimeException("invalid type in open (" + this.objtype + ")");
    }
  }
  /** Store an elementary block of information in a dictionary-type
   * Jlcm object. This method is not available for file-type Jlcm objects.
   * @param key key identification of the block in the dictionary
   * @param value elementary block of information (an Object instance containing
   * a String or an array of identical values). These values can be integers (int),
   * real numbers (float or double), boolean or Complex objects.
   */
  public void put(String key, Object value) {
    int ilong,itylcm;
    if (this.objtype > 2) throw new RuntimeException("put not available for file objects");
    String str = value.getClass().getName();
    if (str.equals("[I")) {
      itylcm = 1;
      ilong = ((int[])value).length;
    } else if (str.equals("[F")) {
      itylcm = 2;
      ilong = ((float[])value).length;
    } else if (str.equals("java.lang.String")) {
      itylcm = 3;
      ilong = (((String)value).length() + 3) / 4;
    } else if (str.equals("[D")) {
      itylcm = 4;
      ilong = 2*((double[])value).length;
    } else if (str.equals("[Z")) {
      itylcm = 5;
      ilong = ((boolean[])value).length;
    } else if (str.equals("[Ljargon.Complex;")) {
      itylcm = 6;
      ilong = 2*((Complex[])value).length;
    } else {
      throw new RuntimeException("invalid type: '" + str + "' key='" + key + "'");
    }
    if ((this.objtype == 1) & (itylcm != 3) & (itylcm != 5) & (itylcm != 6)) {
      // Try to use a reference without copy (pinning). There is no
      // guaranty that this will work, especially if you use a Sun compiler.
      // Keep a reference of value in global to avoid garbage collection
      try {
        this.global_dict.put(key, value);
        this.lcmppd(key, ilong, itylcm, value);
      } catch(Exception e) {
        System.out.println("Exception catched by put: " + e);
        throw new RuntimeException(e.toString());
      }
    } else {
      /* use a copy */
      try {
        this.lcmput(key, ilong, itylcm, value);
      } catch(Exception e) {
        System.out.println("Exception catched by put: " + e);
        throw new RuntimeException(e.toString());
      }
    }
  }
  /** Store an elementary block of information in a list-type Jlcm object. This
   * method is not available for file-type Jlcm objects.
   * @param iset index of the block in the list. The first list element is
   * stored at index 0.
   * @param value elementary block of information (an Object instance containing
   * a String or an array of identical values). These values can be integers (int),
   * real numbers (float or double), boolean or Complex objects.
   */
  public void put(int iset, Object value) {
    int ilong,itylcm;
    if (this.objtype > 2) throw new RuntimeException("put not available for file objects");
    String str = value.getClass().getName();
    if (str.equals("[I")) {
      itylcm = 1;
      ilong = ((int[])value).length;
    } else if (str.equals("[F")) {
      itylcm = 2;
      ilong = ((float[])value).length;
    } else if (str.equals("java.lang.String")) {
      itylcm = 3;
      ilong = (((String)value).length() + 3) / 4;
    } else if (str.equals("[D")) {
      itylcm = 4;
      ilong = 2*((double[])value).length;
    } else if (str.equals("[Z")) {
      itylcm = 5;
      ilong = ((boolean[])value).length;
    } else if (str.equals("[Ljargon.Complex;")) {
      itylcm = 6;
      ilong = 2*((Complex[])value).length;
    } else {
      throw new RuntimeException("invalid type: '" + str + "' iset=" + iset);
    }
    if ((this.objtype == 1) & (itylcm != 3) & (itylcm != 5) & (itylcm != 6)) {
      // Try to use a reference without copy (pinning). There is no
      // guaranty that this will work, especially if you use a Sun compiler.
      // Keep a reference of value in global to avoid garbage collection
      try {
        this.global_list.setElementAt(value, iset);
        this.lcmppl(iset, ilong, itylcm, value);
      } catch(Exception e) {
        System.out.println("Exception catched by put: " + e);
        throw new RuntimeException(e.toString());
      }
    } else {
      /* use a copy */
      try {
        this.lcmpdl(iset, ilong, itylcm, value);
      } catch(Exception e) {
        System.out.println("Exception catched by put: " + e);
        throw new RuntimeException(e.toString());
      }
    }
  }
  /**
   * Return the length of a block of information in a dictionary-type
   * Jlcm object. This method is not available for file-type Jlcm objects.
   * @param key key identification of the block in the dictionary
   * @return length in single word unit for an elementary block; -1 for
   * a daughter dictionary; length of the list for a daughter list.
   */
  public int length(String key) {
    if (this.objtype > 2) throw new RuntimeException("length not available for file objects");
    try {
      int[] info = this.lcmlen(key);
      return info[0];
    } catch(Exception e) {
      System.out.println("Exception catched by length: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /**
   * Return the type of a block of information in a dictionary-type
   * Jlcm object. This method is not available for file-type Jlcm objects.
   * @param key key identification of the block in the dictionary
   * @return =0: dictionary; =1: integer (int); =2: real number (float);
   * =3: String; =4: real number (double); =5: boolean; =6: Complex object;
   * =10: list; =99: the block does not exists or is undefined.
   */
  public int type(String key) {
    if (this.objtype > 2) throw new RuntimeException("type not available for file objects");
    try {
      int[] info = this.lcmlen(key);
      return info[1];
    } catch(Exception e) {
      System.out.println("Exception catched by type: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /**
   * Return the length of a block of information in a list-type Jlcm object.
   * This method is not available for file-type Jlcm objects.
   * @param iset index of the block in the list. The first list element is
   * stored at index 0.
   * @return length in single word unit for an elementary block; -1 for
   * a daughter dictionary; length of the list for a daughter list.
   */
  public int length(int iset) {
    if (this.objtype > 2) throw new RuntimeException("length not available for file objects");
    try {
      int[] info = this.lcmlel(iset);
      return info[0];
    } catch(Exception e) {
      System.out.println("Exception catched by length: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /**
   * Return the type of a block of information in a list-type Jlcm object.
   * This method is not available for file-type Jlcm objects.
   * @param iset index of the block in the list. The first list element is
   * stored at index 0.
   * @return =0: dictionary; =1: integer (int); =2: real number (float);
   * =3: String; =4: real number (double); =5: boolean; =6: Complex object;
   * =10: list; =99: the block does not exists or is undefined.
   */
  public int type(int iset) {
    if (this.objtype > 2) throw new RuntimeException("type not available for file objects");
    try {
      int[] info = this.lcmlel(iset);
      return info[1];
    } catch(Exception e) {
      System.out.println("Exception catched by type: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Recover a daughter Jlcm object or an elementary block of information
   * from a dictionary-type Jlcm object. This method is not available
   * for file-type Jlcm objects.
   * @param key key identification of the block in the dictionary
   * @return daughter Jlcm object (dictionary or list) or elementary
   * block of information (array)
   */
  public Object get(String key) {
    if (this.objtype > 2) throw new RuntimeException("get not available for file objects");
    if (this.type(key) == 0) {
      long daughter = this.lcmgid(key);
      return new Jlcm(daughter, this.objtype, this.name, -1, key);
    } else if (this.type(key) == 10) {
      long daughter = this.lcmgid(key);
      Jlcm new_obj = new Jlcm(daughter, this.objtype, this.name, 0, key);
      new_obj.lcminf();
      new_obj.global_list = new Vector<Object>(new_obj.getLength());
      for (int i=0; i<new_obj.getLength(); i++) new_obj.global_list.addElement(null);
      return new_obj;
    } else if (this.objtype == 1) {
      // Try to use a reference without copy (pinning).
      Object oo = this.global_dict.get(key);
      if (oo != null) {
        try {
          if (this.lcmgpd(key,oo)) return oo;
        } catch(Exception e) {
          System.out.println("Exception catched by get: " + e);
          throw new RuntimeException(e.toString());
        }
      }
    }
    return this.lcmget(key);
  }
  /** Recover a daughter Jlcm object or an elementary block of information
   * from a list-type Jlcm object. This method is not available for file-type
   * Jlcm objects.
   * @param iset index of the block in the list. The first list element is
   * stored at index 0.
   * @return daughter Jlcm object (dictionary or list) or elementary
   * block of information (array)
   */
  public Object get(int iset) {
    if (this.objtype > 2) throw new RuntimeException("get not available for file objects");
    if (this.type(iset) == 0) {
      long daughter = this.lcmgil(iset);
      return new Jlcm(daughter, this.objtype, this.name, -1, " ");
    } else if (this.type(iset) == 10) {
      long daughter = this.lcmgil(iset);
      Jlcm new_obj = new Jlcm(daughter, this.objtype, this.name, 0, " ");
      new_obj.lcminf();
      new_obj.global_list = new Vector<Object>(new_obj.getLength());
      for (int i=0; i<new_obj.getLength(); i++) new_obj.global_list.addElement(null);
      return new_obj;
    } else if (this.objtype == 1) {
      // Try to use a reference without copy (pinning).
      Object oo = this.global_list.elementAt(iset);
      if (oo != null) {
        try {
          if (this.lcmgpl(iset,oo)) return oo;
        } catch(Exception e) {
          System.out.println("Exception catched by get: " + e);
          throw new RuntimeException(e.toString());
        }
      }
    }
    return this.lcmgdl(iset);
  }
  /**
   * Return an Enumeration object for iterating on the keys of an associative
   * table-type Jlcm object. This method is not available for file-type Jlcm
   * objects. Iteration can be performes as in the following example:
   * <PRE>
   * Enumeration ee = my_lcm.keys();
   * while(ee.hasMoreElements()) {
   *   String key = (String)ee.nextElement();
   *   System.out.println("element=" + key);
   * }
   * </PRE>
   */
  public Enumlcm keys() {
    if (this.objtype > 2) throw new RuntimeException("keys not available for file objects");
    return new Enumlcm(this);
  }
  /** Close a Jlcm object.
   * @param saccess =<TT>"KEEP"</TT>: close without destruction of the object
   * content; =<TT>"DESTROY"</TT>: close with destruction of the object content.
   */
  public void close(String saccess) {
    int iact;
    if (saccess.equals("KEEP")) {
      iact = 1;
    } else if (saccess.equals("DESTROY")) {
      iact = 2;
    } else if (saccess.equals("ERASE")) {
      iact = 3;
    } else {
      throw new RuntimeException("invalid condition: '" + saccess + "'");
    }
    if (this.access != 0) {
      if (this.objtype <= 2) {
        try {
          this.lcmcl(iact);
        } catch(Exception e) {
          System.out.println("Exception catched by close (1): " + e);
          throw new RuntimeException(e.toString());
        }
      } else if (this.objtype <= 5) {
        if (iact >= 2) {
          try {
            this.aFile.delete();
            System.out.println("File deleted:" + this.name);
          } catch(Exception e) {
            System.out.println("Exception catched by close (2): " + e);
            throw new RuntimeException(e.toString());
          }
        }
      }
    } else {
      throw new RuntimeException("the object " + this.name + " is already closed");
    }
    if (iact == 2) {
      this.addr = 0;
      this.global_dict = null;
      this.global_list = null;
    }
    this.access = 0;
  }
  /** Print the table of contents of a dictionary- or list-type Jlcm object.
   * This method is not available for file-type Jlcm objects.
   */
  public void lib() {
    if (this.objtype > 2) throw new RuntimeException("lib not available for file objects");
    try {
      this.lcmlib();
    } catch(Exception e) {
      System.out.println("Exception catched by lib: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Creation of a daughter dictionary in a dictionary-type Jlcm object.
   * This method is not available for file-type Jlcm objects.
   * @param key key identification of the daughter dictionary
   * @return daughter Jlcm object (dictionary)
   */
  public Jlcm rep(String key) {
    if (this.objtype > 2) throw new RuntimeException("rep not available for file objects");
    try {
      long daughter = this.lcmdid(key);
      return new Jlcm(daughter, this.objtype, this.name, -1, key);
    } catch(Exception e) {
      System.out.println("Exception catched by rep: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Creation of a daughter list in a dictionary-type Jlcm object.
   * This method is not available for file-type Jlcm objects.
   * @param key key identification of the daughter list
   * @param ilong length of the daughter list
   * @return daughter Jlcm object (list)
   */
  public Jlcm lis(String key, int ilong) {
    if (this.objtype > 2) throw new RuntimeException("lis not available for file objects");
    try {
      long daughter = this.lcmlid(key, ilong);
      return new Jlcm(daughter, this.objtype, this.name, ilong, key);
    } catch(Exception e) {
      System.out.println("Exception catched by lis: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Creation of a daughter dictionary in a list-type Jlcm object.
   * This method is not available for file-type Jlcm objects.
   * @param iset index the daughter dictionary
   * @return daughter Jlcm object (dictionary)
   */
  public Jlcm rep(int iset) {
    if (this.objtype > 2) throw new RuntimeException("rep not available for file objects");
    try {
      long daughter = this.lcmdil(iset);
      return new Jlcm(daughter, this.objtype, this.name, -1, " ");
    } catch(Exception e) {
      System.out.println("Exception catched by rep: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Creation of a daughter list in a list-type Jlcm object.
   * This method is not available for file-type Jlcm objects.
   * @param iset index of the daughter list
   * @param ilong length of the daughter list
   * @return daughter Jlcm object (list)
   */
  public Jlcm lis(int iset, int ilong) {
    if (this.objtype > 2) throw new RuntimeException("lis not available for file objects");
    try {
      long daughter = this.lcmlil(iset, ilong);
      return new Jlcm(daughter, this.objtype, this.name, ilong, " ");
    } catch(Exception e) {
      System.out.println("Exception catched by lis: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Deep copy of a dictionary- or list-type Jlcm object into a memory-resident
   * Jlcm object. This method is not available for file-type Jlcm objects.
   * @param new_name name of the new Jlcm object
   * @return new Jlcm object created by the copy.
   */
  public Jlcm copy(String new_name) {
    if (this.objtype > 2) throw new RuntimeException("copy not available for file objects");
    return this.copy(new_name, 1);
  }
  /** Deep copy of a dictionary- or list-type Jlcm object into a memory-resident
   * or persistent Jlcm object. This method is not available for file-type Jlcm
   * objects.
   * @param new_name name of the new Jlcm object
   * @param imode =1: to create a memory_resident Jlcm object; =2: to create
   * a persistent object
   * @return new Jlcm object created by the copy.
   */
  public Jlcm copy(String new_name, int imode) {
    if (this.objtype > 2) throw new RuntimeException("copy not available for file objects");
    try {
      long new_lcm = this.lcmequ(new_name, imode);
      return new Jlcm(new_lcm, imode, new_name, -1, " ");
    } catch(Exception e) {
      System.out.println("Exception catched by copy: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Serialize and save the object content on a sequential file. This method is not
   * available for file-type Jlcm objects. The name of the sequential file is
   * the catenation of <TT>"_"</TT> with the name of the Jlcm object.
   */
  public void expor() {
    this.expor("_"+this.name);
  }
 /** Serialize and save the object content on a sequential file. This method is not
   * available for file-type Jlcm objects.
   * @param new_name name of the sequential file. This name must begin by
   * character <TT>"_"</TT>.
   */
  public void expor(String new_name) {
    if (this.objtype > 2) throw new RuntimeException("expor not available for file objects");
    if (new_name.indexOf('_') == -1) {
      throw new RuntimeException("export file name ("+new_name+") must contains _");
    }
    try {
      this.lcmexp(0, new_name, 2, 1);
    } catch(Exception e) {
      System.out.println("Exception catched by expor: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Test the consistency of the complete native LCM object embedded in the
   * Jlcm object. This method is not available for file-type Jlcm objects.
   */
  public void val() {
    if (this.objtype > 2) throw new RuntimeException("val not available for file objects");
    this.lcmval(" ");
  }
  /** Test the consistency of the information below a key in a native LCM object
   * embedded in the Jlcm object. This method is not available for file-type
   * Jlcm objects.
   * @param key key of the distionary-type Jlcm object where the consistency
   * ckeck starts.
   */
  public void val(String key) {
    if (this.objtype > 2) throw new RuntimeException("val not available for file objects");
    this.lcmval(key);
  }
  /** Cause an exception (used to debug XABORT in Java).
   */
  public void except() {
    this.lcmcl(78);
  }
  /** Dump the LCM content to the PrintWriter.
   */
  public void dump() {
    if (this.objtype <= 2) {
      try {
        String fileName =  new Date().getTime()+"_"+this.hashCode()+"_"+this.name;
        this.expor(fileName);
        File fin = new File(fileName);
        FileInputStream in = new FileInputStream(fin);
        PrintWriter out = new PrintWriter(System.out);
        int c;
        System.out.println("Dump LCM object named '"+this.name+"' from " +
        "directory '" + this.nammy + "':");
        System.out.print("----------------------------------------");
        System.out.println("----------------------------------------");
        while ((c = in.read()) != -1) out.write(c);
        out.flush();
        System.out.print("----------------------------------------");
        System.out.println("----------------------------------------");
        fin.delete();
        in.close();
      } catch(IOException e) {
        System.out.println("Exception catched by dump: " + e);
        throw new RuntimeException(e.toString());
      }
    } else {
      throw new RuntimeException("dump: unable to dump a file");
    }
  }
  /** Close and destroy a Jlcm object in modification mode; close and
   * keep a Jlcm object in read-only mode.
   */
  protected void finalize() {
    System.out.println("Jlcm finalize called. LCM object=" + this.name);
    if ((this.addr != 0) & (this.access == 1)) {
      System.out.println("Destroy object " + this.name);
      this.close("DESTROY");
    } else if ((this.addr != 0) & (this.access == 2)) {
      System.out.println("Keep object " + this.name);
      this.close("KEEP");
    }
  }
  /**
   * Implements the writeExternal method of the Externalizable interface
   * used to save the contents of the Jlcm object.
   * The writeExternal method is used by the serialization process of
   * a Jlcm object. In the following example, a Jlcm object is transformed
   * into a serialized stream and stored in file <TT>"serialized_object"</TT>:
   * <PRE>
   * try{
   *   FileOutputStream fo = new FileOutputStream("serialized_object");
   *   ObjectOutputStream so = new ObjectOutputStream(fo);
   *   so.writeObject(my_lcm);
   *   so.flush();
   * } catch (Exception ex) {
   *   System.out.println(ex);
   *   System.exit(1);
   * }
   * </PRE>
   */
  public void writeExternal(ObjectOutput out) throws IOException {
    File fil;
    if (this.objtype <= 2) {
      String fileName = new Date().getTime()+"_"+this.hashCode()+"_"+this.name;
      this.expor(fileName);
      fil = new File(fileName);
    } else {
      fil = new File(this.name);
    }
    FileInputStream in = new FileInputStream(fil);
    File filgz = new File(fil.getName()+".gz");
    GZIPOutputStream outgz = new GZIPOutputStream(new FileOutputStream(filgz));
    try {
      byte[] buffer = new byte[1024];
      int len;
      while ((len = in.read(buffer)) >= 0) outgz.write(buffer,0,len);
    } catch(IOException e) {
      System.out.println("Exception catched by writeExternal (1): " + e);
      throw new RuntimeException(e.toString());
    }
    in.close();
    outgz.close();
    if (this.objtype <= 2) fil.delete();
    FileInputStream ingz = new FileInputStream(filgz);
    out.writeObject(this.name);
    out.writeInt(this.objtype);
    out.writeLong(filgz.length());
    try {
      byte[] buffer = new byte[1];
      while (ingz.read(buffer) >= 0) out.write(buffer,0,1);
    } catch(IOException e) {
      System.out.println("Exception catched by writeExternal (2): " + e);
      throw new RuntimeException(e.toString());
    }
    filgz.delete();
  }
  /**
   * Implements the readExternal method of the Externalizable interface
   * used to restore the contents of the Jlcm object.
   * The readExternal method is used by the deserialization process of
   * a Jlcm object. In the following example, a serialized stream is
   * recovered from file <TT>"serialized_object"</TT> and transformed
   * into a Jlcm object:
   * <PRE>
   * try{
   *   FileInputStream fi = new FileInputStream("serialized_object");
   *   ObjectInputStream si = new ObjectInputStream(fi);
   *   my_lcm = (Jlcm)si.readObject();
   * } catch (Exception ex) {
   *   System.out.println(ex);
   *   System.exit(1);
   * }
   * </PRE>
   */
  public void readExternal(ObjectInput in) throws IOException, ClassNotFoundException {
    this.name = (String)in.readObject();
    this.objtype = in.readInt();
    long lengg = in.readLong();
    String fileName = new Date().getTime()+"_"+this.hashCode()+"_"+this.name;
    File filgz = new File(fileName+".gz");
    FileOutputStream outgz = new FileOutputStream(filgz);
    try {
      byte[] buffer = new byte[1];
      for (int i=0; i<lengg; i++) {
        in.read(buffer);
        outgz.write(buffer,0,1);
      }
    } catch(IOException e) {
      System.out.println("Exception catched by readExternal (1): " + e);
      throw new RuntimeException(e.toString());
    }
    outgz.close();
    GZIPInputStream ingz = new GZIPInputStream(new FileInputStream(filgz));
    if (this.objtype <= 2) {
      File fil = new File(fileName);
      FileOutputStream out = new FileOutputStream(fil);
      try {
        byte[] buffer = new byte[1024];
        int len;
        while ((len = ingz.read(buffer)) >= 0) out.write(buffer,0,len);
      } catch(IOException e) {
        System.out.println("Exception catched by readExternal (2): " + e);
        throw new RuntimeException(e.toString());
      }
      filgz.delete();
      out.close();
      this.global_dict = new Hashtable<String, Object>();
      this.nammy = "/";
      this.empty = false;
      this.ilong = -1;
      try {
        this.open("NEW");
      } catch(Exception e) {
        System.out.println("Exception catched by readExternal (3): " + e);
        throw new RuntimeException(e.toString());
      }
      try {
        this.lcmexp(0, fil.getName(), 2, 2);
      } catch(Exception e) {
        System.out.println("Exception catched by readExternal (4): " + e);
        throw new RuntimeException(e.toString());
      }
      fil.delete();
    } else {
      File fout = new File(this.name);
      FileOutputStream out = new FileOutputStream(fout);
      try {
        byte[] buffer = new byte[1024];
        int len;
        while ((len = ingz.read(buffer)) >= 0) out.write(buffer,0,len);
      } catch(IOException e) {
        System.out.println("Exception catched by readExternal (5): " + e);
        throw new RuntimeException(e.toString());
      }
      filgz.delete();
      out.close();
      this.addr = 0;
      this.lrda = 0;
      try {
        this.open("NEW");
      } catch(Exception e) {
        System.out.println("Exception catched by readExternal (6): " + e);
        throw new RuntimeException(e.toString());
      }
    }
    this.access = 1;
  }
  private class Enumlcm implements Enumeration {
    private Jlcm obj;
    private boolean next = true;
    private String first = " ";
    private String current;
    Enumlcm(Jlcm o) {
      this.obj = o;
      try {
        o.lcminf();
      } catch(Exception e) {
        System.out.println("Exception successfully catched by new: " + e);
        throw new RuntimeException(e.toString());
      }
      if (o.isEmpty()) {
        this.next = false;
        return;
      }
      try {
        first = o.lcmnxt(first);
        current = first;
      } catch(Exception e) {
        System.out.println("Exception successfully catched by new: " + e);
        throw new RuntimeException(e.toString());
      }
    }
    public boolean hasMoreElements() {
      return this.next;
    }
    public Object nextElement() {
      if (this.next) {
        try {
          this.current = this.obj.lcmnxt(this.current);
        } catch(Exception e) {
          System.out.println("Exception successfully catched by new: " + e);
          throw new RuntimeException(e.toString());
        }
        this.next = !(this.current.equals(this.first));
        return this.current;
      } else {
        return " ";
      }
    }
  }
}
