package jargon;

/*--------------------------------*/
/* Java-Lifo bindings             */
/*--------------------------------*/

/*
Copyright (C) 2010 Ecole Polytechnique de Montreal

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.
*/

import java.io.*;
/**
 * This class is an implementation of the Java bindings for a last-in-first-out
 * (lifo) stack used with CLE-2000. Lifo management capabilities for a program
 * written in Java are available by using methods belonging to the Lifo class.
 * These methods encapsulate the lifo API calls using Java Native Interface (JNI).
 * <P> A Lifo object in Java can encapsulate a native lifo stack used to
 * manage CLE-2000 parameters.
 * <P> A lifo stack can contain defined or undefined (empty) nodes; used to represent
 * known or unknown parameters, respectively.
 * <P> <I>Note:</I> There is a constraint in CLE-2000. LCM (or XSM) objects and files
 * must be pushed before single-value nodes in the stack.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2010)
 *
 * @author Alain Hebert
 */
public class Lifo {
  /** Print "Lifo class loaded" and load LCM native libraries.
   */
  static {
    System.out.println("Lifo class loaded");
    System.loadLibrary("JlcmNat");
  }
  private static final long serialVersionUID = 1001L;
  private native void openNat();
  private native int closeNat();
  private native Object popNat();
  private native void pushNat(String sname, Object node, String stype, String josname);
  private native Object nodeNat(String sname);
  private native Object posNat(int ipos);
  private native String OSnodeNat(String sname);
  private native String OSposNat(int ipos);
  private native int typenodeNat(String sname);
  private native int typeposNat(int ipos);
  private native int maxNat();
  private native void libNat();
  private transient long addr;  /* address of the lifo stack in memory */
  /**
   * Use this constructor to create an empty Lifo object.
   */
  public Lifo() {
    try {
      openNat();
    } catch(Exception e) {
      System.out.println("Exception catched by open: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Pop the node on top of the Lifo stack. If the node is empty, a <tt>null</tt> is returned
   * @return node object or <tt>null</tt>
   */
  public Object pop() {
    try {
      return popNat();
    } catch(Exception e) {
      System.out.println("Exception catched by pop: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Return the node with a given name or <tt>null</tt>. The lifo stack is not modified
   * @param sname node name (limited to 12 characters)
   * @return node object
   */
  public Object node(String sname) {
    try {
      return nodeNat(sname);
    } catch(Exception e) {
      System.out.println("Exception catched by pop: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Return the node at a given position or <tt>null</tt>. The lifo stack is not modified
   * @param ipos node position in lifo stack
   * @return node object
   */
  public Object node(int ipos) {
    try {
      return posNat(ipos);
    } catch(Exception e) {
      System.out.println("Exception catched by pop: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Return the OSname of the node associated with a given name. The lifo stack is not modified
   * @param sname node name (limited to 12 characters)
   * @return node object
   */
  public String OSName(String sname) {
    try {
      return OSnodeNat(sname);
    } catch(Exception e) {
      System.out.println("Exception catched by OSName: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Return the OSname of the node at a given position. The lifo stack is not modified
   * @param ipos node position in lifo stack
   * @return node object
   */
  public String OSName(int ipos) {
    try {
      return OSposNat(ipos);
    } catch(Exception e) {
      System.out.println("Exception catched by OSName: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Return the type of the node associated with a given name. The lifo stack is not modified
   * @param sname node name (limited to 12 characters)
   * @return node type (3= LCM object; 4= XSM file; 5= seq binary; 6= seq ascii; 7= DA binary;
   11= integer value; 12= real value; 13= character string; 14= double precision value;
   15= logical value). A negative value indicates an empty node.
   */
  public int typeNode(String sname) {
    try {
      return typenodeNat(sname);
    } catch(Exception e) {
      System.out.println("Exception catched by typeName: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Return the type of the node at a given position. The lifo stack is not modified
   * @param ipos node position in lifo stack
   * @return node type (3= LCM object; 4= XSM file; 5= seq binary; 6= seq ascii; 7= DA binary;
   11= integer value; 12= real value; 13= character string; 14= double precision value;
   15= logical value). A negative value indicates an empty node.
   */
  public int typeNode(int ipos) {
    try {
      return typeposNat(ipos);
    } catch(Exception e) {
      System.out.println("Exception catched by typeName: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Push a new node object on top of the Lifo stack
   * @param sname node name (limited to 12 characters)
   * @param node node object to push into the lifo stack. If <tt>"node"</tt> is a
   * class, an empty node is pushed in the lifo stack in order to receive a calculation
   * result
   */
  public void push(String sname, Object node) {
    try {
      pushNat(sname, node, "not_applicable", null);
    } catch(Exception e) {
      System.out.println("Exception catched by push: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Push a new node object with a type on top of the Lifo stack
   * @param sname node name (limited to 12 characters)
   * @param node node object to push into the lifo stack. If <tt>"node"</tt> is a
   * class, an empty node is pushed in the lifo stack in order to receive a calculation
   * result
   * @param stype type of LCM object or file. This variable is chosen among
   * the following values:
   * <ul><dl>
   * <dt> <tt>"LCM"</tt> <dd> memory-resident LCM object (to be used if <tt>"node"</tt>
   * is a <tt>Jlcm</tt> class)
   * <dt> <tt>"XSM"</tt> <dd> XSM file (to be used if <tt>"node"</tt> is a <tt>Jlcm</tt> class)
   * <dt> <tt>"BINARY"</tt> <dd> binary sequential file
   * <dt> <tt>"ASCII"</tt> <dd> ACSII sequential file
   * <dt> <tt>"DA"</tt> <dd> binary direct-access file (with 128-word records)
   */
  public void push(String sname, Object node, String stype) {
    try {
      pushNat(sname, node, stype, null);
    } catch(Exception e) {
      System.out.println("Exception catched by push: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Push a new node object with a type and an <tt>"OSname"</tt> on top of the Lifo stack
   * @param sname node name (limited to 12 characters)
   * @param node node object to push into the lifo stack. If <tt>"node"</tt> is a class,
   * an empty node is pushed in the lifo stack in order to receive a calculation result
   * @param stype type of LCM object or file. This variable is chosen among
   * the following values:
   * <ul><dl>
   * <dt> <tt>"LCM"</tt> <dd> memory-resident LCM object (to be used if <tt>"node"</tt>
   * is a <tt>Jlcm</tt> class)
   * <dt> <tt>"XSM"</tt> <dd> XSM file (to be used if <tt>"node"</tt> is a <tt>Jlcm</tt> class)
   * <dt> <tt>"BINARY"</tt> <dd> binary sequential file
   * <dt> <tt>"ASCII"</tt> <dd> ACSII sequential file
   * <dt> <tt>"DA"</tt> <dd> binary direct-access file (with 128-word records)</ul>
   * @param OSname operating system (OS) name associated to the node (limited to 72 characters).
   * Can be given only for LCM (or XSM) objects or files.
   */
  public void push(String sname, Object node, String stype, String OSname) {
    try {
      pushNat(sname, node, stype, OSname);
    } catch(Exception e) {
      System.out.println("Exception catched by push: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Push a new integer on top of the Lifo stack
   * @param sname node name (limited to 12 characters)
   * @param my_integer integer value to push into the lifo stack
   */
  public void push(String sname, int my_integer) {
    try {
      pushNat(sname, new Integer(my_integer), "not_applicable", null);
    } catch(Exception e) {
      System.out.println("Exception catched by push: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Push a new float on top of the Lifo stack
   * @param sname node name (limited to 12 characters)
   * @param my_float float value to push into the lifo stack
   */
  public void push(String sname, float my_float) {
    try {
      pushNat(sname, new Float(my_float), "not_applicable", null);
    } catch(Exception e) {
      System.out.println("Exception catched by push: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Push a new double on top of the Lifo stack
   * @param sname node name (limited to 12 characters)
   * @param my_double double value to push into the lifo stack
   */
  public void push(String sname, double my_double) {
    try {
      pushNat(sname, new Double(my_double), "not_applicable", null);
    } catch(Exception e) {
      System.out.println("Exception catched by push: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Push a new boolean on top of the Lifo stack
   * @param sname node name (limited to 12 characters)
   * @param my_boolean boolean value to push into the lifo stack
   */
  public void push(String sname, boolean my_boolean) {
    try {
      pushNat(sname, new Boolean(my_boolean), "not_applicable", null);
    } catch(Exception e) {
      System.out.println("Exception catched by push: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Gives the number of nodes in the lifo stack
   * @return number of nodes in lifo stack
   */
  public int getMax() {
    try {
      return maxNat();
    } catch(Exception e) {
      System.out.println("Exception catched by getMax: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Print the table-of-content of a lifo stack
   */
  public void lib() {
    try {
      libNat();
    } catch(Exception e) {
      System.out.println("Exception catched by lib: " + e);
      throw new RuntimeException(e.toString());
    }
  }
  /** Destructor method.
   */
  protected void finalize() {
    int ier;
    while (this.getMax() != 0) {
      this.pop();
    }
    try {
      ier = closeNat();
    } catch(Exception e) {
      System.out.println("Exception catched by close: " + e);
      throw new RuntimeException(e.toString());
    }
    if (ier != 0) throw new RuntimeException("lifo stack close failure");
  }
}
