package jargon;

import java.io.*;
/**
 * The Macrolib operator is used to create a set of macroscopic cross
 * sections for all the mixtures present in the domain.
 * <P>
 * Here is an example of a BeanShell script to define a macrolib:
 * <PRE>
 * myMacro = new Macrolib("myMacro", 1, 1, 1);
 * // mixture 1:
 * water = new Composition("water", "medium");
 * myMacro.setTotal(water, new float[] {0.3683f});
 * myMacro.setScat(water, 0, 1, new float[] {0.3683f}, 1, 1);
 * // mixture 2:
 * fuel = new Composition("fuel", "medium");
 * myMacro.setTotal(fuel, new float[] {0.36522f});
 * myMacro.setNuSigf(fuel, new float[] {0.05564f});
 * myMacro.setChi(fuel, new float[] {1.0f});
 * myMacro.setScat(fuel, 0, 1, new float[] {0.3234f}, 1, 1);
 * // mixture 3:
 * clad = new Composition("clad", "medium");
 * myMacro.setTotal(clad1, new float[] {0.8453f});
 * myMacro.setScat(clad1, 0, 1, new float[] {0.5216f}, 1, 1);
 * </PRE>
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Macrolib extends Operator {
  static {
    System.out.println("Macrolib class loaded");
  }
  static String fr = "definition d'une macrolib";
  static String en = "definition of a macrolib";
  private static final long serialVersionUID = 2L;
  protected Composition[] mixs;
  private MixtureXS[] xs;
  private int ngro;
  private int nifi;
  private int nanis;
  /** type of transport correction that should be generated and stored on
   * the embedded LCM structure. All the operators that will use this
   * Macrolib object will then have access to transport corrected cross
   * sections. This variable is chosen among the following values:
   * <dl>
   * <dt> <tt>"APOL"</tt> <dd> an APOLLO-type transport correction based
   * on the linearly anisotropic scattering cross sections is to be used for
   * the total and isotropic scattering cross sections. This correction assumes
   * that the micro-reversibility principle is valid for all energy groups.
   * P1 scattering information must exist in the Macrolib object.
   * <dt> <tt>"WIMS"</tt> <dd> a WIMS-type transport correction is used.
   * The transport correction is recovered from object variable named
   * <tt>"tranc"</tt>.
   * </dl>
   */
  public String ctra;
  /** set to <tt>true</tt> to specify that the macroscopic scattering cross
   * sections and the fission spectrum have to be normalized.
   */
  public boolean norm;
  /** energy (eV) array which define the limits of the groups
   *(<tt>ngroup+1</tt> elements).  Generally, the lowest index is
   * used to represent the highest energy.
   */
  public float[] energy;
  
  /** set the macroscopic total cross sections in a medium.
   * @param medium Composition object containing an isotopic mixture
   * used as a reference to hold a set of macroscopic cross sections.
   * @param xs group-dependent cross sections
   */
  public void setTotal(Composition medium, float[] xs) throws JargonException {
    MixtureXS mixt = findMixture(medium);
    mixt.total = xs;
  }
  
  /** set the macroscopic transport correction cross sections in a medium.
   * @param medium Composition object containing an isotopic mixture
   * used as a reference to hold a set of macroscopic cross sections.
   * @param xs group-dependent cross sections
   */
  public void setTranc(Composition medium, float[] xs) throws JargonException {
    MixtureXS mixt = findMixture(medium);
    mixt.tranc = xs;
  }
  
  /** set the macroscopic fission cross sections multiplied by the
   * average number of neutron produced per fission in a medium.
   * @param medium Composition object containing an isotopic mixture
   * used as a reference to hold a set of macroscopic cross sections.
   * @param xs group-dependent cross sections
   */
  public void setNuSigf(Composition medium, float[] xs) throws JargonException {
    MixtureXS mixt = findMixture(medium);
    mixt.nusigf = xs;
  }
  
  /** set the fission spectrum in a medium.
   * @param medium Composition object containing an isotopic mixture
   * used as a reference to hold a set of macroscopic cross sections.
   * @param chi group-dependent fission spectrum
   */
  public void setChi(Composition medium, float[] chi) throws JargonException {
    MixtureXS mixt = findMixture(medium);
    mixt.chi = chi;
  }
  
  /** set the fixed sources in a medium.
   * @param medium Composition object containing an isotopic mixture
   * used as a reference to hold a set of macroscopic cross sections.
   * @param fixe group-dependent fixed sources
   */
  public void setFixe(Composition medium, float[] fixe) throws JargonException {
    MixtureXS mixt = findMixture(medium);
    mixt.fixe = fixe;
  }
  
  /** set the isotropic diffusion coefficients in a medium.
   * @param medium Composition object containing an isotopic mixture
   * used as a reference to hold a set of macroscopic cross sections.
   * @param diff group-dependent diffusion coefficients
   */
  public void setDiff(Composition medium, float[] diff) throws JargonException {
    MixtureXS mixt = findMixture(medium);
    mixt.diff = diff;
  }
  
  /** set the power factors in a medium.
   * @param medium Composition object containing an isotopic mixture
   * used as a reference to hold a set of macroscopic cross sections.
   * @param hFactor group-dependent power factors
   */
  public void setHFactor(Composition medium, float[] hFactor) throws JargonException {
    MixtureXS mixt = findMixture(medium);
    mixt.hFactor = hFactor;
  }
  
  /** set the Legendre order-ordered scattering information for each secondary
   * energy group <tt>jg</tt>.
   * @param medium Composition object containing an isotopic mixture
   * used as a reference to hold a set of macroscopic cross sections.
   * @param nanis Legendre order (0 for isotropic scattering).
   * @param jg secondary group index (1 for highest energy).
   * @param xs multigroup macroscopic scattering cross section from the
   * primary group <tt>ig</tt> towards the secondary group <tt>jg</tt>
   * considered for the current mixture and anisotropy level. The elements
   * are ordered using decreasing primary group index <tt>ig</tt>, from
   * <tt>ijj</tt> to <tt>ijj - njj + 1</tt>.
   * @param njj number of primary groups <tt>ig</tt> with non vanishing
   * macroscopic scattering cross section towards the secondary group
   * <tt>jg</tt>.
   * @param ijj group index of the most thermal group with non-vanishing
   * macroscopic scattering cross section towards the secondary group
   * <tt>jg</tt>.
   */
  public void setScat(Composition medium, int nanis, int jg, float[] xs, int njj, int ijj)
  throws JargonException {
    MixtureXS mixt = findMixture(medium);
    mixt.scat[nanis][jg-1].xscat = xs;
    mixt.scat[nanis][jg-1].njj = njj;
    mixt.scat[nanis][jg-1].ijj = ijj;
  }
  
  private MixtureXS findMixture(Composition medium) throws JargonException {
    if (this.mixs == null) {
      this.mixs = new Composition[1];
      this.mixs[0] = medium;
      this.xs = new MixtureXS[1];
      this.xs[0] = new MixtureXS();
      return this.xs[0];
    } else {
      for (int i=0; i<this.mixs.length; i++) {
        if(this.mixs[i].name.equals(medium.name)) {
          return this.xs[i];
        }
      }
      Composition[] work1 = this.mixs;
      this.mixs = new Composition[work1.length+1];
      for (int i=0;i<work1.length;i++) this.mixs[i] = work1[i];
      this.mixs[work1.length] = medium;
      MixtureXS[] work2 = this.xs;
      this.xs = new MixtureXS[work2.length+1];
      for (int i=0;i<work2.length;i++) this.xs[i] = work2[i];
      this.xs[work2.length] = new MixtureXS();
      return this.xs[work2.length];
    }
  }
  private class MixtureXS implements Serializable {
    private static final long serialVersionUID = 3L;
    private float[] total;
    private float[] tranc;
    private float[] nusigf;
    private float[] chi;
    private float[] fixe;
    private float[] diff;
    private float[] hFactor;
    private Scattering[][] scat;
    
    private MixtureXS() {
      this.scat = new Scattering[nanis][ngro];
      for (int j=0;j<nanis;j++) {
        for (int k=0;k<ngro;k++) this.scat[j][k] = new Scattering();
      }
    }
    
    private class Scattering implements Serializable {
      private static final long serialVersionUID = 4L;
      private int njj;
      private int ijj;
      private float[] xscat;
    }
  }

  /** 
   * Use this constructor to create a new Macrolib object.
   * @param name user-defined name of the Macrolib object
   * @param ngro number of energy groups for which the macroscopic cross
   * sections will be provided.
   * @param nifi number of fissile spectrum associated with each mixture.
   * Each fission spectrum generally represents a fissile isotope.
   * @param nanis maximum level of anisotropy permitted in the scattering
   * cross sections.
   */
  public Macrolib(String name, int ngro, int nifi, int nanis) {
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
    this.ngro = ngro;
    this.nifi = nifi;
    this.nanis = nanis;
  }
  /** load the embedded lcmObj object with the content of this Macrolib.
   */
  public void exec() throws JargonException {
    System.out.println("macrolib.exec: aa");
    super.resetJlcm();
    System.out.println("macrolib.exec: bb");
    super.outputJlcm(this.lcmObj);
    System.out.println("macrolib.exec: cc");
    super.exec();
    System.out.println("macrolib.exec: dd");
  }
  protected void makeText() {
    this.operator="MAC: ";
    super.write("EDIT", this.edit);
    super.write("NGRO", this.ngro);
    super.write("NMIX", this.mixs.length);
    super.write("NIFI", this.nifi);
    super.write("ANIS", this.nanis);
    if (this.ctra != null) {
      super.write("\nCTRA", this.ctra);
    }
    if (this.norm)  super.write("\nNORM");
    if (this.energy != null) {
      super.write("\nENER", this.energy);
    }
    super.write("\nREAD INPUT");
    for (int imix=0;imix<this.mixs.length;imix++) {
      if (this.xs[imix].total == null) continue;
      super.write("\nMIX"); super.write(imix+1);
      super.write("TOTAL"); super.write(this.xs[imix].total);
      if (this.xs[imix].diff != null) {
        super.write("DIFF", this.xs[imix].diff);
      }
      if (this.xs[imix].tranc != null) {
        super.write("TRANC", this.xs[imix].tranc);
      }
      if (this.xs[imix].nusigf != null) {
        super.write("NUSIGF", this.xs[imix].nusigf);
      }
      if (this.xs[imix].chi != null) {
        super.write("CHI", this.xs[imix].chi);
      }
      if (this.xs[imix].fixe != null) {
        super.write("FIXE", this.xs[imix].fixe);
      }
      if (this.xs[imix].hFactor != null) {
        super.write("H-FACTOR", this.xs[imix].hFactor);
      }
      if (this.xs[imix].scat != null) {
        super.write("SCAT");
        for (int is=0;is<this.nanis;is++) {
          for (int js=0;js<this.ngro;js++) {
            super.write(this.xs[imix].scat[is][js].njj);
            super.write(this.xs[imix].scat[is][js].ijj);
            super.write(this.xs[imix].scat[is][js].xscat);
          }
        }
      }
    }
    super.write(";");
  }
}
