package jargon;

/**
 * The Mccg operator is a specialization of the Excell tracking operator
 * used produces a Jlcm structure containing the energy-independent
 * numerotation information required to solve the transport equation over
 * a 2D or 3D domain using the method of characteristics MCCG of Igor Suslov.
 * The tracking file produced by Excell is used together with additional
 * data related to the Suslov technique.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002).
 * The embedded Fortran native application was written by Igor Suslov, Alain
 * Hebert and Romain Le Tellier.
 *
 * @author Alain Hebert
 */
public class Mccg extends Excell {
  static {
    System.out.println("Mccg class loaded");
  }
  static String fr = "definition d'un tracking Mccg";
  static String en = "definition of an Mccg tracking";
  private static final long serialVersionUID = 45L;
  /** type of quadrature used to perform polar integration with the method
   * of characteristics. This variable is chosen among the following values:
   * <dl>
   * <dt> <tt>"LCMD"</tt> <dd> optimized (McDaniel type) polar integration
   * angles are to be selected. This is the default option.
   * <dt> <tt>"GAUS"</tt> <dd> Gauss polar integration angles are to be
   * selected.
   * <dt> <tt>"CACA"</tt> <dd> Cactus type equal weight polar integration
   * angles are to be selected.
   * <dt> <tt>"CACB"</tt> <dd> Cactus type uniformly distributed integration
   * polar angles are to be selected.
   * </dl>
   */
  public String moc;
  /** maximum number of inner (one-speed) iterations.
   */
  public int innerMax = 0;
  /** convergence criterion for flux components in the inner iteration.
   */
  public float innerEps = 0.0f;
  /** set to <tt>true</tt>/<tt>false</tt> to activate/desactivate the
   * ACA preconditioning of inner iterations in case where a transport
   * solution is selected.
   */
  public boolean aca = true;
  /** set to <tt>true</tt>/<tt>false</tt> to activate/desactivate the SCR
   * preconditioning of inner iterations in case where a transport solution
   * is selected.
   */
  public boolean scr = false;
  /** dimension of the Krylov subspace for the GMRES (if <tt>krylov</tt>>0)
   * or Bi-CGSTAB (if <tt>krylov</tt><0) acceleration of inner iterations.
   * Setting <tt>krylov = 0</tt> desactivate the Krylov subspace acceleration.
   */
  public int krylov = 10;
  /** set to <tt>true</tt> to use an ACA diffusion flux solution instead
   * of a transport solution. No inner iterations are performed in this case.
   */
  public boolean diffusion = false;
  /** step characteristics criterion. Set to <tt>hdd = 0.0</tt> to use
   * the step characteristics method. Otherwise, use the diamond differencing
   * scheme with a fixup related to <tt>hdd</tt>. Using a value of <tt>hdd</tt>
   * greater than zero may cause the failure of the GMRES acceleration, which
   * is not compatible with a <i>random</i> fixup of the flux. The only way to
   * use diamond differencing in a way consistent with GMRES acceleration
   * is to set a very high value of <tt>hdd</tt> (e.g., <tt>hdd = 2000.0</tt>).
   */
  public float hdd = 0.0f;
  /**
   * Use this constructor to create a new Mccg object.
   * @param name user-defined name of the Mccg object
   */
  public Mccg(String name) {
    super(name);
  }
  /** load the embedded lcmObj structure corresponding to this Mccg tracking.
   * @param a 2D or 3D geometry to analyse.
   */
  public void exec(Gigogne a) throws JargonException {
    super.makeText();
    super.exec(a);
    this.operator="MCCGT: ";
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(this.lcmObj);
    super.inputJlcm(this.fileObj);
    super.inputJlcm(a.lcmObj);
    super.exec2();
  }
  public void text() {
    super.text();
    super.text2();
  }
  protected void makeText2() {
    if (this.moc != null) {
      if ((!this.moc.equals("LCMD")) & (!this.moc.equals("GAUS"))
        & (!this.moc.equals("CACA")) & (!this.moc.equals("CACB"))) {
        throw new RuntimeException("invalid type (" + this.moc + ")");
      }
      super.write(this.moc);
    }
    if (this.innerMax != 0) super.write("\nMAXI", this.innerMax);
    if (this.innerEps != 0.0f) super.write("\nEPSI", this.innerEps);
    if (this.aca) super.write("\nAAC", 1);
    if (!this.aca) super.write("\nAAC", 0);
    if (this.scr) super.write("\nSCR", 1);
    if (!this.scr) super.write("\nSCR", 0);
    super.write("\nKRYL", this.krylov);
    if (this.diffusion) super.write("\nDIFC", 1);
    if (this.hdd != 0.0f) super.write("\nHDD", this.hdd);
    super.write(";");
  }
}
