package jargon;

import java.util.Vector;
/**
 * The Microlib operator produces a Jlcm structure containing the
 * microscopic cross section and library data required in a lattice
 * calculation. This data is recovered and interpolated from specific
 * cross section libraries. Instances of three classes (Microlib,
 * Composition and Isotope) are required to define the  media present
 * in the lattice, as shown in the following UML diagram:
 * <P><img src="../../src/figures/mic_umlj.gif">
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Microlib extends Operator {
  static {
    System.out.println("Microlib class loaded");
  }
  static String fr = "definition d'une microlib";
  static String en = "definition of a microlib";
  private static final long serialVersionUID = 30L;
  /** type of microlib processing. This variable is chosen among the
   * following values:
   * <dl>
   * <dt> <tt>"SKIP"</tt> <dd> recover the user-defined microlib data
   * without processing any library (i.e., without temperature and/or
   * dilution interpolation).
   * <dt> <tt>"INTR"</tt> <dd> temperature and dilution interpolation
   * of the microscopic cross sections present in the libraries. The bin-type
   * cross-section data is not processed (default option).
   * <dt> <tt>"SUBG"</tt> <dd> calculation of the physical probability
   * tables using the temperature-interpolated cross-section data as
   * input. The bin-type cross-section data is not processed.
   * <dt> <tt>"PTSL"</tt> <dd> calculation of the CALENDF-type mathematical
   * probability tables using the bin-type cross-section data as input.
   * <dt> <tt>"NEWL"</tt> <dd> calculation of a DRAGLIB library using the
   * temperature-interpolated cross-section data as input. The bin-type
   * cross-section data is processed.
   * </dl>
   */
  public String type;
  /** type of transport correction. This variable is chosen among the
   * following values:
   * <dl>
   * <dt> <tt>"NONE"</tt> <dd> no transport correction should be used
   * in this calculation (default option)
   * <dt> <tt>"APOL"</tt> <dd> APOLLO-type transport correction based
   * on the linearly anisotropic scattering cross sections is to be used
   * for the total and isotropic scattering cross sections. This correction
   * assumes that the micro-reversibility principle is valid for all energy
   * groups. This type of correction uses P1 scattering information present
   * on the library.
   * <dt> <tt>"WIMS"</tt> <dd> transport correction is to be used for the
   * total and isotropic scattering cross sections. This type of correction
   * uses directly a transport-correction cross section provided on the library.
   * <dt> <tt>"OLDW"</tt> <dd> WIMS-type transport correction based on the
   * linearly anisotropic scattering cross sections is to be used for the
   * total and isotropic scattering cross sections. This correction
   * assumes that the micro-reversibility principle is valid only for
   * groups energies less than 4.0 eV. For the remaining groups a 1/E
   * current spectrum is considered in the evaluation of the transport
   * correction. This type of correction uses P1 scattering information
   * present on the library.
   * </dl>
   */
  public String ctra;
  protected int nmix;
  /** radioactive/burnup decay chain.
   */
  public String chain;
  /** option selection. This variable is chosen among the following values:
   * <dl>
   * <dt> <tt>"ADJ"</tt> <dd> production of adjoint macroscopic cross sections.
   * By default, direct cross sections are produced.
   * <dt> <tt>"PROM"</tt> <dd> prompt neutrons are to be considered for the
   * calculation of the fission spectrum. By default, the contribution due to
   * delayed neutrons is considered. This option is only compatible with a
   * MATXS or MATXS2 format library.
   * <dt> <tt>"MACR"</tt> <dd> force the calculation of the embedded macrolib.
   * By default, the embedded macrolib is computed, <i>except if</i> one of
   * the key words <tt>SKIP</tt>, <tt>INTR</tt>, <tt>SUBG</tt>, <tt>PTSL</tt>
   * or <tt>NEWL</tt> is used.
   * </dl>
   */
  public String[] options;
  /** array of Composition objects describing the medium-ordered information
   * required to recover and interpolate the microscopic cross section and
   * and library data.
   */
  public Composition[] mixs;
  /** 
   * Use this constructor to create a Microlib object of a specified type.
   * @param name user-defined name of the Microlib object
   * @param type type of microlib processing. This variable is chosen among
   * the following values: <tt>"SKIP"</tt>, <tt>"INTR"</tt>, <tt>"SUBG"</tt>,
   * <tt>"PTSL"</tt> or <tt>"NEWL"</tt>.
   */
  public Microlib(String name, String type) {
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
    if ((!type.equals("SKIP"))&(!type.equals("INTR"))&
       (!type.equals("SUBG"))&(!type.equals("PTSL"))&
       (!type.equals("NEWL"))) {
      throw new RuntimeException("invalid type (" + type + ")");
    }
    this.type = type;
  }
  /** 
   * Use this constructor to create a Microlib object of undefined type.
   * @param name user-defined name of the Microlib object
   */
  public Microlib(String name) {
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
  }
  /** load the embedded lcmObj object corresponding to this Microlib.
   */
  public void exec() throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.exec();
  }
  /** update the embedded lcmObj of an existing Microlib.
   * @param a existing Microlib object
   */
  public void exec(Microlib a) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.exec();
  }
  /** transform an arbitrary Composition array into a Composition
   * array of type medium
   */
  private Composition[] makeMedium() {
    double avoga = 0.6022094; // Avogadro number
    String[] listisot = new String[]{
     "H2O"     ,"D2O"     ,"BE9"     ,"CH2"     ,"H_ZRH"   ,"ZR_ZRH"  ,
     "PB"      ,"U238"    ,"U237"    ,"U236"    ,"U235"    ,"U234"    ,
     "U233"    ,"U232"    ,"TH228"   ,"TH230"   ,"TH232"   ,"PA231"   ,
     "PA233"   ,"PU236"   ,"PU238"   ,"PU239"   ,"PU240"   ,"PU241"   ,
     "PU242"   ,"NP237"   ,"NP238"   ,"NP239"   ,"AM241"   ,"AM242M"  ,
     "AM243"   ,"CM242"   ,"CM243"   ,"CM244"   ,"CM245"   ,"CM246"   ,
     "CM247"   ,"AG107"   ,"AG109"   ,"ZRNAT"   ,"ZR90"    ,"ZR91"    ,
     "ZR92"    ,"ZR94"    ,"ZR96"    ,"SM144"   ,"SM147"   ,"SM148"   ,
     "SM149"   ,"SM150"   ,"SM152"   ,"SM154"   ,"GD152"   ,"GD153"   ,
     "GD154"   ,"GD155"   ,"GD156"   ,"GD157"   ,"GD158"   ,"GD160"   ,
     "HF174"   ,"HF176"   ,"HF177"   ,"HF178"   ,"HF179"   ,"HF180"   ,
     "EU151"   ,"EU152"   ,"EU152M"  ,"EU153"   ,"IN115"   ,"ER162"   ,
     "ER164"   ,"ER166"   ,"ER167"   ,"ER168"   ,"DY160"   ,"DY161"   ,
     "DY162"   ,"DY163"   ,"DY164"   ,"DY165"   ,"GRAPHITE","H1"      ,
     "H2"      ,"H3"      ,"HE3"     ,"HE4"     ,"LI6"     ,"LI7"     ,
     "B10"     ,"B11"     ,"C"       ,"N14"     ,"O16"     ,"F19"     ,
     "NA23"    ,"MG"      ,"AL27"    ,"SI"      ,"P31"     ,"S32"     ,
     "S33"     ,"S34"     ,"CL"      ,"K"       ,"CA"      ,"TINAT"   ,
     "V"       ,"CR50"    ,"CR52"    ,"CR53"    ,"CR54"    ,"MN55"    ,
     "FE54"    ,"FE56"    ,"FE57"    ,"FE58"    ,"CO59"    ,"NI58"    ,
     "NI60"    ,"NI61"    ,"NI62"    ,"NI64"    ,"CU"      ,"ZN64"    ,
     "NB93"    ,"MO"      ,"MO92"    ,"MO94"    ,"MO95"    ,"MO96"    ,
     "MO97"    ,"MO98"    ,"CD106"   ,"CD110"   ,"CD111"   ,"CD112"   ,
     "CD113"   ,"CD114"   ,"CD116"   ,"SN112"   ,"SN114"   ,"SN115"   ,
     "SN116"   ,"SN117"   ,"SN118"   ,"SN119"   ,"SN120"   ,"SN122"   ,
     "SN124"   ,"TB159"   ,"HO165"   ,"TA181"   ,"W182"    ,"W183"    ,
     "W184"    ,"W186"    ,"BI209"   ,"ABS1SV"  ,"DIFEQ1"  ,"KR83PF"  ,
     "ZR93PF"  ,"ZR95PF"  ,"ZR96PF"  ,"NB95PF"  ,"MO95PF"  ,"MO96PF"  ,
     "MO97PF"  ,"MO98PF"  ,"MO99PF"  ,"MO100PF" ,"TC99PF"  ,"RU101PF" ,
     "RU102PF" ,"RU103PF" ,"RU104PF" ,"RU105PF" ,"RU106PF" ,"RH103PF" ,
     "RH105PF" ,"PD105PF" ,"PD106PF" ,"PD107PF" ,"PD108PF" ,"AG109PF" ,
     "CD113PF" ,"IN115PF" ,"TE127MPF","TE129MPF","I127PF"  ,"I129PF"  ,
     "I131PF"  ,"I135PF"  ,"XE131PF" ,"XE132PF" ,"XE133PF" ,"XE135PF" ,
     "XE136PF" ,"CS133PF" ,"CS134PF" ,"CS135PF" ,"CS136PF" ,"CS137PF" ,
     "LA139PF" ,"CE141PF" ,"CE144PF" ,"PR141PF" ,"PR143PF" ,"ND143PF" ,
     "ND144PF" ,"ND145PF" ,"ND146PF" ,"ND147PF" ,"ND148PF" ,"ND150PF" ,
     "PM147PF" ,"PM148PF" ,"PM148MPF","PM149PF" ,"PM151PF" ,"SM147PF" ,
     "SM148PF" ,"SM149PF" ,"SM150PF" ,"SM151PF" ,"SM152PF" ,"SM153PF" ,
     "SM154PF" ,"EU153PF" ,"EU154PF" ,"EU155PF" ,"EU156PF" ,"EU157PF" ,
     "GD154PF" ,"GD155PF" ,"GD156PF" ,"GD157PF" ,"PSU5U"   ,"PSU8U"   ,
     "PSP9U"   ,"PSP0U"   ,"PSP1U"   ,"PSP2U"   ,"PSA1U"   ,"PSA2U"   ,
     "RH105PFS","XE135PFS","SM149PFS","TC99"    ,"RH103"   ,"CS133"   ,
     "ND145"   ,"BIDON"   ,"BNAT"    ,"CNAT"};
    double[] massisot = new double[]{
       18.0105, 20.0231,  9.0122, 14.0267,  1.0078, 91.2196,207.2101,
     238.0510,237.0484,236.0458,235.0442,234.0405,233.0450,232.0333,
     228.0289,230.0361,232.0333,231.0347,233.0399,236.0458,238.0496,
     239.0526,240.0542,241.0568,242.0584,237.0484,238.0510,239.0526,
     241.0568,242.0604,243.0610,242.0584,243.0610,244.0626,245.0652,
     246.0668,247.0725,106.9054,108.9045, 91.2240, 89.9047, 90.9059,
      91.9050, 93.9063, 95.9079,143.9163,146.9151,147.9147,148.9173,
     149.9169,151.9201,153.9223,151.9201,152.9218,153.9213,154.9229,
     155.9221,156.9241,157.9247,159.9269,173.9402,175.9404,176.9400,
     177.9396,178.9503,179.9499,150.9195,151.9251,151.9218,152.9207,
     114.9041,161.8907,163.9081,165.9304,166.9320,167.9427,159.9249,
     160.9275,161.9270,162.9286,163.9282,164.9318, 12.0110,  1.0078,
       2.0141,  3.0160,  3.0160,  4.0362,  6.0151,  7.0160, 10.0129,
      11.0093, 12.0110, 14.0033, 15.9949, 18.9982, 22.9898, 24.3051,
      26.9818, 28.0853, 30.9740, 31.9720, 32.9710, 33.9680, 35.4526,
      39.1019, 40.0803, 47.8684, 50.9416, 49.9460, 51.9400, 52.9402,
      53.9384, 54.9380, 53.9394, 55.9350, 56.9351, 57.9337, 58.9332,
      57.9357, 59.9308, 60.9314, 61.9280, 63.9282, 63.5398, 63.9291,
      92.9032, 95.9400, 91.9068, 93.9051, 94.9100, 95.9047, 96.9100,
      97.9051,105.9098,109.9031,110.9047,111.9033,112.8999,113.9035,
     115.9047,111.9053,113.8985,114.9031,115.9017,116.9033,117.9018,
     118.9035,119.9020,121.9032,123.9054,158.9253,164.9308,180.9545,
     181.9531,182.9517,183.9502,185.9575,208.9803,  1.0087,  1.0087,
      82.9141, 92.9061, 94.9080, 95.9079, 94.9068, 94.9100, 95.9047,
      96.9100, 97.9051, 98.9078, 99.9073, 98.9063,100.9058,101.9044,
     102.9063,103.9056,104.9082,105.9078,102.9050,104.9062,104.9012,
     105.9038,106.9054,107.9040,108.9045,112.8999,114.9041,126.9052,
     128.9074,126.9042,128.9054,130.9066,134.9100,130.9055,131.9041,
     132.9057,134.9069,135.9075,132.9037,133.9073,134.9090,135.9075,
     136.9071,138.9063,140.9083,143.9133,140.9085,142.9107,142.9077,
     143.9103,144.9129,145.9135,146.9161,147.9167,149.9209,146.9151,
     147.9177,147.9177,148.9183,150.9215,146.9151,147.9147,148.9173,
     149.9169,150.9165,151.9201,152.9217,153.9223,152.9207,153.9223,
     154.9229,155.9245,156.9251,153.9213,154.9229,155.9221,156.9241,
       1.0087,  1.0087,  1.0087,  1.0087,  1.0087,  1.0087,  1.0087,
       1.0087,104.9062,134.9069,148.9173, 98.9063,102.9050,132.9037,
     144.9129,  1.0087, 10.8110, 12.0000};

    Vector<Composition> mediumObj = new Vector<Composition>();
    for (int imix=0; imix<this.mixs.length; imix++) {
      Composition medium = this.mixs[imix];
      if (!mediumObj.contains(medium)) mediumObj.addElement(medium);
      if (medium.isoCompo != null) {
        for (int icI=0; icI<medium.isoCompo.length; icI++) {
          Composition.CompoInner icomp = medium.isoCompo[icI];
          if (icomp.composition != null) mediumObj.addElement(icomp.composition);
        }
      } else if (medium.alias != null) {
        mediumObj.addElement(medium.alias);
      }
    }

    int nbmili = 0;
    int nbmela = 0;
    int nbmate = 0;
    int nmil = 0;
    int nmel = 0;
    int nmat = 0;
    Vector<Object> mela = new Vector<Object>();
    Vector<Object> mate = new Vector<Object>();
    Vector<Composition> mili = new Vector<Composition>();
    Vector<Object> melacomp = new Vector<Object>();
    Vector<Float> melaval = new Vector<Float>();
    Vector<Integer> melanum = new Vector<Integer>(); melanum.addElement(new Integer(1));
    Vector<Object> matecomp = new Vector<Object>();
    Vector<Float> mateval = new Vector<Float>();
    Vector<Integer> matenum = new Vector<Integer>(); matenum.addElement(new Integer(1));
    Vector<Object> milicomp = new Vector<Object>();
    Vector<Float> milival = new Vector<Float>();
    Vector<Integer> milinum = new Vector<Integer>(); milinum.addElement(new Integer(1));
    Vector<Object> comptemp;
    Vector<Float> valtemp;
    Vector<Object> stemp;
    int[] numtemp;
    for (int i=0; i<mediumObj.size(); i++) {
      Composition item = mediumObj.elementAt(i);
      if (item.type.equals("mixture")) {
        nbmela++;
        mela.addElement(item);
        for (int j=0; j<item.isoCompo.length; j++) {
          nmel++;
          if (item.isoCompo[j].isotope != null) {
            melacomp.addElement(item.isoCompo[j].isotope);
          } else if (item.isoCompo[j].composition != null) {
            melacomp.addElement(item.isoCompo[j].composition);
          }
          melaval.addElement(new Float(item.isoCompo[j].concentration));
        }
        melanum.addElement(new Integer(nmel+1));
        if (item.norm) {
          float sum = 0.0f;
          for (int j=0; j<item.isoCompo.length; j++) {
            sum+=item.isoCompo[j].concentration;
          }
          for (int j=0; j<item.isoCompo.length; j++) {
            item.isoCompo[j].concentration/=sum;
          }
        }
      } else if (item.type.equals("material")) {
        nbmate++;
        mate.addElement(item);
        if (item.isoCompo != null) {
          for (int j=0; j<item.isoCompo.length; j++) {
            nmat++;
            if (item.isoCompo[j].isotope != null) {
              matecomp.addElement(item.isoCompo[j].isotope);
            } else if (item.isoCompo[j].composition != null) {
              matecomp.addElement(item.isoCompo[j].composition);
            }
            mateval.addElement(new Float(item.isoCompo[j].concentration));
          }
        } else if (item.alias != null) {
          nmat++;
          matecomp.addElement(item.alias);
          mateval.addElement(new Float(item.alias.massDensity));
        }
        matenum.addElement(new Integer(nmat+1));
      } else if (item.type.equals("medium")) {
        nbmili++;
        mili.addElement(item);
        if (item.isoCompo != null) {
          for (int j=0; j<item.isoCompo.length; j++) {
            nmil++;
            milicomp.addElement(item.isoCompo[j].isotope);
            milival.addElement(new Float(item.isoCompo[j].concentration));
          }
        } else if (item.alias != null) {
          nmil++;
          milicomp.addElement(item.alias);
          milival.addElement(new Float(0.0f));
        }
        milinum.addElement(new Integer(nmil+1));
      }
    }
    //
    // reorder the mixture list in increasing order of definition
    int ind=1;
    boolean mord=true;
    int[] ntemp = new int[melacomp.size()];
    for (int i=0; i<melacomp.size(); i++) {
      ntemp[i] = mela.indexOf(melacomp.elementAt(i)) + 1;
    }
    stemp = new Vector<Object>();
    while (mord) {
      for (int i=0; i<nbmela; i++) {
        Composition item = (Composition)mela.elementAt(i);
        int ik = (melanum.elementAt(i)).intValue();
        int numelaf = (melanum.elementAt(i+1)).intValue() - 1;
        boolean mstock = !stemp.contains(item);
        while (mstock & ik<=numelaf) {
          if (ntemp[ik-1]>0) {
            mstock = stemp.contains(melacomp.elementAt(ik-1));
          } else {
            mstock = true;
          }
          ik++;
        }
        if (mstock) {
          stemp.addElement(item);
          ind++;
        }
      }
      if (ind-1 == nbmela) mord=false;
    }
    ind = 1;
    numtemp = new int[stemp.size()+1]; numtemp[0] = 1;
    comptemp = new Vector<Object>();
    valtemp = new Vector<Float>();
    for (int i=0; i<stemp.size(); i++) {
      Composition item = (Composition)stemp.elementAt(i);
      int iold = mela.indexOf(item);
      int numelad = (melanum.elementAt(iold)).intValue();
      int numelaf = (melanum.elementAt(iold+1)).intValue() - 1;
      for (int ikl=numelad; ikl<numelaf+1; ikl++) {
        comptemp.addElement(melacomp.elementAt(ikl-1));
        valtemp.addElement(melaval.elementAt(ikl-1));
        ind++;
      }
      numtemp[i+1] = ind;
    }
    //
    // copy in original Vectors
    melacomp.removeAllElements();
    melaval.removeAllElements();
    int mySize = (melanum.elementAt(nbmela)).intValue();
    for (int i=0; i<mySize-1; i++) {
      melacomp.addElement(comptemp.elementAt(i));
      melaval.addElement(valtemp.elementAt(i));
    }
    mela.removeAllElements();
    melanum.removeAllElements();
    for (int i=0; i<nbmela; i++) {
      mela.addElement(stemp.elementAt(i));
      melanum.addElement(new Integer(numtemp[i]));
    }
    melanum.addElement(new Integer(numtemp[nbmela]));
    //
    // reorder the material list in increasing order of definition
    ind = 1;
    mord = true;
    ntemp = new int[matecomp.size()];
    for (int i=0; i<matecomp.size(); i++) {
      ntemp[i] = mate.indexOf(matecomp.elementAt(i)) + 1;
    }
    stemp = new Vector<Object>();
    while (mord) {
      for (int i=0; i<nbmate; i++) {
        Composition item = (Composition)mate.elementAt(i);
        int ik = (matenum.elementAt(i)).intValue();
        int numatef = (matenum.elementAt(i+1)).intValue() - 1;
        boolean mstock = !stemp.contains(item);
        while (mstock & ik<=numatef) {
          if (ntemp[ik-1]>0) {
            mstock = stemp.contains(matecomp.elementAt(ik-1));
          } else {
            mstock = true;
          }
          ik++;
        }
        if (mstock) {
          stemp.addElement(item);
          ind++;
        }
      }
      if (ind-1 == nbmate) mord=false;
    }
    ind = 1;
    numtemp = new int[stemp.size()+1]; numtemp[0] = 1;
    comptemp = new Vector<Object>();
    valtemp = new Vector<Float>();
    for (int i=0; i<stemp.size(); i++) {
      Composition item = (Composition)stemp.elementAt(i);
      int iold = mate.indexOf(item);
      int numated = (matenum.elementAt(iold)).intValue();
      int numatef = (matenum.elementAt(iold+1)).intValue() - 1;
      for (int ikl=numated; ikl<numatef+1; ikl++) {
        comptemp.addElement(matecomp.elementAt(ikl-1));
        valtemp.addElement(mateval.elementAt(ikl-1));
        ind++;
      }
      numtemp[i+1] = ind;
    }
    //
    // copy in original Vectors
    matecomp.removeAllElements();
    mateval.removeAllElements();
    mySize = (matenum.elementAt(nbmate)).intValue();
    for (int i=0; i<mySize-1; i++) {
      matecomp.addElement(comptemp.elementAt(i));
      mateval.addElement(valtemp.elementAt(i));
    }
    mate.removeAllElements();
    matenum.removeAllElements();
    for (int i=0; i<nbmate; i++) {
      mate.addElement(stemp.elementAt(i));
      matenum.addElement(new Integer(numtemp[i]));
    }
    matenum.addElement(new Integer(numtemp[nbmate]));
    //
    // process the mixtures made of mixtures to obtain a mixture list
    // exclusively made of isotopes
    ind = 1;
    numtemp = new int[nbmela+1]; numtemp[0] = 1;
    float[] massmel = new float[nbmela];
    float[] propmel = new float[nbmela];
    comptemp = new Vector<Object>();
    valtemp = new Vector<Float>();
    for (int i=0; i<nbmela; i++) {
      int numelad = (melanum.elementAt(i)).intValue();
      int numelaf = (melanum.elementAt(i+1)).intValue() - 1;
      float rmassm = 0.0f;
      float propm = 0.0f;
      if (((Composition)mela.elementAt(i)).method.equals("number_fract")) {
        for (int k=numelad; k<numelaf+1; k++) {
          Isotope item = (Isotope)melacomp.elementAt(k-1);
          if (mela.contains(item)) {
            // the mixture is already transformed in isotope components
            int kk = mela.indexOf(item);
            int itempd = numtemp[kk];
            int itempf = numtemp[kk+1] - 1;
            float value = (melaval.elementAt(k-1)).floatValue();
            rmassm += value * massmel[kk];
            propm += value;
            for (int kkk=itempd; kkk<itempf+1; kkk++) {
              comptemp.addElement(comptemp.elementAt(kkk-1));
              float val3 = (valtemp.elementAt(kkk-1)).floatValue();
              valtemp.addElement(new Float(val3 * value));
              ind++;
            }
          } else {
            comptemp.addElement(item);
            valtemp.addElement(melaval.elementAt(k-1));
            int mm = -1;
            for (int j=0; j<listisot.length; j++) {
              if (item.symbol.equals(listisot[j])) {
                mm = j;
                break;
              }
            }
            if (mm == -1) throw new RuntimeException("component " + item.symbol + " is unknown");
            float val3 = (valtemp.elementAt(ind-1)).floatValue();
            rmassm += val3 * massisot[mm];
            propm += val3;
            ind++;
          }
        }
        numtemp[i+1] = ind;
        massmel[i] = rmassm;
        propmel[i] = propm;
      } else if (((Composition)mela.elementAt(i)).method.equals("mass_fract")) {
        for (int k=numelad; k<numelaf+1; k++) {
          Isotope item = (Isotope)melacomp.elementAt(k-1);
          float value = (melaval.elementAt(k-1)).floatValue();
          if (mela.contains(item)) {
            // the mixture is already transformed in isotope components
            int kk = mela.indexOf(item);
            int itempd = numtemp[kk];
            int itempf = numtemp[kk+1] - 1;
            rmassm += value / massmel[kk];
            propm += value;
            for (int kkk=itempd; kkk<itempf+1; kkk++) {
              comptemp.addElement(comptemp.elementAt(kkk-1));
              float val3 = (valtemp.elementAt(kkk-1)).floatValue();
              valtemp.addElement(new Float(val3 * value / massmel[kk]));
              ind++;
            }
          } else {
            comptemp.addElement(item);
            int mm = -1;
            for (int j=0; j<listisot.length; j++) {
              if (item.symbol.equals(listisot[j])) {
                mm = j;
                break;
              }
            }
            if (mm == -1) throw new RuntimeException("component " + item.symbol + " is unknown");
            valtemp.addElement(new Float((float)(value / massisot[mm])));
            rmassm += value / massisot[mm];
            propm += value;
            ind++;
          }
        }
        numtemp[i+1] = ind;
        massmel[i] = propm / rmassm;
        propmel[i] = propm;
        rmassm = massmel[i];
        for (int iii=numtemp[i]; iii<numtemp[i+1]; iii++) {
          float val3 = (valtemp.elementAt(iii-1)).floatValue();
          valtemp.removeElementAt(iii-1);
          valtemp.insertElementAt(new Float(val3 * rmassm), iii-1);
        }
      }
    }
    //
    // copy in original Vectors
    melacomp.removeAllElements();
    melaval.removeAllElements();
    for (int i=0; i<comptemp.size(); i++) {
      melacomp.addElement(comptemp.elementAt(i));
      melaval.addElement(valtemp.elementAt(i));
    }
    melanum.removeAllElements();
    for (int i=0; i<nbmela+1; i++) {
      melanum.addElement(new Integer(numtemp[i]));
    }
    // process materials made of materials to obtain a material list
    // exclusively made of level 1 materials defined in number densities
    ind = 1;
    numtemp = new int[nbmate+1]; numtemp[0] = 1;
    comptemp = new Vector<Object>();
    Vector<Float> conc = new Vector<Float>();
    for (int i=0; i<nbmate; i++) {
      Composition item = (Composition)mate.elementAt(i);
      int numated = (matenum.elementAt(i)).intValue();
      int numatef = (matenum.elementAt(i+1)).intValue() - 1;
      int itmate = -1;
      if ((item.isoCompo != null) && (item.isoCompo[0].composition.type == "material")) {
        itmate=0; // &MATE
      } else if ((item.alias != null) && (item.alias.type == "mixture")) {
        itmate=1; // &MELA
      } else if ((item.isoCompo != null) && (item.isoCompo[0].isotope != null)) {
        itmate=2; // &CONC
      } else if ((item.alias != null) && (item.alias.isotope != null)) {
        itmate=3; // &ISOT
      }
      if (itmate == 3) {
        Isotope amate = (Isotope)matecomp.elementAt(numated-1);
        int ll = -1;
        for (int j=0; j<listisot.length; j++) {
          if (amate.symbol.equals(listisot[j])) {
           ll = j;
           break;
          }
        }
        if (ll == -1) throw new RuntimeException("component " + amate.symbol + " is unknown");
        comptemp.addElement(amate);
        float value = (mateval.elementAt(numated-1)).floatValue();
        conc.addElement(new Float((float)(value / massisot[ll] * avoga)));
        ind++;
      } else if (itmate == 2) {
        for (int k=numated; k<numatef+1; k++) {
          comptemp.addElement(matecomp.elementAt(k-1));
          conc.addElement(mateval.elementAt(k-1));
          ind++;
        }
      } else if (itmate == 1) {
        int ll = mela.indexOf(matecomp.elementAt(numated-1));
        int itempd = (melanum.elementAt(ll)).intValue();
        int itempf = (melanum.elementAt(ll+1)).intValue() - 1;
        for (int k=itempd; k<itempf+1; k++) {
          comptemp.addElement(melacomp.elementAt(k-1));
          float value1 = (mateval.elementAt(numated-1)).floatValue();
          float value2 = (melaval.elementAt(k-1)).floatValue();
          conc.addElement(new Float((float)(value1 * value2 * avoga / massmel[ll])));
          ind++;
        }
      } else if (itmate == 0) {
        for (int iii=numated; iii<numatef+1; iii++) {
          int mm = mate.indexOf(matecomp.elementAt(iii-1));
          int itempd = (melanum.elementAt(mm)).intValue();
          int itempf = (melanum.elementAt(mm+1)).intValue() - 1;
          for (int k=itempd; k<itempf+1; k++) {
            comptemp.addElement(comptemp.elementAt(k-1));
            float value1 = (mateval.elementAt(iii-1)).floatValue();
            float value2 = (conc.elementAt(k-1)).floatValue();
            conc.addElement(new Float(value1 * value2));
            ind++;
          }
        }
      }
      numtemp[i+1] = ind;
    }
    //
    // process multiply-defined isotopes
    ind = 1;
    ntemp = new int[numtemp[nbmate]];
    matecomp.removeAllElements();
    Vector<Float> conc2 = new Vector<Float>();
    for (int i=0; i<nbmate; i++) {
      int numated = numtemp[i];
      int numatef = numtemp[i+1] - 1;
      stemp = new Vector<Object>();
      valtemp = new Vector<Float>();
      for (int jj=numated; jj<numatef+1; jj++) {
        if (stemp.contains(comptemp.elementAt(jj-1))) {
          int k = stemp.indexOf(comptemp.elementAt(jj-1));
          float value1 = (valtemp.elementAt(k)).floatValue();
          float value2 = (conc.elementAt(jj-1)).floatValue();
          valtemp.removeElementAt(k);
          valtemp.insertElementAt(new Float(value1 * value2), k);
        } else {
          stemp.addElement(comptemp.elementAt(jj-1));
          valtemp.addElement(conc.elementAt(jj-1));
          ind++;
        }
      }
      ntemp[i+1] = ind;
      for (int kl=ntemp[i]; kl<ntemp[i+1]; kl++) {
        matecomp.addElement(stemp.elementAt(kl-ntemp[i]));
        conc2.addElement(valtemp.elementAt(kl-ntemp[i]));
      }
    }
    //
    // recover concentrations
    Composition[] numberDensities = new Composition[this.mixs.length];
    for (int imix=0; imix<this.mixs.length; imix++) {
      numberDensities[imix] = new Composition("scratch");
      numberDensities[imix].name = this.mixs[imix].name;
      numberDensities[imix].temperature = this.mixs[imix].temperature;
      int i = mili.indexOf(this.mixs[imix]);
      int numilid = (milinum.elementAt(i)).intValue();
      int numilif = (milinum.elementAt(i+1)).intValue() - 1;
      Vector<Object> isoTemp = new Vector<Object>();
      Vector<Float> concTemp = new Vector<Float>();
      if (this.mixs[imix].isoCompo != null) {
        for (int ij=numilid; ij<numilif+1; ij++) {
          isoTemp.addElement(milicomp.elementAt(ij-1));
          concTemp.addElement(milival.elementAt(ij-1));
        }
      } else if (this.mixs[imix].alias != null) {
        int j = mate.indexOf(milicomp.elementAt(numilid-1));
        if (j == -1) {
          String name = ((Isotope)milicomp.elementAt(numilid-1)).name;
          throw new RuntimeException("item " + name + " is not a material in "
          + this.mixs[imix].name);
        }
        int numated = ntemp[j];
        int numatef = ntemp[j+1] - 1;
        for (int kk=numated; kk<numatef+1; kk++) {
          isoTemp.addElement(matecomp.elementAt(kk-1));
          concTemp.addElement(conc2.elementAt(kk-1));
        }
      }
      Isotope[] isoArray = new Isotope[isoTemp.size()];
      float[] concArray = new float[isoTemp.size()];
      for (int ii=0; ii<isoTemp.size(); ii++) {
        isoArray[ii] = (Isotope)isoTemp.elementAt(ii);
        concArray[ii] = (concTemp.elementAt(ii)).floatValue();
      }
      numberDensities[imix].setComposition(isoArray, concArray);
    }
    return numberDensities;
  }

  protected void makeText() {
    this.operator="LIB: ";
    super.write("EDIT", super.edit);
    if (this.nmix == 0) {
      super.write("NMIX", this.mixs.length);
    } else {
      super.write("NMIX", this.nmix);
    }
    if (this.type != null) super.write(this.type);
    if (this.options != null) super.write("\n",this.options);
    if (this.ctra != null) super.write("\nCTRA",this.ctra);
    if (this.chain != null) super.write("\n", this.chain);
    if (this.mixs != null) {
       Composition[] numberDensities = this.makeMedium();
       Vector<String[]> lib_list = new Vector<String[]>();
       for (int i=0; i<numberDensities.length; i++) {
         for (int j=0; j<numberDensities[i].isoCompo.length; j++) outer: {
           String item_lib = numberDensities[i].isoCompo[j].isotope.lib;
           String item_fil = numberDensities[i].isoCompo[j].isotope.fil;
           for (int k=0; k<lib_list.size(); k++) {
             String[] str = lib_list.elementAt(k);
             if ((item_lib.equals(str[0]))&(item_fil.equals(str[1]))) break outer;
           }
           lib_list.addElement(new String[]{item_lib, item_fil});
         }
       }
       System.out.println("Number of libraries: " + lib_list.size());
       for (int i=0; i<lib_list.size(); i++) {
         String[] str = lib_list.elementAt(i);
         super.write("\nMIXS LIB:", str[0]);
         super.write("FIL:", str[1]);
         for (int imix=0; imix<numberDensities.length; imix++) {
           Composition item = numberDensities[imix];
           if (!item.type.equals("medium")) {
             throw new RuntimeException("invalid type (" + item.type +
             ") for composition " + item.name + ". medium expected.");
           }
           boolean flag = true;
           for (int k=0; k<numberDensities[imix].isoCompo.length; k++) {
             Isotope iso = item.isoCompo[k].isotope;
             if ((str[0].equals(iso.lib))&(str[1].equals(iso.fil))) {
               if (flag) {
                 super.write("\nMIX",imix+1);
                 super.write(item.temperature+273.16f);
                 super.write("(*"+item.name+"*)");
                 flag = false;
               }
               super.write("\n  ", iso.name);
               super.write("=", iso.libName);
               super.write(item.isoCompo[k].concentration);
               if ((iso.inrs != 0)&iso.corr) super.write("CORR");
               if (iso.inrs != 0) super.write(iso.inrs);
               if ((iso.shib != null)&(iso.inrs != 0)) super.write("SHIB", iso.shib);
               if ((iso.irset != null)&(iso.inrs != 0)&&(iso.irset.gC == -999.0f)) {
                 if (iso.irset.group == -1) {
                   super.write("IRSET PTSL NONE");
                 } else {
                   super.write("IRSET PTSL", iso.irset.group);
                 }
               } else if ((iso.irset != null)&(iso.inrs != 0)) {
                 super.write("IRSET", iso.irset.gC); super.write(iso.irset.group);
               }
               if (iso.noev) super.write("NOEV");
             }
           }
         }
       }
    } else {
       super.write("\nMIXS");
    }
    super.write("\n;");
  }
}
