package jargon;

import java.io.*;
import java.util.Vector;
/**
 * This abstract class is intented to be subclassed into a specific
 * operator. An operator is generally implemented in a native
 * language such as C or Fortran. Its input is formatted into REDGET
 * compatible data and its output is recovered into embedded objects 
 * <tt>lcmObj</tt>, <tt>fileObj</tt> and/or <tt>epsObj</tt>.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public abstract class Operator implements Serializable {
  static {
    System.out.println("Operator abstract class loaded");
    System.loadLibrary("JargonNat");
    System.out.println("\n-----------------------------------------");
    System.out.println("| DRAGON: code neutronique deterministe |");
    System.out.println("|             Version Java              |");
    System.out.println("-----------------------------------------");
    System.out.flush();
  }
  /** String identification of the specific operator
   */
  public String operator = null;
  /** user-defined name of this operator instance
   */
  public String name;
  /** user-defined edition index. Increasing value of <tt>edit</tt> will
   * cause increasing amount of listing information. Set <tt>edit</tt>
   * to zero to avoid listing information.
   */
  public int edit;
  /** embedded LCM object
   */
  public Jlcm lcmObj;
  /** embedded file object
   */
  public Jlcm fileObj;
  /** embedded Encapsulated Post Script (eps) file object
   */
  public Jlcm epsObj;
  static private StringWriter fout;
  static private int npos;
  private transient Vector<Jlcm> inputJlcmScratch; // Scratch storage
  private transient Vector<Jlcm> outputJlcmScratch; // Scratch storage
  
  /** call the native operator using JNI
   * @param jname string containing the name of the REDGET input file
   * @param jcommand name of the operator
   * @param jkentry array of input/output LCM containers required by the operator
   */
  static public native synchronized void execNat(String jname, String jcommand, Jlcm[] jkentry);
  static private native void exitNat();
  /** Use this constructor to create a new Operator object.
   */
  Operator() {
    this.edit = 1;
  }
  
  abstract protected void makeText();
  protected void makeText2() {
  }
  
  protected void resetJlcm() {
    this.outputJlcmScratch = new Vector<Jlcm>();
    this.inputJlcmScratch = new Vector<Jlcm>();
  }
  
  protected void outputJlcm(Jlcm element) {
    this.outputJlcmScratch.addElement(element);
  }
 
  protected void inputJlcm(Jlcm element) {
    this.inputJlcmScratch.addElement(element);
  }
  
  protected Jlcm[] getJlcm(boolean edit) {
    int nb = this.outputJlcmScratch.size();
    for (int in=0; in<this.inputJlcmScratch.size(); in++) {
       Jlcm o = this.inputJlcmScratch.elementAt(in);
       if (!this.outputJlcmScratch.contains(o)) nb++;
    }
    for (int out=0; out<this.outputJlcmScratch.size(); out++) {
       Jlcm o = this.outputJlcmScratch.elementAt(out);
       if (o.getAccess() == 2) {
         o.close("KEEP");
         o.open("READ/WRITE");
       }
    }
    Jlcm[] Jlcmarray = new Jlcm[nb];
    nb = this.outputJlcmScratch.size();
    this.outputJlcmScratch.copyInto(Jlcmarray);
    for (int in=0; in<this.inputJlcmScratch.size(); in++) {
       Jlcm o = this.inputJlcmScratch.elementAt(in);
       if (!this.outputJlcmScratch.contains(o)) {
         o.close("KEEP");
         o.open("READ-ONLY");
         Jlcmarray[nb++] = o;
       } else {
         o.close("KEEP");
         o.open("READ/WRITE");
       }
    }
    if (edit) {
      System.out.println("----------------------------------------------------------");
      for (int in=0; in<Jlcmarray.length; in++) {
        Jlcm o = Jlcmarray[in];
        System.out.print(in + " name=" + o.getName() + " access=" + o.getAccess());
        System.out.print(" new=" + o.isNew());
        if ((!o.isNew()) & (o.getType() <= 2)) System.out.print(" signature=" + (String)o.get("SIGNATURE"));
        System.out.println();
      }
      System.out.println("----------------------------------------------------------");
    }
    return Jlcmarray;
  }
  
  protected void write(int data) {
    String str = new String();
    str=String.valueOf(data)+" ";
    int lntxt = str.length();
    if (this.npos + lntxt > 72) {
      for (int i=0; i<72-this.npos+12; i++) this.fout.write(" ");
      this.npos = 4;
    }
    this.npos += lntxt;
    this.fout.write(str);
  }
  
  protected void write(float data) {
    String str = new String();
    str=String.valueOf(data)+" ";
    int lntxt = str.length();
    if (this.npos + lntxt > 72) {
      for (int i=0; i<72-this.npos+12; i++) this.fout.write(" ");
      this.npos = 4;
    }
    this.npos += lntxt;
    this.fout.write(str);
  }
    
  protected void write(Object data) {
    char type1 = data.getClass().getName().charAt(0);
    if (type1 == '[') {
      char type2 = data.getClass().getName().charAt(1);
      if ((type2 == 'L')|(type2 == '[')) {
        for (int i=0; i<((Object[])data).length; i++) {
          this.write(((Object[])data)[i]);
        }
      } else if (type2 == 'I') {
        for (int i=0; i<((int[])data).length; i++) {
          this.write(((int[])data)[i]);
        }
      } else if (type2 == 'F') {
        for (int i=0; i<((float[])data).length; i++) {
          this.write(((float[])data)[i]);
        }
      } else {
        throw new RuntimeException("unknown type:" + data.getClass().getName());
      }
    } else {
      String str = new String();
      str = data.toString();
      int index = str.indexOf('\n');
      if (index == -1) {
        str = str + " ";
        int lntxt = str.length();
        if ((this.npos + lntxt > 72) & (lntxt < 68)) {
          for (int i=0; i<72-this.npos+12; i++) this.fout.write(" ");
          this.npos = 4;
        } else if (this.npos + lntxt > 72) {
          for (int i=0; i<72-this.npos+8; i++) this.fout.write(" ");
          this.npos = 0;
        }
        this.npos += lntxt;
        this.fout.write(str);
      } else {
        String subs = new String();
        if (index > 0) {
          subs = str.substring(0,index);
          this.write(subs);
        }
        for (int i=0; i<72-this.npos+12; i++) this.fout.write(" ");
        this.npos = 4;
        if (index < str.length()-1) {
          subs = str.substring(index+1,str.length());
          this.write(subs);
        }
      }
    }
  }
  
  protected void write(Object data1, int data2) {
    this.write(data1); this.write(data2);
  }
  
  protected void write(Object data1, float data2) {
    this.write(data1); this.write(data2);
  }
  
  protected void write(Object data1, Object data2) {
    this.write(data1); this.write(data2);
  }
  /** print the operator data in a form compatible with REDGET.
   */
  public void text() {
    this.npos = 0;
    this.fout = new StringWriter();
    this.makeText();
    for (int i=0; i<80-this.npos; i++) this.fout.write(" ");
    System.out.println("----------------------------------------------------------");
    String my_text = this.fout.toString();
    int iof = 0;
    for (int i=0; i<my_text.length()/80; i++) {
      int minIof =  Math.min(iof+80, my_text.length());
      System.out.println(my_text.substring(iof, minIof));
      iof += 80;
    }
    System.out.println("----------------------------------------------------------");
  }
  protected void text2() {
    this.npos = 0;
    this.fout = new StringWriter();
    this.makeText2();
    for (int i=0;i<80-this.npos; i++) this.fout.write(" ");
    System.out.println("----------------------------------------------------------");
    String my_text = this.fout.toString();
    int iof = 0;
    for (int i=0; i<my_text.length()/80; i++) {
      int minIof =  Math.min(iof+80, my_text.length());
      System.out.println(my_text.substring(iof, minIof));
      iof += 80;
    }
    System.out.println("----------------------------------------------------------");
  }
  protected void exec() throws JargonException {
    this.npos = 0;
    this.fout = new StringWriter();
    this.makeText();
    for (int i=0;i<80-this.npos;i++) this.fout.write(" ");
    String my_text = this.fout.toString();
    String my_name = new String("_input");
    try {
      FileWriter filein = new FileWriter(my_name);
      BufferedWriter os = new BufferedWriter(filein);
      int iof = 0;
      for (int i=0; i<my_text.length()/80; i++) {
        int minIof =  Math.min(iof+80, my_text.length());
        os.write(my_text.substring(iof, minIof)); os.newLine();
        iof += 80;
      }
      os.close();
    } catch(Exception ex) {
      System.out.println("Jargon exception encountered in Operator:" + ex);
      throw new JargonException(ex.toString());
    }
    try {
      execNat(my_name, this.operator, this.getJlcm(true));
    } catch(Exception ex) {
      System.out.println("Jargon exception encountered in Operator:" + ex);
      throw new JargonException(ex.toString());
    }
    Runtime.getRuntime().gc();
  }
  protected void exec2() throws JargonException {
    this.npos = 0;
    this.fout = new StringWriter();
    this.makeText2();
    for (int i=0;i<80-this.npos;i++) this.fout.write(" ");
    String my_text = this.fout.toString();
    String my_name = new String("_input");
    try {
      BufferedWriter os = new BufferedWriter(new FileWriter(my_name));
      int iof = 0;
      for (int i=0; i<my_text.length()/80; i++) {
        int minIof =  Math.min(iof+80, my_text.length());
        os.write(my_text.substring(iof, minIof)); os.newLine();
        iof += 80;
      }
      os.close();
    } catch(Exception ex) {
      System.out.println("Jargon exception 1 encountered in Operator:" + ex);
      throw new JargonException(ex.toString());
    }
    try {
      execNat(my_name, this.operator, this.getJlcm(true));
    } catch(Exception ex) {
      System.out.println("Jargon exception 2 encountered in Operator:" + ex);
      throw new JargonException(ex.toString());
    }
    Runtime.getRuntime().gc();
  }
  /** erase the LCM structures embedded in this Operator object.
   */
  public void reset() {
    if (this.lcmObj != null) {
      int my_type = this.lcmObj.getType();
      this.lcmObj.close("KEEP");
      this.lcmObj.open("READ/WRITE");
      this.lcmObj.close("DESTROY");
      if (my_type == 1) {
        this.lcmObj = new Jlcm("LCM", this.name);
      } else {
        this.lcmObj = new Jlcm("XSM", this.name);
      }
    }
    if (this.fileObj != null) {
      int my_type = this.lcmObj.getType();
      this.fileObj.close("KEEP");
      this.fileObj.open("READ/WRITE");
      this.fileObj.close("DESTROY");
      if (my_type == 3) {
        this.fileObj = new Jlcm("BINARY", this.name);
      } else if (my_type == 4) {
        this.fileObj = new Jlcm("ASCII", this.name);
      } else if (my_type == 5) {
        this.fileObj = new Jlcm("DA", this.name, this.fileObj.getLrda());
      }
    }
    if (this.epsObj != null) {
      this.epsObj.close("KEEP");
      this.epsObj.open("READ/WRITE");
      this.epsObj.close("DESTROY");
      this.epsObj = new Jlcm("ASCII", this.name);
    }
  }
  /** call the native <tt>END:</tt> operator.
   */
  public void end() throws JargonException {
    System.out.println("End from Operator");
    String my_name = new String("_input");
    try {
      BufferedWriter os = new BufferedWriter(new FileWriter(my_name));
      os.write(";");
      os.close();
    } catch(Exception ex) {
      System.out.println("Jargon exception 1 encountered in Operator: END:");
      throw new JargonException(ex.toString());
    }
    try {
      execNat(my_name,"END:",null);
    } catch(Exception ex) {
      System.out.println("Jargon exception 2 encountered in Operator: END:");
      throw new JargonException(ex.toString());
    }
  }
  /** call the native <tt>ABORT:</tt> operator.
   */
  public void abort() throws JargonException {
    System.out.println("Abort from Operator");
    String my_name = new String("_input");
    try {
      BufferedWriter os = new BufferedWriter(new FileWriter(my_name));
      os.write(";");
      os.close();
    } catch(Exception ex) {
      System.out.println("Jargon exception 1 encountered in Operator: ABORT:");
      throw new JargonException(ex.toString());
    }
    try {
      execNat(my_name,"ABORT:",null);
    } catch(Exception ex) {
      System.out.println("Jargon exception 2 encountered in Operator: ABORT:");
      throw new JargonException(ex.toString());
    }
  }
}
