package jargon;

/** Execution of a single branch calculation within SimpleCompo.
 * The RepriseProc Procedure is intended to be executed remotely
 * within the SimpleCompo calculational scheme.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class RepriseProc extends Procedure {
  private static final long serialVersionUID = 47L;
  static {
    System.out.println("RepriseProc class loaded");
  }
  // Parameter perturbation in a lattice calculation.
  private String name;
  private float time;
  private float normFact;
  private Microlib myMicrolib;
  private Evo evoObj;
  private Edition editObj;
  private Gigogne myGigogne;
  private String solutionType;
  /** Use this constructor to create a new RepriseProc procedure.
   * @param name user-defined name of the new RepriseProc procedure.
   * @param time time associated to this branch calculation.
   * @param normFact flux normalization factor.
   * @param myMicrolib Microlib object with empty embedded LCM structures.
   * @param evoObj Evo object containing the burnup history.
   * @param editObj Edition object containing the condensation/homogenization
   * specifications. The embedded LCM structures are empty.
   * @param myGigogne Gigogne object containing the geometry.
   * @param solutionType type of flux calculation. Can be <tt>"Sybil"</tt> or
   * <tt>"Excell"</tt>.
   */
  public RepriseProc(String name, float time, float normFact, Microlib myMicrolib,
    Evo evoObj, Edition editObj, Gigogne myGigogne, String solutionType) {
    this.name = name;
    this.time = time;
    this.normFact = normFact;
    this.myMicrolib = myMicrolib;
    this.evoObj = evoObj;
    this.editObj = editObj;
    this.myGigogne = myGigogne;
    this.solutionType = solutionType;
  }
  /** implementation of the RepriseProc procedure. Implement the ProcedureIntf
   * interface. This class is public as required by the Remote Method Invocation
   * (RMI) mechanism but should not be called directly. Use the <tt>run()</tt>
   * method instead.
   */
  public Operator exec() throws JargonException {
    System.out.println("enter in RepriseProc");
    //------------------------------- MICROLIB -------------------------------
    myMicrolib.edit = 0; myMicrolib.type = "INTR";
    myMicrolib.options = new String[]{"MACR"};
    myMicrolib.exec(myMicrolib);
    //------------------------------------------------------------------------
    //
    Tracking trackObj = null;
    if (this.solutionType.equals("Sybil")) {
      //--------------------------------- SYBIL ------------------------------
      // Tracking of the geometry for Eurydice-2
      Sybil sybilTrack = new Sybil("trackObj");
      sybilTrack.edit = 1;
      sybilTrack.title = "MULTICELL PWR ASSEMBLY (RepriseProc)";
      sybilTrack.maxr = 300;
      sybilTrack.exec(myGigogne);
      trackObj = sybilTrack;
      //----------------------------------------------------------------------
    } else if (this.solutionType.equals("Excell")) {
      //-------------------------------- EXCELL ------------------------------
      // Tracking of the geometry for Excell
      Excell excellTrack = new Excell("trackObj");
      excellTrack.edit = 0;
      excellTrack.title = "EXCELL PWR ASSEMBLY (RepriseProc)";
      excellTrack.maxr = 300;
      excellTrack.trak = "TISO";
      excellTrack.nangl = 12;
      excellTrack.dens = 10.0f;
      excellTrack.exec(myGigogne);
      trackObj = excellTrack;
      //----------------------------------------------------------------------
    }
    //
    //-------------------------------- Autop ---------------------------------
    Autop myShi = new Autop("myShi", "Shiba");
    myShi.passes = 2;
    myShi.exec(myMicrolib, trackObj);
    //------------------------------------------------------------------------
    //
    //--------------------------------- ASM ----------------------------------
    Asm myAsm = new Asm("myAsm", "PIJ");
    myAsm.ecco = true;
    myAsm.exec(myShi, trackObj);
    //------------------------------------------------------------------------
    //
    //--------------------------------- FLUX ---------------------------------
    Flux fluxObj = new Flux("fluxObj", "K");
    fluxObj.leak = new String[]{"B1", "ECCO"};
    fluxObj.buckling = 0.0f;
    fluxObj.exec(myAsm, myShi, trackObj);
    System.out.println("The value of K-EFFECTIVE is " + fluxObj.getKeff());
    //------------------------------------------------------------------------
    //
    //------------------------------- EDITION --------------------------------
    editObj.exec(fluxObj, myShi, trackObj, myGigogne);
    //------------------------------------------------------------------------
    //
    // Normalization to the reactor power
    evoObj.edit = 0;
    evoObj.setSave(this.time, "DAY", "W/CC", this.normFact);
    evoObj.exec(evoObj, myShi, fluxObj, trackObj);
    //
    //--------------------------------- COMPO --------------------------------
    // Compo object initialization
    Compo compoObj = new Compo(this.name);
    compoObj.setGlobal("FTMP", "TEMP", myShi, new Composition("fuel", "medium"));
    compoObj.setGlobal("WTMP", "TEMP", myShi, new Composition("mod", "medium"));
    compoObj.setLocal("FLUB", "FLUB");
    compoObj.exec();
    // Compo object construction
    compoObj.setSet(this.time, "DAY");
    compoObj.exec(compoObj, editObj, evoObj, myShi);
    //------------------------------------------------------------------------
    // At this point, the tracking binary file is read-only; it cannot be
    // garbage-collected without a reset().
    trackObj.reset();
    return compoObj;
  }
}
