package jargon;

/** This class is an example implementation of a computational scheme
 * dedicated to a single flux calculation.
 * <P>
 * An end-user can use the SimpleCompo computational scheme from a
 * graphical user interface or from a simple BeanShell script, as follows:
 * <PRE>
 * myScheme = new SimpleCell("myScheme");
 * myScheme.setGigogne(assmbObj);
 * myScheme.setMacrolib(myMacro);
 * myScheme.setSolutionType("Sybil");
 * myScheme.setType("ROT+");
 * myScheme.setCylinder("ASKE");
 * myScheme.setOption("B");
 * myScheme.setLeakType(new String[]{"B0","SIGS"});
 * myScheme.run();
 * </PRE>
 * where it is assumed that objects <tt>assmbObj</tt> (class Gigogne) and
 * <tt>myMacro</tt> (class Macrolib) already exist. The resulting Flux
 * object is recovered in <tt>(Flux)myScheme.operatorOut</tt>.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class SimpleCell extends ComputationalScheme {
  private static final long serialVersionUID = 800L;
  static {
    System.out.println("SimpleCell class loaded");
  }
  private Gigogne myGeometry;
  private Macrolib myMacrolib;
  private int myEdit = 1;
  private String myType = "DP00";
  private String myCylinder = "SANC";
  private String myOption = "K";
  private String[] myLeakType;
  private String trivacType = "DUAL";
  private int[] order = new int[]{2, 2};
  private int maxr = 300;
  private String solutionType = "Sybil";
  /** 
   * Use this constructor to create a new SimpleCell object.
   * @param name user-defined name of the SimpleCell object
   */
  public SimpleCell(String name) {
    super(name);
  }
  /** set the geometry.
   * @param myGeometry user-defined Gigogne object.
   */
  public void setGigogne(Gigogne myGeometry) {
    this.myGeometry = myGeometry;
  }
  /** set the cross section information.
   * @param myMacrolib user-defined Macrolib object.
   */
  public void setMacrolib(Macrolib myMacrolib) {
    this.myMacrolib = myMacrolib;
  }
  /** set the edition index for this computational scheme.
   * @param myEdit edition index. Increasing value of <tt>myEdit</tt> will
   * cause increasing amount of listing information. Set <tt>myEdit</tt>
   * to zero to avoid listing information.
   */
  public void setEdit(int myEdit) {
    this.myEdit = myEdit;
  }
  /** set the number of computer words used in the dynamic allocation
   * algorithm of the tracking operator.
   * @param maxr number of computer words. 
   */
  public void setMaxr(int maxr) {
    this.maxr = maxr;
  }
  /** set the type of multicell approximation for Sybil.
   * @param myType  This variable is chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"ROTH"</tt> <dd> the isotropic DP0 components of the inter-cell
   * current is used with the incoming current being averaged over all the
   * faces surrounding a cell. The global collision matrix is calculated in an
   * annular model. Only used when 2D assembly of cells are considered.
   * <dt> <tt>"ROT+"</tt> <dd> the isotropic DP0 components of the inter-cell
   * current is used. The global collision matrix is calculated in an annular
   * model. Only used when 2D assembly of cells are considered.
   * <dt> <tt>"DP00"</tt> <dd> the isotropic DP0 components of the inter-cell
   * current is used. The global collision matrix are computed explicitly.
   * Only used when 2D assembly of cells are considered.
   * <dt> <tt>"DP01"</tt> <dd> the linearly anisotropic DP1 components of the
   * inter-cell current are used. This hypothesis implies 12 currents per
   * cell in a cartesian geometry and 18 currents per cell for an hexagonal
   * geometry. Linearly anisotropic reflection is used. Only used when 2D
   * assembly of cells are considered.
   * </dl></ul>
   */
  public void setType(String myType) {
    this.myType = myType;
    if ((!myType.equals("ROTH"))&(!myType.equals("ROT+"))&
       (!myType.equals("DP00"))&(!myType.equals("DP01"))) {
      throw new RuntimeException("invalid type (" + myType + ")");
    }
  }
  /** set the type of cylinderization for approximations <tt>"ROTH"</tt>
   * and <tt>"ROT+"</tt> in Sybil.
   * @param myCylinder  This variable is chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"WIGN"</tt> <dd> use of a <i>Wigner</i> cylinderization which
   * preserves the volume of the external crown. Note that an assembly of
   * rectangular cells having unequal volumes cannot use a <i>Wigner</i>
   * cylinderization.
   * <dt> <tt>"ASKE"</tt> <dd> use of an <i>Askew</i> cylinderization which
   * preserves both the external surface of the cells and the material balance
   * of the external crown (by a modification of its concentration). Note
   * that an assembly of rectangular cells having unequal volumes can use
   * an <i>Askew</i> cylinderization.
   * <dt> <tt>"SANC"</tt> <dd> use of a <i>Sanchez</i> cylinderization. This
   * model uses a <i>Wigner</i> cylinderization for computing the collision
   * PIJ and leakage PIS probabilities. However, the reciprocity and
   * conservation relations used to compute the incoming PSJ and transmission
   * PSS probabilities are defined in the rectangular cell (with the exact
   * surface). Note that an assembly of rectangular cells having unequal
   * volumes can use a <i>Sanchez</i> cylinderization. This is the default
   * option.
   * </dl></ul>
   */
  public void setCylinder(String myCylinder) {
    this.myCylinder = myCylinder;
    if ((!myCylinder.equals("WIGN"))&(!myCylinder.equals("ASKE"))&
       (!myCylinder.equals("SANC"))) {
      throw new RuntimeException("invalid type (" + myType + ")");
    }
  }
  /** set the type of flux calculation. This variable is chosen among
  *  the following values:
   * <ul><dl>
   * <dt> <tt>"N"</tt> <dd> no flux calculation is to be performed.
   * This option is usually activated when one simply wishes to initialize
   * the neutron flux distribution and to store this information in the
   * embedded LCM structure of the Flux object.
   * <dt> <tt>"S"</tt> <dd> a fixed source problem is to be treated. Such
   * problem can also include fission source contributions.
   * <dt> <tt>"K"</tt> <dd> a fission source eigenvalue problem is to be
   * treated. The eigenvalue is then the effective multiplication factor
   * with a fixed buckling. In this case, the fixed sources, if any is
   * present on the Macrolib or Microlib object, are not used.
   * <dt> <tt>"B"</tt> <dd> a fission source eigenvalue problem is to be
   * treated. The eigenvalue in this case is the critical buckling with a
   * fixed effective multiplication factor. The buckling eigenvalue has
   * meaning only in the case of a cell without leakages. It is also
   * possible to use an open geometry with <tt>"VOID"</tt> boundary
   * conditions  provided it is closed by the Asm operator using the
   * options <tt>"NORM"</tt> or <tt>"ALSB"</tt>.
   * <dt> <tt>"L"</tt> <dd> a non-multiplicative medium eigenvalue problem
   * is to be treated. The eigenvalue in this case is the critical buckling
   * with vanishing fission cross sections. The buckling eigenvalue has
   * meaning only in the case of a cell without leakages. It is also
   * possible to use an open geometry with <tt>"VOID"</tt> boundary
   * conditions  provided it is closed by the Asm operator using the
   * options <tt>"NORM"</tt> or <tt>"ALSB"</tt>.
   * </dl></ul>
   */
  public void setOption(String myOption) {
    this.myOption = myOption;
  }
  /** set the type of leakage approximation for the flux calculation.
   * @param myLeakType  The variable <tt>myLeakType[0]</tt> is chosen
   * among the following values:
   * <ul><dl>
   * <dt> <tt>"LKRD"</tt> <dd> the leakage coefficients are recovered
   * from the LCM data structure embedded in the Microlib object. The
   * <tt>"LKRD"</tt> option is not available with the <tt>"ECCO"</tt>
   * and <tt>"HETE"</tt> leakage models.
   * <dt> <tt>"P0"</tt> <dd> the leakage coefficients are calculated
   * using a P0 model
   * <dt> <tt>"P1"</tt> <dd> the leakage coefficients are calculated
   * using a P1 model
   * <dt> <tt>"B0"</tt> <dd> the leakage coefficients are calculated
   * using a B0 model
   * <dt> <tt>"B1"</tt> <dd> the leakage coefficients are calculated
   * using a B1 model
   * <dt> <tt>"B0TR"</tt> <dd> the leakage coefficients are calculated
   * using a B0 model with transport correction.
   * </dl></ul>
   * The variable <tt>myLeakType[1]</tt> is chosen among the following
   * values:
   * <ul><dl>
   * <dt> <tt>"PNLR"</tt> <dd> that the elements of the scattering
   * modified collision probability matrix are multiplied by the adequate
   * non-leakage homogeneous buckling dependent factor.
   * <dt> <tt>"PNL"</tt> <dd> the elements of the collision probability
   * matrix are multiplied by the adequate non-leakage homogeneous buckling
   * dependent factor.
   * <dt> <tt>"SIGS"</tt> <dd> an homogeneous buckling correction is to be
   * applied on the scattering cross section.
   * <dt> <tt>"ALBS"</tt> <dd> an homogeneous buckling contribution is
   * introduced by a group dependent correction of the albedo. This leakage
   * model is restricted to the collision probability method. It is then
   * necessary to define the geometry with an external boundary condition
   * of type <tt>"VOID"</tt> and to close the region in operator Asm using
   * the <tt>"ALBS"</tt> option
   * <dt> <tt>"ECCO"</tt> <dd>  perform an ECCO-type leakage calculation
   * taking into account isotropic streaming effects. This method
   * introduces an heterogeneous buckling contribution as a group dependent
   * correction to the source term. It is then necessary to set the option
   * <tt>"ECCO"</tt> in operator Asm.
   * <dt> <tt>"HETE"</tt> <dd> perform a Tibere-type leakage calculation
   * taking into account anisotropic streaming effects. This method
   * introduces an heterogeneous buckling contribution as a group dependent
   * correction to the source term. The heterogeneous buckling contribution
   * is introduced in the Bn model using directional collision probabilities
   * (PIJK method). It is then necessary to set the option <tt>"PIJK"</tt>
   * in operator Asm.
   * </dl></ul>
   */
  public void setLeakType(String[] myLeakType) {
    this.myLeakType = myLeakType;
  }
  /** set the type of solution algorithm.
   * @param solutionType This variable is chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"Sybil"</tt> <dd> Interface current method.
   * <dt> <tt>"Excell"</tt> <dd> Collision probabilities defined over
   * <dt> <tt>"Trivac"</tt> <dd> Finite element solution of the diffusion
   * equation.
   * </dl></ul>
   */
  public void setSolutionType(String solutionType) {
    this.solutionType = solutionType;
  }
  /** set the polynomial basis for the finite element method.
   * @param myTypeBase  type of finite element approach. This variable is chosen
   * among the following values:
   * <ul><dl>
   * <dt> <tt>"PRIM"</tt> <dd> variational collocation method.
   * <dt> <tt>"DUAL"</tt> <dd> mixed-dual finite element discretization. This
   * option is not available for hexagonal geometries.
   * <dt> <tt>"MCFD"</tt> <dd> variable-order mesh-centered finite differences
   * (nodal collocation method). This option is not available for hexagonal
   * geometries. The mesh centered finite difference approximation is the
   * default option and is generally set using <tt>trivacType = "MCFD"</tt> and
   * <tt>order[0] = 1</tt>. The <tt>"MCFD"</tt> approximations are numerically
   * equivalent to the <tt>"DUAL"</tt> approximations with <tt>order[1] = 2</tt>.
   * </dl></ul>
   * @param myTypeOrder one- or two-component array containing the discretization
   * type. <tt>order[0]</tt> is the order of the finite element representation.
   * Its value is chosen among the following values:
   * <ul><dl>
   * <dt> 1 <dd> linear polynomials or finite differences (default value)
   * <dt> 2 <dd> parabolic polynomials
   * <dt> 3 <dd> cubic polynomials
   * </dl></ul>
   * <tt>order[1]</tt> is the type of quadrature used to integrate the mass
   * matrices in case where <tt>trivacType = "DUAL"</tt>. Its value is chosen among
   * the following values:
   * <ul><dl>
   * <dt> 1 <dd> analytical integration
   * <dt> 2 <dd> Gauss-Lobatto quadrature (default value)
   * <dt> 3 <dd> Gauss-Legendre quadrature
   * </dl></ul>
   * The analytical integration corresponds to classical finite elements;
   * the Gauss-Lobatto quadrature corresponds to a variational or nodal
   * type collocation and the Gauss-Legendre quadrature corresponds to
   * superconvergent finite elements.
   * </dl></ul>
   */
  public void setBase(String myTypeBase, int[] myTypeOrder) {
    this.trivacType = myTypeBase;
    this.order = myTypeOrder;
  }
  /** implementation of the computational scheme. Implement the ProcedureIntf
   * interface. This class is public as required by the Remote Method Invocation
   * (RMI) mechanism but should not be called directly. Use the <tt>run()</tt>
   * method instead.
   * @return the Flux object containing the resulting multigroup flux.
   */
  public Operator exec() throws JargonException {
    System.out.println("Executing exec() of SimpleCell");
    System.out.println("SimpleCell: aa");
    this.myMacrolib.exec();
    System.out.println("SimpleCell: bb");
    this.myGeometry.exec(this.myMacrolib);
    System.out.println("SimpleCell: cc");
    //
    Tracking myTrack = null;
    if (this.solutionType.equals("Sybil")) {
      //--------------------------------- SYBIL ------------------------------
      // Tracking of the geometry for Eurydice-2
      Sybil sybilTrack = new Sybil("myTrack");
      sybilTrack.edit = this.myEdit;
      sybilTrack.title = "SimpleCell tracking with Sybil";
      sybilTrack.maxr = this.maxr;
      sybilTrack.type = this.myType;
      sybilTrack.cylinder = this.myCylinder;
      sybilTrack.exec(myGeometry);
      myTrack = sybilTrack;
      //----------------------------------------------------------------------
    } else if (this.solutionType.equals("Excell")) {
      //-------------------------------- EXCELL ------------------------------
      // Tracking of the geometry for Excell
      Excell excellTrack = new Excell("myTrack");
      excellTrack.edit = this.myEdit;
      excellTrack.title = "SimpleCell tracking with Excell";
      excellTrack.maxr = this.maxr;
      excellTrack.trak = "TISO";
      excellTrack.nangl = 12;
      excellTrack.dens = 10.0f;
      excellTrack.exec(myGeometry);
      myTrack = excellTrack;
      //----------------------------------------------------------------------
    } else if (this.solutionType.equals("Trivac")) {
      //-------------------------------- TRIVAC ------------------------------
      // Tracking of the geometry for trivac
      Trivac trivacTrack = new Trivac("myTrack", this.trivacType, this.order);
      trivacTrack.edit = this.myEdit;
      trivacTrack.title = "SimpleCell tracking with Trivac";
      trivacTrack.maxr = this.maxr;
      trivacTrack.exec(myGeometry);
      myTrack = trivacTrack;
      //----------------------------------------------------------------------
    }
    //
    Flux myFlux = null;
    if (this.solutionType.equals("Trivac")) {
      //-------------------------------- TRIVAA --------------------------------
      Trivaa myAsm = new Trivaa("myAsm");
      myAsm.exec(myMacrolib, myTrack);
      //------------------------------------------------------------------------
      //
      //--------------------------------- FLUX ---------------------------------
      myFlux = new Flux("myFlux", this.myOption);
      myFlux.exec(myAsm, myTrack, myMacrolib);
      System.out.println("The value of K-EFFECTIVE is " + myFlux.getKeff());
      //------------------------------------------------------------------------
    } else {
      //--------------------------------- ASM ----------------------------------
      Asm myAsm = new Asm("myAsm", "PIJ");
      myAsm.exec(myMacrolib, myTrack);
      //------------------------------------------------------------------------
      //
      //--------------------------------- FLUX ---------------------------------
      myFlux = new Flux("myFlux", this.myOption);
      if (this.myLeakType != null) myFlux.leak = this.myLeakType;
      myFlux.exec(myAsm, myMacrolib, myTrack);
      System.out.println("The value of K-EFFECTIVE is " + myFlux.getKeff());
      //------------------------------------------------------------------------
    } 
    //
    return myFlux;
  }
}
