package jargon;

import java.util.*;
/** This class is an example implementation of a computational scheme
 * dedicated to the construction of a multi-parameter reactor database.
 * <P>
 * An end-user can use the SimpleCompo computational scheme from a
 * graphical user interface or from a simple BeanShell script, as follows:
 * <PRE>
 * cote = 1.262082f;
 * lame = 1.322082f;
 * myScheme = new SimpleCompo("myScheme");
 * myScheme.setEdit(1);
 * myScheme.setThreadPool(new ThreadPool(bsh.args));
 * myScheme.setGigogne(assmbObj);
 * myScheme.setVolume(((cote*7.0f)+(lame*2.0f))*((cote*7.0f)+(lame*2.0f)));
 * myScheme.setNormalizationFactor(3.016e17f);
 * myScheme.setMicrolib(micrObj);
 * myScheme.setBurnupList(new float[]{9.375f, 18.75f, 37.5f, 75.0f, 500.0f});
 * myScheme.setAutopBurnupList(new float[]{9.375f, 18.75f, 37.5f, 500.0f});
 * myScheme.setTempWaterList(new float[]{300.16f, 400.16f, 500.16f});
 * myScheme.setTempFuelList(new float[]{309.9f, 579.9f});
 * myScheme.setIsotopesEdit(new Isotope[]{U235,U238,PU239,PU240,PU241,PU242,
 *                                        AM241,AM242M,XE135PF});
 * myScheme.setSolutionType("Sybil");
 * myScheme.run();
 * compoObj = (Compo)myScheme.operatorOut;
 * </PRE>
 * where it is assumed that objects <tt>assmbObj</tt> (class Gigogne) and
 * <tt>micrObj</tt> (class Microlib) already exist and that the array
 * <tt>bsh.args</tt> contains the list of servers. This computational scheme
 * is executed in two steps. First, the instances variables describing the
 * calculation are set using the methods with prefix <tt>set</tt>. Second,
 * the computational scheme is executing using the <tt>run()</tt> method
 * inherited from the Procedure class.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class SimpleCompo extends ComputationalScheme implements Observer {
  private static final long serialVersionUID = 850L;
  static {
    System.out.println("SimpleCompo class loaded");
  }
  private ThreadPool myThreadPool;
  private int myEdit = 0;
  private Gigogne myGeom;
  private float myVolume;
  private float myNormalizationFactor;
  private Microlib myMicrolib;
  private float[] myBurnupList;
  private float[] myAutopBurnupList;
  private float[] myTempWaterList;
  private float[] myTempFuelList;
  private Isotope[] myIsotopesEdit;
  private Compo myCompo;
  private String solutionType = "Sybil";
  /** 
   * Use this constructor to create a new SimpleCompo object.
   * @param name user-defined name of the SimpleCompo object
   */
  public SimpleCompo(String name) {
    super(name);
  }
  /** set the edition index for this computational scheme.
   * @param myEdit edition index. Increasing value of <tt>myEdit</tt> will
   * cause increasing amount of listing information. Set <tt>myEdit</tt>
   * to zero to avoid listing information.
   */
  public void setEdit(int myEdit) {
    this.myEdit = myEdit;
  }
  /** set the ThreadPool. A local calculation is performed if the
   * <tt>myThreadPool</tt> variable is not set.
   * @param myThreadPool ThreadPool object.
   */
  public void setThreadPool(ThreadPool myThreadPool) {
    this.myThreadPool = myThreadPool;
  }
  /** set the geometry.
   * @param myGeom user-defined Gigogne object.
   */
  public void setGigogne(Gigogne myGeom) {
    this.myGeom = myGeom;
  }
  /** set the volume of the complete domain. This information is required
   * by the normalization factor calculation.
   * @param myVolume volume of the complete domain.
   */
  public void setVolume(float myVolume) {
    this.myVolume = myVolume;
  }
  /** set the normalization factor.
   * @param myNormalizationFactor power normalization factor
   */
  public void setNormalizationFactor(float myNormalizationFactor) {
    this.myNormalizationFactor = myNormalizationFactor;
  }
  /** set the cross section information.
   * @param myMicrolib user-defined Microlib object.
   */
  public void setMicrolib(Microlib myMicrolib) {
    this.myMicrolib = myMicrolib;
  }
  /** set the burnup steps in MYday/Tonne corresponding to the flux
   * calculations.
   * @param myBurnupList list of burnup values.
   */
  public void setBurnupList(float[] myBurnupList) {
    this.myBurnupList = myBurnupList;
  }
  /** set the burnup steps in MYday/Tonne corresponding to the
   * self-shielding calculations.
   * @param myAutopBurnupList list of burnup values.
   */
  public void setAutopBurnupList(float[] myAutopBurnupList) {
    this.myAutopBurnupList = myAutopBurnupList;
  }
  /** set the moderator temperatures used as global parameters.
   * @param myTempWaterList list of temperature values in Celsius.
   */
  public void setTempWaterList(float[] myTempWaterList) {
    this.myTempWaterList = myTempWaterList;
  }
  /** set the fuel temperatures used as global parameters.
   * @param myTempFuelList list of temperature values in Celsius.
   */
  public void setTempFuelList(float[] myTempFuelList) {
    this.myTempFuelList = myTempFuelList;
  }
  /** set the particularized isotopes of the outout Compo object.
   * @param myIsotopesEdit list of Isotope objects.
   */
  public void setIsotopesEdit(Isotope[] myIsotopesEdit) {
    this.myIsotopesEdit = myIsotopesEdit;
  }
  /** set the type of solution algorithm.
   * @param solutionType This variable is chosen among the following values:
   * <ul><dl>
   * <dt> <tt>"Sybil"</tt> <dd> Interface current method.
   * <dt> <tt>"Excell"</tt> <dd> Collision probabilities defined over
   * the complete assembly.
   * </dl></ul>
   */
  public void setSolutionType(String solutionType) {
    this.solutionType = solutionType;
  }
  /** implementation of the computational scheme. Implement the ProcedureIntf
   * interface. This class is public as required by the Remote Method Invocation
   * (RMI) mechanism but should not be called directly. Use the <tt>run()</tt>
   * method instead.
   * @return the Compo object containing the resulting multiparameter reactor
   * database.
   */
  public Operator exec() throws JargonException {
    System.out.println("Executing exec() of SimpleCompo");
    int iset = 0; // the number of jobs already running in the ThreadPool
    if (this.myThreadPool != null ) iset = this.myThreadPool.getNumberOfJobs();
    //
    this.myMicrolib.exec();
    this.myGeom.exec(this.myMicrolib);
    Composition mod = this.myMicrolib.mixs[0];
    Composition fuel = this.myMicrolib.mixs[1];
    Composition gaine = this.myMicrolib.mixs[2];
    //------------------------------------------------------------------------
    //
    Tracking myTrack = null;
    if (this.solutionType.equals("Sybil")) {
      //--------------------------------- SYBIL ------------------------------
      // Tracking of the geometry for Eurydice-2
      Sybil sybilTrack = new Sybil("myTrack");
      sybilTrack.edit = 5;
      sybilTrack.title = "SimpleCompo tracking with Sybil";
      sybilTrack.maxr = 300;
      sybilTrack.exec(myGeom);
      myTrack = sybilTrack;
      //----------------------------------------------------------------------
    } else if (this.solutionType.equals("Excell")) {
      //-------------------------------- EXCELL ------------------------------
      // Tracking of the geometry for Excell
      Excell excellTrack = new Excell("myTrack");
      excellTrack.edit = 0;
      excellTrack.title = "SimpleCompo tracking with Excell";
      excellTrack.maxr = 300;
      excellTrack.trak = "TISO";
      excellTrack.nangl = 12;
      excellTrack.dens = 10.0f;
      excellTrack.exec(myGeom);
      myTrack = excellTrack;
      //----------------------------------------------------------------------
    }
    //
    //-------------------------------- Autop ---------------------------------
    Autop myShi = new Autop("myShi", "Shiba");
    myShi.passes = 2;
    myShi.exec(myMicrolib, myTrack);
    //------------------------------------------------------------------------
    //
    //--------------------------------- COMPO --------------------------------
    // Initialize the compo object
    myCompo = new Compo("myCompo");
    myCompo.setGlobal("FTMP", "TEMP", myShi, fuel);
    myCompo.setGlobal("WTMP", "TEMP", myShi, mod);
    myCompo.setLocal("FLUB", "FLUB");
    myCompo.comment = "'First line of comment'\n'Second line of comment'\n";
    myCompo.exec();
    //------------------------------------------------------------------------
    //
    //--------------------------------- ASM ----------------------------------
    Asm myAsm = new Asm("myAsm", "PIJ");
    myAsm.ecco = true;
    myAsm.exec(myShi, myTrack);
    //------------------------------------------------------------------------
    //
    //--------------------------------- FLUX ---------------------------------
    Flux myFlux = new Flux("myFlux", "K");
    myFlux.leak = new String[]{"B1", "ECCO"};
    myFlux.buckling = 0.0f;
    myFlux.exec(myAsm, myShi, myTrack);
    System.out.println("The value of K-EFFECTIVE is " + myFlux.getKeff());
    //------------------------------------------------------------------------
    //
    // Compute the normalization factor
    //
    float normFct1 = (this.myNormalizationFactor * 1.60207e-13f) / this.myVolume;

    float normFct2 = normFct1 / (2.651005f * 1.00115f);
    System.out.println("volume_assemblage= " + this.myVolume + " cm**3. in-fuel power= "
    + normFct2 + " MW/tonne");
    System.out.println("normalization power= " + normFct1 + " W/CC");
    //
    // Burnup loop #####################################################
    //
    int edfpos_flag=0;
    float evobeg = 0.0f;
    boolean autop = true;
    //--------------------------------- EVO ----------------------------------
    Evo myEvo = new Evo("myEvo");
    myEvo.solution = "RUNG";
    myEvo.eps2 = 100.0f;
    // -----------------------------------------------------------------------
    int istSS = 0;
    for (int istep=0; istep<this.myBurnupList.length; istep++) {
      float evoend=this.myBurnupList[istep] / normFct2;
      System.out.println("Pas evolution " + (istep+1) + " entre " + evobeg + " et "
      + evoend + " jour:");
      if (istep == 0) {
        // Execution de l'objet Burnup pour le premier pas d'evolution
        myEvo.edit = 2;
        myEvo.expm = 1.0f;
        myEvo.satureInit = true;
        myEvo.setDepl(evobeg, evoend, "DAY", "W/CC", normFct1);
        myEvo.exec(myShi, myFlux, myTrack);
      } else {
        // Execution de l'objet Burnup pour les pas suivants d'evolution
        myEvo.edit = this.myEdit;
        myEvo.expm = 1.0e15f;
        myEvo.satureInit = false;
        myEvo.extr = !autop;
        myEvo.setDepl(evobeg, evoend, "DAY", "W/CC", normFct1);
        myEvo.exec(myEvo, myShi, myFlux, myTrack);
      }
      System.out.println("Nominal flux at step " + (istep+1) + " and at " + evoend + " DAY:");
      //
      // Self-Shielding calculation
      autop = (this.myAutopBurnupList[istSS] == this.myBurnupList[istep]);
      if (autop) {
        istSS++;
        System.out.println("self-shielding at step " + (istSS+1) + " and at " + evoend + " DAY:");
        //-------------------------------- Autop ---------------------------------
        myShi.passes = 1;
        myShi.exec(myShi, myMicrolib, myTrack);
        //------------------------------------------------------------------------
      }
      //
      //--------------------------------- ASM ----------------------------------
      myAsm = new Asm("myAsm", "PIJ");
      myAsm.ecco = true;
      myAsm.exec(myShi, myTrack);
      //------------------------------------------------------------------------
      //
      //--------------------------------- FLUX ---------------------------------
      myFlux.exec(myFlux, myAsm, myShi, myTrack);
      System.out.println("The value of K-EFFECTIVE is " + myFlux.getKeff());
      //------------------------------------------------------------------------
      float step2 = this.myBurnupList[istep];
      //
      //------------------------------- EDITION ---------------------------------
      Edition myEdit = new Edition("myEdit");
      myEdit.options = new String[]{"P1W"};
      myEdit.cond = new int[]{74, 99};
      myEdit.micr2 = this.myIsotopesEdit;
      myEdit.save = "EDITCDAT   1";
      myEdit.sph = new Bivac("Sph_obj", "DUAL", new int[]{2,2});
      myEdit.exec(myFlux, myShi, myTrack, myGeom);
      //-------------------------------------------------------------------------
      //
      // Normalization to the reactor power
      myEvo.edit = this.myEdit;
      myEvo.unsetDepl();
      myEvo.setSave(evoend, "DAY", "W/CC", normFct1);
      myEvo.exec(myEvo, myShi, myFlux, myTrack);
      //
      //--------------------------------- COMPO ---------------------------------
      // Compo object construction
      myCompo.edit = this.myEdit + 1;
      myCompo.setSet(evoend, "DAY");
      myCompo.text();
      myCompo.exec(myCompo, myEdit, myEvo, myShi);
      //-------------------------------------------------------------------------
      //
      // Perform perturbation calculations
      for (int jstep=0; jstep<this.myTempWaterList.length; jstep++) {
        float tempw = this.myTempWaterList[jstep];
        for (int kstep=0; kstep<this.myTempFuelList.length; kstep++) {
          float tempf = this.myTempFuelList[kstep];
          if (!(tempw == 300.16f & tempf == 579.9f)) {
            System.out.println("perturbation calculation at T_water=" + tempw +
            "K and T_fuel=" + tempf + "K");
            //
            // Build the infinite-dilution perturbed microlib
            //--------------------------------- MICROLIB -----------------------------
            mod.temperature = tempw - 273.16f ;
            fuel.temperature = tempf - 273.16f ;
            Microlib myReprise = new Microlib("myReprise", "SKIP");
            myReprise.edit = this.myEdit;
            myReprise.ctra = "APOL"; // set type of transport correction
            myReprise.chain = myMicrolib.chain;
            myReprise.mixs = new Composition[]{(Composition)mod.clone(),
                             (Composition)fuel.clone(), gaine};
            myReprise.exec(); // Microlib interpolation using Dragon
            //------------------------------------------------------------------------
            // Recover the number densities at evoend
            myEvo.edit = this.myEdit;
            myEvo.setSave("NOSA");
            myEvo.unsetDepl();
            myEvo.setSet(evoend, "DAY");
            myEvo.exec(myEvo, myReprise, myFlux, myTrack);
            //----------------------------- repriseObj ------------------------------
            // Remote branch calculation.
            myEdit.reset(); // remove the embedded LCM object
            RepriseProc repriseObj = new RepriseProc("repriseObj", evoend, normFct1,
                                     myReprise, myEvo, myEdit, myGeom, this.solutionType);
            repriseObj.addObserver(this);
            if (this.myThreadPool == null) {
              repriseObj.run();
            } else {
              this.myThreadPool.run(repriseObj);
            }
            //-------------------------------------------------------------------------
          }
        }
      }
      //
      // Reset nominal conditions
      //--------------------------------- MICROLIB -----------------------------
      mod.temperature = 27.0f ;
      fuel.temperature = 306.74f ;
      myMicrolib.mixs = new Composition[]{mod, fuel, gaine};
      myMicrolib.exec(); // Microlib interpolation using Dragon
      //------------------------------------------------------------------------
      // Recover the number densities at evoend
      myEvo.edit = this.myEdit;
      myEvo.setSave("NOSA");
      myEvo.unsetDepl();
      myEvo.setSet(evoend, "DAY");
      myEvo.exec(myEvo, myShi, myFlux, myTrack);
      //------------------------------------------------------------------------
      evobeg = evoend;
    }
    // End of burnup loop ##############################################
    if (this.myThreadPool != null) {
      // ensure that remote calculations are terminated
      this.myThreadPool.join(iset);
    }
    System.out.println("--- End of SimpleCompo procedure ---");
    return myCompo;
  }
  /** implement Observer interface. Catenate a Compo object produced by
   * a remote calculation into the main Compo object.
   */
  public void update(Observable obj, Object arg) {
    if (((String)arg).equals("run completed")) {
      //--------------------------------- COMPO ---------------------------------
      // Concatenation of Cpo_2 into Cpo
      Compo remoteCompo = (Compo)(((Procedure)obj).operatorOut);
      System.out.println("Add remote compo " + remoteCompo.name + " to " + myCompo.name);
      myCompo.edit = this.myEdit + 1;
      myCompo.unSet();
      try {
        myCompo.exec(myCompo, remoteCompo);
      } catch (JargonException e) {
        throw new RuntimeException(e.toString());
      }
      //-------------------------------------------------------------------------
    }
  }
}
