package jargon;

/**
 * The Sybil tracking operator produces a Jlcm structure containing the
 * energy-independent numerotation information required to solve the
 * integral transport equation using 1D collision probabilities and/or
 * the interface current method (Eurydice).
 * <P>
 * The geometries that can be treated by the operator Sybil are:
 * <ol>
 * <li> The homogeneous geometry <tt>"HOMOGE"</tt>.
 * <li> The one-dimensional geometries <tt>"SPHERE"</tt>, <tt>"TUBE"</tt> and
 * <tt>"CAR1D"</tt>.
 * <li> The two-dimensional geometries <tt>"CAR2D"</tt> and <tt>"HEX"</tt>
 * including respectively <tt>"CARCEL"</tt> and <tt>"HEXCEL"</tt>
 * sub-geometries as well as <tt>"VIRTUAL"</tt> sub-geometries. 
 * <li> SIJ-type two-dimensional geometries.
 * <li> Double heterogeneity media.
 * </ol>
 * <P>
 * A Sybil object may be used within the Edition operator to perform SPH
 * equivalence calculations.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Sybil extends Tracking {
  static {
    System.out.println("Sybil class loaded");
  }
  static String fr = "definition d'un tracking Sybil";
  static String en = "definition of a Sybil tracking";
  private static final long serialVersionUID = 150L;
  /** maximum number of interface currents surrounding the blocks. The
   * default value is <tt>maxcur = max(18, 4*maxr)</tt> where <tt>maxr</tt>
   * is an instance variable of the Tracking superclass.
   */
  public int maxj;
  /** maximum amount of memory required to store the integration lines. The
   * default value is <tt>maxint = 10000</tt>.
   */
  public int maxz;
  /** type of multicell approximation. This variable is chosen among the
   * following values:
   * <dl>
   * <dt> <tt>"ROTH"</tt> <dd> the isotropic DP0 components of the inter-cell
   * current is used with the incoming current being averaged over all the
   * faces surrounding a cell. The global collision matrix is calculated in an
   * annular model. Only used when 2D assembly of cells are considered.
   * <dt> <tt>"ROT+"</tt> <dd> the isotropic DP0 components of the inter-cell
   * current is used. The global collision matrix is calculated in an annular
   * model. Only used when 2D assembly of cells are considered.
   * <dt> <tt>"DP00"</tt> <dd> the isotropic DP0 components of the inter-cell
   * current is used. The global collision matrix are computed explicitly.
   * Only used when 2D assembly of cells are considered.
   * <dt> <tt>"DP01"</tt> <dd> the linearly anisotropic DP1 components of the
   * inter-cell current are used. This hypothesis implies 12 currents per
   * cell in a cartesian geometry and 18 currents per cell for an hexagonal
   * geometry. Linearly anisotropic reflection is used. Only used when 2D
   * assembly of cells are considered.
   * </dl>
   */
  public String type;
  /** type of cylinderization for approximations <tt>"ROTH"</tt> and
   * <tt>"ROT+"</tt>. This variable is chosen among the following values:
   * <dl>
   * <dt> <tt>"WIGN"</tt> <dd> use of a <i>Wigner</i> cylinderization which
   * preserves the volume of the external crown. Note that an assembly of
   * rectangular cells having unequal volumes cannot use a <i>Wigner</i>
   * cylinderization.
   * <dt> <tt>"ASKE"</tt> <dd> use of an <i>Askew</i> cylinderization which
   * preserves both the external surface of the cells and the material balance
   * of the external crown (by a modification of its concentration). Note
   * that an assembly of rectangular cells having unequal volumes can use
   * an <i>Askew</i> cylinderization.
   * <dt> <tt>"SANC"</tt> <dd> use of a <i>Sanchez</i> cylinderization. This
   * model uses a <i>Wigner</i> cylinderization for computing the collision
   * PIJ and leakage PIS probabilities. However, the reciprocity and
   * conservation relations used to compute the incoming PSJ and transmission
   * PSS probabilities are defined in the rectangular cell (with the exact
   * surface). Note that an assembly of rectangular cells having unequal
   * volumes can use a <i>Sanchez</i> cylinderization. This is the default
   * option.
   * </dl>
   */
  public String cylinder;
  /** number of basis points for the angular integration of the blocks in a
   * one-dimensional geometry. This parameter is not used for <tt>CAR1D</tt>
   * geometries. If a Gauss-Legendre or Gauss-Jacobi quadrature is used,
   * the values of <tt>iqua1</tt> allowed are: 1 to 20, 24, 28, 32 or 64. The
   * default value is <tt>iqua1 = 5</tt>.
   */
  public int qua1;
  /** two-component array used to define the two-dimensional integration
   * parameters.
   * <P> <tt>qua2[0]</tt> is the number of basis points for the angular
   * integration of the blocks in a two-dimensional geometry appearing during
   * assembly calculations. If a Gauss-Legendre or Gauss-Jacobi formula is
   * used the values allowed for <tt>qua2[0]</tt> are: 1 to 20, 24, 28, 32 or
   * 64. The default value is <tt>qua2[0] = 6</tt> and represents the number
   * of angles in (0, pi/4) for Cartesian geometries and  (0, pi/6) for
   * hexagonal geometries.
   * <P> <tt>qua2[1]</tt> is the number of basis points for the spatial
   * integration of the blocks in a two-dimensional geometry appearing during
   * assembly calculations. The default value is <tt>qua2[1] = 3</tt>.
   */
  public int[] qua2;
  /** number of basis point for the numerical integration of each
   * micro-structure in cases involving double heterogeneity (Bihet).
   */
  public int quab;
  /** type of 2D quadrature used to integrate the collision probabilities.
   * This variable is chosen among the following values:
   * <dl>
   * <dt> <tt>"EQW"</tt> <dd> use of equal-weight quadrature.
   * <dt> <tt>"GAUS"</tt> <dd> use of the Gauss-Legendre or the Gauss-Jacobi
   * quadrature. This is the default option.
   * </dl>
   */
  public String weights;
  /** set to <tt>true</tt> to specify that all the integration lines are
   * to be printed. This option should only be used when absolutely necessary
   * because it generates a rather large amount of output. Only used when 2D
   * assembly of cells are considered.
   */
  public boolean lign = false;
  /** set to <tt>true</tt> to specify that square cells are to be treated as
   * if they were rectangular cells, with the inherent loss in performance
   * that this entails. This option is of purely academic interest.
   */
  public boolean rect = false;
  /**
   * Use this constructor to create a new Sybil object.
   * @param name user-defined name of the Sybil object
   */
  public Sybil(String name) {
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
  }
  /** load the embedded lcmObj structure corresponding to this Sybil tracking.
   * @param a 1D or 2D geometry to analyse.
   */
  public void exec(Gigogne a) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.exec();
  }
  protected void makeText() {
    this.operator="SYBILT: ";
    super.write("EDIT", super.edit);
    super.write("\nTITLE '"); super.write(this.title); super.write("'");
    if (this.maxr != 0) super.write("\nMAXR", this.maxr);
    if (this.maxj != 0) super.write("\nMAXJ", this.maxj);
    if (this.maxj != 0) super.write("\nMAXZ", this.maxz);
    if (this.type != null) super.write(this.type);
    if (this.cylinder != null) super.write(this.cylinder);
    if (this.qua1 != 0) super.write("\nQUA1", this.qua1);
    if (this.qua2 != null) super.write("\nQUA2", this.qua2);
    if (this.quab != 0) super.write("\nQUAB", this.quab);
    if (this.weights != null) super.write(this.weights);
    if (this.lign) super.write("\nLIGN");
    if (this.rect) super.write("\nRECT");
    if (this.halt) super.write("\nHALT");
    super.write(";");
  }
}
