package jargon;

import java.util.*;
import java.rmi.*;
import java.io.*;
/**
 * This class is used to create and manage a pool of threads.
 * One and only one deamon thread of execution is started on each
 * server. Each submitted job is assigned to a free thread of
 * execution. If the maximum pool size is reached, jobs wait for
 * a free thread.
 * <p>
 * Once the ThreadPool object <tt>myPool</tt> is created, any
 * task <tt>myTask</tt> implementing the Runnable interface can be
 * submitted to the pool by using the following command:
 * <PRE>
 * myPool.run(myTask);
 * </PRE>
 */
public class ThreadPool implements Serializable {
 private static final long serialVersionUID = 600L;
 static {
  if (System.getSecurityManager() == null) {
   System.setSecurityManager(new RMISecurityManager());
  }
 }
 private LinkedList<Runnable> tasks;
 private int numberOfJobs = 0; // the number of jobs in the pool
 /** Use this constructor to create a ThreadPool object with a
  * predefined number of servers.
  * @param servers IP identification of the servers.
  */
 public ThreadPool(String[] servers) {
  tasks = new LinkedList<Runnable>();
  for (int i=0; i<servers.length; i++) {
   TaskThread thread = new TaskThread(this);
   thread.server = servers[i];
   thread.setDaemon(true);
   thread.start();
  }
 }
 /** Use this constructor to create a ThreadPool object without
  * setting the servers.
  */
 public ThreadPool() {
  tasks = new LinkedList<Runnable>();
 }
 /** return the number of jobs in the pool. This value can be
  * subsequently used by the <tt>join(iset)</tt> method.
  */
 public int getNumberOfJobs() {
  return this.numberOfJobs;
 }
 /** add a new server to the ThreadPool object.
  * @param server IP identification of the new server.
  */
 public void add(String server) {
  TaskThread thread = new TaskThread(this);
  thread.server = server;
  thread.setDaemon(true);
  thread.start();
 }

 class TaskThread extends Thread implements Serializable {
  private static final long serialVersionUID = 601L;
  private ThreadPool pool;
  private String server;

  public TaskThread(ThreadPool thePool) {
   pool = thePool;
  }

  public void run() {
   while (true) {
    // blocks until job
    Procedure job = (Procedure)pool.getNext();
    try {
     job.thread = this;
     String serverURL = "rmi://" + this.server + "/ProcServer";
     System.out.println("Send request to server=" + this.server);
     System.out.flush();
     ProcServerIntf procServerIntf = (ProcServerIntf)Naming.lookup(serverURL);
     System.out.println("ProcServer lookup in ThreadPool");
     Thread.sleep(500);
     job.operatorOut = procServerIntf.executeProc(job);
     job.thread = null;
     job.notifyObserver();
     this.pool.numberOfJobs--;
    } catch (Exception e) {
     System.out.println("Exception catched by ThreadPool: " + e);
     throw new RuntimeException(e.toString());
    }
   }
  }
 }
 /** add a new job to the pool.
  * @param task job implementing the Runnable interface.
  */
 public void run(Runnable task) {
  synchronized (tasks) {
   this.numberOfJobs++;
   tasks.addLast(task);
   tasks.notify();
  }
 }
 private Runnable getNext() {
  Runnable returnVal = null;
  synchronized (tasks) {
   while (tasks.isEmpty()) {
    try {
     tasks.wait();
    } catch (InterruptedException e) {
     System.err.println("Interrupted: " + e);
    }
   }
   returnVal = tasks.removeFirst();
  }
  return returnVal;
 }
 /** wait until all jobs have been executed.
  */
 public void join() {
  this.join(0);
 }
 /** wait until <tt>iset</tt> jobs are remaining in the pool.
  */
 public void join(int iset) {
  while (this.numberOfJobs > iset) {
   try {
    Thread.sleep(4000);
   } catch(Exception e) {
    System.err.println("Exception catched by ThreadPool: " + e);
   }
  }
 }
}
