package jargon;

/**
 * The Trivac tracking operator produces a Jlcm structure containing the
 * energy-independent numerotation information required to solve the
 * 3D diffusion equation with Trivac. A Trivac-type tracking data structure
 * is holding the information related to the ADI partitionning of the
 * system matrices in 1D, 2D or 3D.
 * <P>
 * <B>Author:</B> Alain Hebert, Ecole Polytechnique de Montreal (2002)
 *
 * @author Alain Hebert
 */
public class Trivac extends Tracking {
  static {
    System.out.println("Trivac class loaded");
  }
  static String fr = "definition d'un tracking trivac";
  static String en = "definition of a trivac tracking";
  private static final long serialVersionUID = 75L;
  private String trivacType;
  private int[] order;
  private int isplh = -1;
  /** width of a vectorial register. <tt>iseg</tt> is generally a multiple of 64.
   * By default, <tt>iseg = 0</tt> and the supervectorization algorithm is not used.
   */
  public int iseg = 0;
  /** index used to control the printing in supervectorization subroutines.
   * <tt>=0</tt> for no print; <tt>=1</tt> for minimum printing (default value).
   * Larger values produce increasing amounts of output.
   */
  public int impv; 
  /** 
   * Use this constructor to create a new Trivac object in hexagonal geometry.
   * Only first order (linear) finite differences are currently available.
   * @param name user-defined name of the Trivac object
   * @param trivacType type of finite element approach. This variable is chosen
   * among the following values:
   * <ul><dl>
   * <dt> <tt>"PRIM"</tt> <dd> mesh-corner filite differences.
   * <dt> <tt>"MCFD"</tt> <dd> mesh-centered finite differences.
   * </dl></ul>
   * @param isplh type of hexagonal mesh-splitting. The values permitted are:
   * <ul><dl>
   * <dt> 1 <dd> full hexagons
   * <dt> 2 <dd> splitting each hexagon into 6 triangles
   * <dt> 3 <dd> splitting each hexagon into 24 triangles
   * <dt> 5 <dd> splitting each hexagon into 96 triangles
   * </dl></ul>
   */
  public Trivac(String name, String trivacType, int isplh) {
    if ((!trivacType.equals("PRIM")) & (!trivacType.equals("DUAL"))) {
      throw new RuntimeException("invalid type (" + trivacType + ")");
    }
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
    this.trivacType = trivacType;
    this.order = new int[]{1};
    this.isplh = isplh;
  }
  /** 
   * Use this constructor to create a new Trivac object in Cartesian geometry.
   * @param name user-defined name of the Trivac object
   * @param trivacType type of finite element approach. This variable is chosen
   * among the following values:
   * <ul><dl>
   * <dt> <tt>"PRIM"</tt> <dd> variational collocation method.
   * <dt> <tt>"DUAL"</tt> <dd> mixed-dual finite element discretization. This
   * option is not available for hexagonal geometries.
   * <dt> <tt>"MCFD"</tt> <dd> variable-order mesh-centered finite differences
   * (nodal collocation method). This option is not available for hexagonal
   * geometries. The mesh centered finite difference approximation is the
   * default option and is generally set using <tt>trivacType = "MCFD"</tt> and
   * <tt>order[0] = 1</tt>. The <tt>"MCFD"</tt> approximations are numerically
   * equivalent to the <tt>"DUAL"</tt> approximations with <tt>order[1] = 2</tt>.
   * </dl></ul>
   * @param order one- or two-component array containing the discretization type.
   * <tt>order[0]</tt> is the order of the finite element representation.
   * Its value is chosen among the following values:
   * <ul><dl>
   * <dt> 1 <dd> linear polynomials or finite differences (default value)
   * <dt> 2 <dd> parabolic polynomials
   * <dt> 3 <dd> cubic polynomials
   * </dl></ul>
   * <tt>order[1]</tt> is the type of quadrature used to integrate the mass
   * matrices in case where <tt>trivacType = "DUAL"</tt>. Its value is chosen among
   * the following values:
   * <ul><dl>
   * <dt> 1 <dd> analytical integration
   * <dt> 2 <dd> Gauss-Lobatto quadrature (default value)
   * <dt> 3 <dd> Gauss-Legendre quadrature
   * </dl></ul>
   * The analytical integration corresponds to classical finite elements;
   * the Gauss-Lobatto quadrature corresponds to a variational or nodal
   * type collocation and the Gauss-Legendre quadrature corresponds to
   * superconvergent finite elements.
   * </dl></ul>
   */
  public Trivac(String name, String trivacType, int[] order) {
    if ((!trivacType.equals("PRIM")) & (!trivacType.equals("DUAL")) &
    (!trivacType.equals("MCFD"))) {
      throw new RuntimeException("invalid type (" + trivacType + ")");
    }
    super.lcmObj = new Jlcm("LCM", name);
    super.name = name;
    this.trivacType = trivacType;
    this.order = order;
    this.isplh = -1;
  }
  /** load the embedded lcmObj structure corresponding to this Trivac tracking.
   * @param a 1D or 2D Cartesian or hexagonal geometry to analyse.
   */
  public void exec(Gigogne a) throws JargonException {
    super.resetJlcm();
    super.outputJlcm(this.lcmObj);
    super.inputJlcm(a.lcmObj);
    super.exec();
  }
  protected void makeText() {
    this.operator="TRIVAT: ";
    super.write("EDIT", super.edit);
    if (super.title != null) {
      super.write("\nTITLE '"); super.write(this.title); super.write("'");
    }
    if (super.maxr != 0) super.write("\nMAXR", this.maxr);
    super.write("\n", this.trivacType);
    if (this.trivacType.equals("DUAL")) {
      super.write(this.order[0]);
      super.write(this.order[1]);
    } else {
      super.write(this.order[0]);
    }
    if (this.isplh != -1) super.write(this.isplh);
    if (this.iseg != 0) {
      super.write("\nVECT", this.iseg);
      if (this.impv != 0) super.write("\nPRTV", this.impv);
    }
    super.write(";");
  }
}
