package texmaker;

import com.sun.javadoc.*;
import java.io.*;
import java.util.*;
import java.awt.*;

/**
 * This class generates the Latex code of the documentation of a class.
 * <P>
 * Each class or interface is a section of the document.
 * @author F. Serin
 * @version 1.2, June 6, 2000.
 */
class TexClass extends TexDoc {
/**
 * The package of this containing this {@link #cls class}.
 */
PackageDoc pkg;
/**
 * The class documented by this instance.
 */
ClassDoc cls;

/**
 * Create the section dedicated to this class.
 * @param cls commented class in this section
 * @param pkg the package containing this class cls
 * @param outFile the file where the section is writen.
 */
TexClass(ClassDoc cls,PackageDoc pkg,PrintWriter outFile){
	this.pkg=pkg;
	this.cls=cls;
	this.outFile=outFile;
	createChapter();
	} // end of this class constructor

/**
 * Create the section head.
 */
private void createChapter(){
	printIntro();

	Vector fields=new Vector();
	Vector staticFields=new Vector();
	Vector constructors=new Vector();
	Vector abstractMethods=new Vector();
	Vector abstractStaticMethods=new Vector();
	Vector staticMethods=new Vector();
	Vector methods=new Vector();
	FieldDoc[] theFields=cls.fields();
	ConstructorDoc[] theConstructors=cls.constructors();
	MethodDoc[] theMethods=cls.methods();
	for (int i=0; i<theFields.length; ++i){
		if (theFields[i].isStatic()) staticFields.add(theFields[i]);
		else fields.add(theFields[i]);
		} // end of the loop

	for (int i=0; i<theConstructors.length; ++i){
		constructors.add(theConstructors[i]);
		} // end of the loop

	for (int i=0; i<theMethods.length; ++i){
		if (theMethods[i].isStatic()) {
			if (theMethods[i].isAbstract())
				abstractStaticMethods.add(theMethods[i]);
			else
				staticMethods.add(theMethods[i]);
			}
		else {
			if (theMethods[i].isAbstract())
				abstractMethods.add(theMethods[i]);
			else
				methods.add(theMethods[i]);
			}
		} // end of the loop

	printFields("Static field",staticFields);
	printFields("Field",fields);
	printConstructors("Constructor",constructors);
	printMethods("Abstract static method",abstractStaticMethods);
	printMethods("Abstract method",abstractMethods);
	printMethods("Static method",staticMethods);
	printMethods("Method",methods);

	} // end of createDocument method

/**
 * Print the introduction to the class.
 */
private void printIntro(){
	StringBuffer chap=new StringBuffer("\\section[");
	chap.append(getNature());
	chap.append(cls.name());
	chap.append("]{{\\normalsize ");
	chap.append(pkg.name());
	chap.append("}\\\\");
	chap.append(getNature());
	chap.append(cls.name());
	chap.append(" \\label{");
	chap.append(pkg.name());
	chap.append(".");
	chap.append(cls.name());
	chap.append("}}");
	print(chap.toString());

	StringBuffer mess=new StringBuffer(cls.name());
	ClassDoc exts=cls.superclass();
	ClassDoc[] impl=cls.interfaces();
	printTitle(2,"Presentation");
	mess.append(" is ");
	mess.append(visibility(cls));
	mess.append(".");
	print(mess.toString());

	if (exts != null){
		mess=new StringBuffer("It extends {\\bf ");
		mess.append(exts.qualifiedName());
		mess.append("}.");
		print(mess.toString());
		}

	if (impl.length>0){
		if (cls.isInterface())
			mess=new StringBuffer("It extends {\\bf ");
		else
			mess=new StringBuffer("It implements {\\bf ");
		mess.append(impl[0].qualifiedName());
		for (int i=1; i<impl.length; ++i){
			mess.append(", ");
			mess.append(impl[i].qualifiedName());
			} // end of the loop
		mess.append("}.");
		print(mess.toString());
		} // end of if

	printComment(cls);

	Tag[] authorTag=cls.tags("author");
	Tag[] sinceTag=cls.tags("since");
	Tag[] versionTag=cls.tags("version");

	//printTitle(3,"Informations");
	printAuthor(authorTag);
	printVersion(versionTag);
	printSince(sinceTag);

	} // end of method

/**
 * Print the members of the class.
 * <P>
 * There are seven kinds of members : static fields, fields, 
 * constructors, static methods, abstract methods, 
 * static abstract methods, and methods.
 * @param title one kind of member
 * @param members contains FieldDoc objects
 */
private void printFields(String title,Vector members){
if (notEmptySection(title,members)){
	StringBuffer fullName;
	StringBuffer label, secTitl;
	Enumeration enum=members.elements();
	FieldDoc elt;
	while (enum.hasMoreElements()){
		elt=(FieldDoc)enum.nextElement();
		label=new StringBuffer(pkg.name());
		label.append(".");
		label.append(cls.name());
		label.append(".");
		label.append(elt.name());

		secTitl=new StringBuffer(elt.name());
		//print("\\rule{0.7\\textwidth}{0.3mm}");
                print("\\vskip 0.2cm");
		printTitle(2,secTitl.toString(),label.toString());

		print(fullName(elt));
		printComment(elt);
		printSince(elt.tags("since"));
		printSeeAlso(4,elt);
		} // end of the loop
	} // end of if
	} // end of printFields method

/**
 * Print the members of the class.
 * <P>
 * There are seven kinds of members : static fields, fields, 
 * constructors, static methods, abstract methods, 
 * static abstract methods, and methods.
 * @param title one kind of member
 * @param members contains MemberDoc objects
 */
private void printConstructors(String title,Vector members){
if (notEmptySection(title,members)){
	StringBuffer label, secTitl;
	Enumeration enum=members.elements();
	ConstructorDoc elt;
	while (enum.hasMoreElements()){
		elt=(ConstructorDoc)enum.nextElement();
		label=new StringBuffer(pkg.name());
		label.append(".");
		label.append(cls.name());
		label.append(".");
		label.append(elt.name());
		label.append(elt.signature());

		secTitl=new StringBuffer(elt.name());
		addShortSignature(secTitl,elt);
		//print("\\rule{0.7\\textwidth}{0.3mm}");
                print("\\vskip 0.2cm");
		printTitle(2,secTitl.toString(),label.toString());

		print(fullName(elt));
		printComment(elt);
		printParam(4,elt);
		printSince(elt.tags("since"));
		printSeeAlso(4,elt);
		} // end of the loop
	} // end of if
	} // end of printConstructors method
/**
 * Print the members of the class.
 * <P>
 * There are seven kinds of members : static fields, fields, 
 * constructors, static methods, abstract methods, 
 * static abstract methods, and methods.
 * @param title one kind of member
 * @param members contains MemberDoc objects
 */
private void printMethods(String title,Vector members){
if (notEmptySection(title,members)){
	StringBuffer fullName;
	StringBuffer label, secTitl;
	Enumeration enum=members.elements();
	MethodDoc elt;
	while (enum.hasMoreElements()){
		elt=(MethodDoc)enum.nextElement();
		label=new StringBuffer(pkg.name());
		label.append(".");
		label.append(cls.name());
		label.append(".");
		label.append(elt.name());
		label.append(elt.signature());

		secTitl=new StringBuffer(elt.name());
		addShortSignature(secTitl,elt);
		//print("\\rule{0.7\\textwidth}{0.3mm}");
                print("\\vskip 0.2cm");
		printTitle(2,secTitl.toString(),label.toString());

		print(fullName(elt));
		printComment(elt);
		printParam(4,elt);
		printReturn(elt.returnType(),elt.tags("return"));
		printSince(elt.tags("since"));
		printSeeAlso(4,elt);
		} // end of the loop
	} // end of if
	} // end of printMethods method

/**
 * Say if the section is not empty.
 * <P>
 * If section is not empty, print the title 
 * (with "s" if there is more than one element in the vector)
 * and sort its elements.
 * @param v set of members
 * @return true is the vector v is not empty.
 */
private boolean notEmptySection(String title,Vector v){
	if (v.size()==0) return false;

	print("\\vspace{3ex}");
	print("\\fbox{");
	print("\\begin{minipage}{0.8\\textwidth}");
	if (v.size()==1)
		print("\\section*{{\\sf "+title+"}}");
	else {
		print("\\section*{{\\sf "+title+"s}}");
		Collections.sort(v);
		}
	print("\\end{minipage}");
	print("} \\par");
	return true;
	} // end of notEmptySection method

/**
 * Print the seeAlso tags
 * (with Tex ref if the reference is included in this document).
 * @param i level of the title.
 * @param doc the document.
 */
public void printSeeAlso(int i,Doc doc){
	SeeTag[] tags=doc.seeTags();
	if (tags.length>0) {
		printTitle(i,"See also");
		for (int j=0; j<tags.length; ++j)
			print(text(tags[j]));
		} // fi
	} //end of method

/**
 * Print the explanations given in comments.
 * @param i level of the title.
 * @param doc commented element.
 */
public void printParam(int i,ExecutableMemberDoc doc){
	ParamTag[] tags=doc.paramTags();
	if (tags.length>0) {
		printTitle(i,"Parameters");
		for (int j=0; j<tags.length; ++j)
			print(text(tags[j]));
	} // fi
	} // end of method

/**
 * Print the comments after the version tag.
 * @param tags version number and date.
 */
public void printVersion(Tag[] tags){
	if ((TexDoclet.version) && (tags.length>0))
		print("{\\bf Version}~: "+text(tags[0]));
	} // end of method

/**
 * Print informations about return value.
 */
public void printReturn(Type returnType, Tag[] returnTag){
	StringBuffer mess;
	String theName=returnType.typeName();
	if (theName.equals("void"))
		mess=new StringBuffer("{\\it No returned value}. ");
	else {
		mess=new StringBuffer("Returned value of type {\\bf ");
		mess.append(theName);
		mess.append(returnType.dimension());
		mess.append("}. ");
		}
	for (int i=0; i<returnTag.length; ++i){
		mess.append(text(returnTag[i]));
		mess.append(". ");
		}
	print(mess.toString());
	}
/**
 * Print informations about the authors.
 */
public void printAuthor(Tag[] authorTag){
	if (TexDoclet.author) 
	if (authorTag.length>0){
		StringBuffer mess=new StringBuffer("{\\bf Author");
		if (authorTag.length>1) mess.append("s");
		mess.append("}~: ");
		mess.append(authorTag[0].text());
		for (int i=1; i<authorTag.length; ++i){
			mess.append(", ");
			mess.append(authorTag[i].text());
			}
		mess.append(".");
		print(mess.toString());
		}
	else
		print("{\\it Anonymous author}.");
	} // end of printAuthor method

/**
 * Give the visibility of a member.
 * @return public,protected, private or package private.
 */
public String visibility(ProgramElementDoc doc){
	if (doc.isPublic()) return ("public");
	else
	if (doc.isProtected()) return ("protected");
	else
	if (doc.isPrivate()) return ("private");
	else
	return ("package private");
	} // end of method

/**
 * Return the nature of the class.
 * @return Interface, Abstract Class, Inner Class or Class.
 * @since 1.0.1
 */
String getNature(){
	if (cls.isInterface())
		return ("Interface ");
	else
	if (cls.isAbstract())
		return ("Abstract Class ");
	else
	if (cls.containingClass()==null)
		return ("Class ");
	else
		return("Inner Class ");
	} // end of getNature method

/**
 * Example : public java.lang.String word.
 */
String fullName(FieldDoc elt){
	StringBuffer fullName=new StringBuffer("{\\tt ");
	fullName.append(elt.type());
	fullName.append(" ");
	fullName.append(elt.name());
	fullName.append("}");
	return (fullName.toString());
	}

/**
 * Example : public MyClass (java.lang.String word).
 */
String fullName(ConstructorDoc elt){
	StringBuffer fullName=new StringBuffer("{\\tt ");
	fullName.append(visibility(elt));
	fullName.append(" ");
	fullName.append(elt.name());
	fullName.append(" \\normalsize{");
	fullName.append(signature(elt));
	fullName.append("}}");
	return (fullName.toString());
	}

/**
 * Example : public void myMethod (java.lang.String word).
 */
String fullName(MethodDoc elt){
	StringBuffer fullName=new StringBuffer("{\\tt ");
	fullName.append(visibility(elt));
	fullName.append(" ");
	fullName.append((elt.returnType()).typeName());
	fullName.append(" ");
	fullName.append(elt.name());
	fullName.append(" \\normalsize{");
	fullName.append(signature(elt));
	fullName.append("}}");
	return (fullName.toString());
	}


} // end of TexClass class declaration
