package texmaker;

import com.sun.javadoc.*;
import java.io.*;
import java.util.*;
import java.awt.*;

/**
 * This class generates the Latex code of the documentation of a class.
 * <P>
 * Each class or interface is a chapter of the document.
 * @author F. Serin
 * @version 1.2, June 16, 2000.
 */
abstract class TexDoc {
/**
 * The Tex file where the parts or chapters are writen.
 * @see TexPackage
 * @see TexClass
 */
protected PrintWriter outFile;

/**
 * Print the comments of this element.
 * @param i level of the title.
 * @param doc the commented document.
 */
public void printComment(Doc doc){
	Tag[] tags=doc.inlineTags();
	if (tags.length>0) {
		print(text(tags));
		}
	else
		print("{\\it No comment}.");
	} // end of PrintComment method


/**
 * Print the comments after the since tag.
 * @param tags information about the last release of this element.
 */
public void printSince(Tag[] tags){
	if ((TexDoclet.version) && (tags.length>0))
		print("{\\bf Since} "+text(tags[0]));
	} // end of method

/**
 * Translate tags into text.
 */
public String text(Tag[] tags){
	StringBuffer str=new StringBuffer(50);
	for (int i=0; i<tags.length; ++i){
		if ((tags[i].kind()).equals("@see"))
		str.append(text((SeeTag)tags[i]));
		else
		str.append(text(tags[i]));
		}
	return (str.toString());
	} // end of method

/**
 * Expunge a string of its HTML tags.
 */
public String text(Tag tag){
	return (Java2Latex.fromHTML(tag.text()));
	} // end of printText method

/**
 * Add reference as foot note. This method is an
 * extension of {@link #text(Tag) text method}.
 */
public String text(SeeTag tag){
	StringBuffer label=new StringBuffer(Java2Latex.fromHTML(tag.label()));
	String s;
	MemberDoc mem=tag.referencedMember();
	ClassDoc clas=tag.referencedClass();
	PackageDoc pack=tag.referencedPackage();

	if (mem==null)
		if (clas==null)
			if (pack==null) {
				label.append(" {\\it ");
				label.append(tag.text());
				label.append("}");
				return (label.toString());
				}
			else
				s=reference(pack);
		else {
			s=reference(clas);
			pack=clas.containingPackage();
			}
	else {
		s=reference(mem);
		pack=mem.containingPackage();
		}

	if (label.length()<1) {
		label.append("{\\it ");
		label.append(s);
		if (TexDoclet.contains(pack))
			label.append("}\\footnote{");
		}
	else {
		label.append("\\footnote{{\\it ");
		label.append(s);
		label.append("} ");
		}
	if (TexDoclet.contains(pack)){
		label.append("see \\S{} \\ref{");
		label.append(s);
		label.append("}, page \\pageref{");
		label.append(s);
		label.append("}");
		}
	label.append("}");

	return (label.toString());
	}

public String shortSignature(ExecutableMemberDoc elt){
	StringBuffer ret=new StringBuffer("(");
	Parameter[] param=elt.parameters();
	if (param.length==1){
		ret.append((param[0].type()).typeName());
		ret.append((param[0].type()).dimension());
		}
	else
	if (param.length>1){
		ret.append((param[0].type()).typeName());
		ret.append((param[0].type()).dimension());
		for (int i=1; i<param.length; ++i){
			ret.append(", ");
			ret.append((param[i].type()).typeName());
			ret.append((param[i].type()).dimension());
			}
		}
	ret.append(")");
	return (ret.toString());
	} // end of method

/**
 * Return a visible signature 'parameters) for constructors and methods.
 * @since 1.0.1: erase the first parameter of inner class' constructor.
 */
public String signature(ExecutableMemberDoc elt){
	int i=0;
	if (elt.isConstructor() &&
(elt.containingClass()).containingClass()!=null) i=1;
	StringBuffer ret=new StringBuffer("(");
	Parameter[] param=elt.parameters();
	if (param.length==(i+1)){
		ret.append((param[i].type()).typeName());
		ret.append((param[i].type()).dimension());
		ret.append(" ");
		ret.append(param[i].name());
		}
	else
	if (param.length>(1+i)){
		ret.append((param[i].type()).typeName());
		ret.append((param[i].type()).dimension());
		ret.append(" ");
		ret.append(param[i].name());
		for (int j=(1+i); j<param.length; ++j){
			ret.append(", ");
			ret.append((param[j].type()).typeName());
			ret.append((param[j].type()).dimension());
			ret.append(" ");
			ret.append(param[j].name());
			}
		}
	ret.append(")");
	return (ret.toString());
	} // end of signature method

/**
 * Add the signature of the ExecutableMemberDoc to
 * the sTringBuffer.
 */
void addShortSignature(StringBuffer secTitl,ExecutableMemberDoc elt){
	secTitl.append(" \\small{");
	secTitl.append(shortSignature(elt));
	secTitl.append("}");
	}

/**
 * Return the reference of a MemberDoc.
 * <P>
 * For example texdoclet.TexClass.print(String) for a method.
 */
String reference(MemberDoc mem){
	StringBuffer ref=new StringBuffer(mem.qualifiedName());

	if (mem.isMethod())
			ref.append(((MethodDoc)mem).signature());
	else
	if (mem.isConstructor())
			ref.append(((ConstructorDoc)mem).signature());
	return (ref.toString());
	}

/**
 * Return the reference of a ClassDoc.
 * <P>
 * For example java.lang.String for a class, 
 */
String reference(ClassDoc mem){
	return (mem.qualifiedName());
	}

/**
 * Return the reference of a PackageDoc.
 * <P>
 * For example java.lang.
 */
String reference(PackageDoc mem){
	return (mem.name());
	}


/**
 * Method to simplify the code writing.
 * <P>
 * Do {@link TexClass#outFile outFile}.println(s).
 * with control of code.
 * @see Java2Latex#fromCode(String)
 * @param s line to print in the Tex file.
 */
public void print(String s){
	outFile.print(Java2Latex.fromCode(s));
	outFile.println("\\par");
	} // end of print method

/**
 * Method to simplify the code writing.
 * @param i level of the title (from 0 to 6).
 * @param s text of the title.
 * @see #theTitle(int)
 */
public void printTitle(int i,String s){
	print(theTitle(i)+s+"}");
	}

/**
 * Method to simplify the code writing.
 * <P>
 * Do masterDoc.println(s).
 * @param i level of the title (from 0 to 6).
 * @param s text of the title.
 * @param lab label used to reference this title
 */
public void printTitle(int i,String s, String lab){
	print(theTitle(i)+s+"\\label{"+lab+"}}");
	}

/**
 * Print a title in the document.
 * @param i level of the title. From 0 for <I>part</I> 
 * to 6 for <I>subparagraph</I>.
 */
public static String theTitle(int i){
	switch(i) {
	case 1 : return ("\\section{");
	case 2 : return ("\\subsection{");
	case 3 : return ("\\subsubsection{");
	case 4 : return ("\\paragraph{");
	case 5 : return ("\\subparagraph{");
	default : return ("{\\bf ");
	} // end of switch
	} // end of theTitle method

} // end of TexDoc class declaration
